//****************************************************************************************
//****************************************************************************************
//  Project Name:		TCP/IP DRIVER
//						ARP (ADDRESS RESOLUTION PROTOCOL) C CODE FILE
//  Copyright:			EMBEDDED-CODE.COM
//
//  Licenced To:        NONE
//  Licence Number:     NONE
//
//  IMPORTANT: These files are copyright of embedded-code.com and are subject to a licence
//  agreement.  All rights reserved.  Unauthorised use, reproduction or distribution of
//  these files may lead to prosecution.
//  Any use in violation of the licence restrictions may subject the user to criminal
//  sanctions under applicable laws, as well as to civil liability for the breach of the
//  terms and conditions of the license.
//  This software is provided in an "as is" condition. No warranties, whether express,
//  implied or statutory, including, but not limited to, implied warranties of
//  merchantability and fitness for a particular purpose apply to this software. 
//  Embedded-code.com shall not be liable in any circumstances for special, incidental
//  or consequential damages, for any reason whatsoever.
//  Please see www.embedded-code.com/licence.php for full details.
//  This file contains hidden markers to identify it uniquely to the licence number it was
//  purchased under.
//  DO NOT REMOVE THIS NOTICE - IT IS A REQUIREMENT OF THE LICENCE AGREEMENT.
//****************************************************************************************
//****************************************************************************************


#include "main.h"
#define ARP_C
#include "eth-arp.h"
#undef ARP_C

#include "eth-main.h"
#include "eth-nic.h"



//************************************
//************************************
//********** INITIALISE ARP **********
//************************************
//************************************
//Called from the tcp_ip_initialise function
void arp_initialise (void)
{

	sm_arp = SM_ARP_IDLE;

	arp_last_received_response.mac_address.v[0] = 0;
	arp_last_received_response.mac_address.v[1] = 0;
	arp_last_received_response.mac_address.v[2] = 0;
	arp_last_received_response.mac_address.v[3] = 0;
	##########################.###########.#[#] = #;
	##########################.###########.#[#] = #;
	##########################.##########.### = ##########;
}



//********************************************
//********************************************
//********** ARP RESOLVE IP ADDRESS **********
//********************************************
//********************************************
//Returns 1 if request was sent, 0 if request can't be sent right now
BYTE arp_resolve_ip_address(IP_ADDR *ip_address_to_resolve)
{
	DEVICE_INFO remote_device_info;


	//Setup ready to tx a packet
	if (nic_setup_tx() == 0)
		return(0);		//nic is not currently ready for a new tx packet

	//Clear the last resolved arp ip address
	arp_last_received_response.ip_address.val = 0x00000000;

	//Setup packet to tx
	remote_device_info.mac_address.v[0] = 0xff;
	##################.###########.#[#] = ####;
	##################.###########.#[#] = ####;
	##################.###########.#[#] = ####;
	##################.###########.#[#] = ####;
	##################.###########.#[#] = ####;
	
	####################.########## = *#####################;

	//Send the packet
	##############(&####################, ####################);
	
	######(#);
}



//*********************************************
//*********************************************
//********** CHECK ARP RESOLVE STATE **********
//*********************************************
//*********************************************
//ip_address_being_resolved = The IP address that is waiting to be resolved
//resolved_mac_address = The mac address for the resolved IP address will be stored to here
//Returns 1 if resolved, 0 if not yet resolved
BYTE arp_is_resolve_complete (IP_ADDR *ip_address_being_resolved, MAC_ADDR *resolved_mac_address)
{
	if ((arp_last_received_response.ip_address.val == ip_address_being_resolved->val) ||	//If IP address matches last returned ARP response then arp is resolved
         (arp_last_received_response.ip_address.val == our_gateway_ip_address.val))			//or if the last returned IP address is the gateway IP address then arp
     {    																					//is resolved (this would mean the target IP is on a different subnet so
         																					//access to it should go via the gateway)
        resolved_mac_address->v[0] = arp_last_received_response.mac_address.v[0];
        resolved_mac_address->v[1] = arp_last_received_response.mac_address.v[1];
        resolved_mac_address->v[2] = arp_last_received_response.mac_address.v[2];
        #####################>#[#] = ##########################.###########.#[#];
        #####################>#[#] = ##########################.###########.#[#];
        #####################>#[#] = ##########################.###########.#[#];
        ######(#);
    }
    ######(#);
}



//*******************************************
//*******************************************
//********** PROCESS ARP RX PACKET **********
//*******************************************
//*******************************************
//Called from the recevied packet ethernet stack state machine
//Returns 1 if ARP was processed, 0 if couldn't do it this time
BYTE arp_process_rx (void)
{
	static DEVICE_INFO remote_device_info;
	ARP_PACKET arp_packet;


	switch(sm_arp)
	{
	case SM_ARP_IDLE:
		//-----------------------------------------------------------------------------
		//-----------------------------------------------------------------------------
		//----- AN ARP PACKET IS WAITING TO BE READ OUT OF THE NIC RECEIVE BUFFER -----
		//-----------------------------------------------------------------------------
		//-----------------------------------------------------------------------------

		//READ THE PACKET INTO OUR BUFFER
		//We can't do this as the ARP_PACKET structure is not aligned for 32bit architectures so will often be padded:
		//if (!nic_read_array((BYTE*)&arp_packet, sizeof(arp_packet)))
		//	return(1);									//Error - packet was too small - dump it
		
		//Instead we do this to avoid padding problems:
		if (!nic_read_array((BYTE*)&arp_packet.hardware_type, 2))
			######(#);									//Error - packet was too small - dump it
		## (!##############((####*)&##########.########, #))
			######(#);									//Error - packet was too small - dump it
		## (!##################(&##########.############))
			######(#);									//Error - packet was too small - dump it
		## (!##################(&##########.############))
			######(#);									//Error - packet was too small - dump it
		## (!##############((####*)&##########.#######, #))
			######(#);									//Error - packet was too small - dump it
		## (!##############((####*)&##########.###############, #))
			######(#);									//Error - packet was too small - dump it
		## (!##############((####*)&##########.##############, #))
			######(#);									//Error - packet was too small - dump it
		## (!##############((####*)&##########.###############, #))
			######(#);									//Error - packet was too small - dump it
		## (!##############((####*)&##########.##############, #))
			######(#);									//Error - packet was too small - dump it

		//Swap the bytes in hardware type, protocol & operation
		##########.############## = #################(##########.##############);
		##########.######## = #################(##########.########);
		##########.####### = #################(##########.#######);

		//DUMP THE PACKET FROM THE NIC BUFFER
		####################();

		//Check hardware type
		//(Don't care about the high byte, the low byte should be 0x01)
		## ((##########.############## & ######) != ######)
			######(#);

		//Check protocol
		//(Should be 0x0800)
		## (##########.######## != ######)
			######(#);

		//Check the mac address length
		## (##########.############# != ######(########))
			######(#);

		//Check the IP address length
		## (##########.############# != ######(#######))
			######(#);

		//CHECK THE TARGET IP ADDRESS MATCHES US
		## ((##########.###############.#[#] != ###############.#[#]) ||
			(##########.###############.#[#] != ###############.#[#]) ||
			(##########.###############.#[#] != ###############.#[#]) ||
			(##########.###############.#[#] != ###############.#[#]))
		{
			//----- PACKET IS NOT FOR US -----
			######(#);
		}

		//----- CHECK THE OP CODE -----
		//(Don't care about the high byte)
		## ((##########.####### & ######) == ####################)
		{
			//--------------------------------
			//----- ARP REQUEST RECEIVED -----
			//--------------------------------
			//Store the remote device details in case we can't tx our response now
			####################.###########.#[#] = ##########.#################.#[#];
			####################.###########.#[#] = ##########.#################.#[#];
			####################.###########.#[#] = ##########.#################.#[#];
			####################.###########.#[#] = ##########.#################.#[#];
			####################.###########.#[#] = ##########.#################.#[#];
			####################.###########.#[#] = ##########.#################.#[#];
			####################.##########.### = ##########.###############.###;
			
			###### = ###################;
			//Don't break so that we attempt to reply now;
		}
		#### ## ((##########.####### & ######) == #####################)
		{
			//---------------------------------
			//----- ARP RESPONSE RECEIVED -----
			//---------------------------------
			//Store the response ready for the requesting function to check when its next called
			#####################.###########.#[#] = ##########.#################.#[#];
			#####################.###########.#[#] = ##########.#################.#[#];
			#####################.###########.#[#] = ##########.#################.#[#];
			#####################.###########.#[#] = ##########.#################.#[#];
			#####################.###########.#[#] = ##########.#################.#[#];
			#####################.###########.#[#] = ##########.#################.#[#];
			#####################.##########.### = ##########.###############.###;
			######(#);
		}
		####
		{
			//---------------------------
			//----- INVALID OP CODE -----
			//---------------------------
			######(#);
		}



	#### ###################:		//case SM_ARP_IDLE will fall into this case if we want to try to reply to a received request
		//-----------------------------
		//-----------------------------
		//----- SEND ARP RESPONSE -----
		//-----------------------------
		//-----------------------------
		## (#############())
		{
			//--------------------------------------------
			//----- WE CAN TRANSMIT OUR RESPONSE NOW -----
			//--------------------------------------------
			##############(&####################, #####################);
			###### = ###########;
			######(#);
		}
		####
		{
			//-----------------------------------------------------------
			//----- CAN'T TX OUR RESPONSE NOW - TRY AGAIN NEXT TIME -----
			//-----------------------------------------------------------
			######(#);
		}

	}
	######(#);
}



//***********************************
//***********************************
//********** ARP TX PACKET **********
//***********************************
//***********************************
//nic_setup_tx() must have been called first
void arp_tx_packet (DEVICE_INFO *remote_device_info, WORD op_code)
{

	//----- WRITE THE ETHERNET HEADER -----
	write_eth_header_to_nic(&remote_device_info->mac_address, ETHERNET_TYPE_ARP);

	//----- WRITE THE ARP PACKET -----
	//SEND HARDWARE TYPE [2]
	nic_write_next_byte((BYTE)(ETHERNET_HARDWARE_TYPE >> 8));
	nic_write_next_byte((BYTE)(ETHERNET_HARDWARE_TYPE & 0x00ff));

	//SEND PROTOCOL TYPE [2]
	nic_write_next_byte((BYTE)(ETHERNET_PROTOCOL_ARP >> 8));
	nic_write_next_byte((BYTE)(ETHERNET_PROTOCOL_ARP & 0x00ff));

	//SEND MAC ADDRESS LENGTH [1]
	#####################((####)######(########));

	//SEND IP ADDRESS LENGTH [1]
	#####################((####)######(#######));

	//SEND OP CODE [2]
	#####################((####)(####### >> #));
	#####################((####)(####### & ######));

	//SEND OUR MAC ADDRESS [6]
	#####################(#################.#[#]);
	#####################(#################.#[#]);
	#####################(#################.#[#]);
	#####################(#################.#[#]);
	#####################(#################.#[#]);
	#####################(#################.#[#]);

	//SEND OUR IP ADDRESS [4]
	#####################(###############.#[#]);
	#####################(###############.#[#]);
	#####################(###############.#[#]);
	#####################(###############.#[#]);

	//SEND DESTINATION MAC ADDRESS [6]
	## (####### == #####################)
	{
		//Sending a response - return to senders mac
		#####################(#####################>###########.#[#]);
		#####################(#####################>###########.#[#]);
		#####################(#####################>###########.#[#]);
		#####################(#####################>###########.#[#]);
		#####################(#####################>###########.#[#]);
		#####################(#####################>###########.#[#]);
	}
	####
	{
		//Sending a request - broadcast
		#####################(####);
		#####################(####);
		#####################(####);
		#####################(####);
		#####################(####);
		#####################(####);
	}

	//SEND DESTINATION IP ADDRESS [4]
	//If destination is not on the same subnet as us then send this to our gateway instead
	//Once the gateway returns its mac address the pending TCP/IP communication will automatically get sent to the
	//gateway which will do its job and forward packets on.
	## (((###############.#[#] ^ #####################>##########.#[#]) & #################.#[#]) ||
		((###############.#[#] ^ #####################>##########.#[#]) & #################.#[#]) ||
		((###############.#[#] ^ #####################>##########.#[#]) & #################.#[#]) ||
		((###############.#[#] ^ #####################>##########.#[#]) & #################.#[#]) )
	{
		//TARGET IS NOT ON OUR SUBNET - USE THE GATEWAY IP ADDRESS
		#####################(#####################.#[#]);
		#####################(#####################.#[#]);
		#####################(#####################.#[#]);
		#####################(#####################.#[#]);
	}
	####
	{
		//TARET IS ON OUR SUBNET - USE THE TARGET IP ADDRESS
		#####################(#####################>##########.#[#]);
		#####################(#####################>##########.#[#]);
		#####################(#####################>##########.#[#]);
		#####################(#####################>##########.#[#]);
	}



	//----- SEND THE PACKET -----
	##############();

	######;
}














