//****************************************************************************************
//****************************************************************************************
//  Project Name:		TCP/IP DRIVER
//						ARP (ADDRESS RESOLUTION PROTOCOL) C CODE HEADER FILE
//  Copyright:			EMBEDDED-CODE.COM
//
//  Licenced To:        NONE
//  Licence Number:     NONE
//
//  IMPORTANT: These files are copyright of embedded-code.com and are subject to a licence
//  agreement.  All rights reserved.  Unauthorised use, reproduction or distribution of
//  these files may lead to prosecution.
//  Any use in violation of the licence restrictions may subject the user to criminal
//  sanctions under applicable laws, as well as to civil liability for the breach of the
//  terms and conditions of the license.
//  This software is provided in an "as is" condition. No warranties, whether express,
//  implied or statutory, including, but not limited to, implied warranties of
//  merchantability and fitness for a particular purpose apply to this software. 
//  Embedded-code.com shall not be liable in any circumstances for special, incidental
//  or consequential damages, for any reason whatsoever.
//  Please see www.embedded-code.com/licence.php for full details.
//  This file contains hidden markers to identify it uniquely to the licence number it was
//  purchased under.
//  DO NOT REMOVE THIS NOTICE - IT IS A REQUIREMENT OF THE LICENCE AGREEMENT.
//****************************************************************************************
//****************************************************************************************






//##################################
//##################################
//########## USING DRIVER ##########
//##################################
//##################################

//#################################
//##### INCOMING ARP REQUESTS #####
//#################################
//Are dealt with automatically by the stack - no application intervention is required

//####################################
//##### TO RESOLVE AN IP ADDRESS #####
//####################################
//Call the 'arp_resolve_ip_address' function with the IP address that needs resolving (that you want the mac address to be returning from)
//Then periodically call the 'arp_is_resolve_complete' function to determine when the ARP response has been received.
//The calling function must use a timeout in case of no response.
//If trying to resolve an IP address that is not on our subnet then the resolve function will automatically resolve the address for the gateway
//(our_gateway_ip_address) which will then do its job and forward on communications to the remote device.  Therefore for IP addresses outside of our
//subnet bear in mind that just because ARP was successful the remote IP address is not necessarily actually available.
//
//RESOLVING AN IP ADDRESS EXAMPLE:-
/*
	IP_ADDR destination_ip_address;
	MAC_ADDR destination_mac_address;


	case SM_GET_DESTINATION_MAC_ADDRESS:
		//----- DO THE ARP REQUEST -----
		destination_ip_address.v[0] = 213;
		destination_ip_address.v[1] = 171;
		destination_ip_address.v[2] = 193;
		destination_ip_address.v[3] = 5;

		if (arp_resolve_ip_address(&destination_ip_address))
		{
			arp_response_timeout_timer = 10;
			our_state = SM_GET_DESTINATION_MAC_ADDRESS_WAIT
		}
		//Could not transmit right now - try again next time
		break;

	case SM_GET_DESTINATION_MAC_ADDRESS_WAIT:
		//----- WAIT FOR ARP RESPONSE -----
		if (arp_is_resolve_complete (&destination_ip_address, &destination_mac_address))
		{
			//ARP IS COMPLETE
		}

		if (arp_response_timeout_timer == 0)
		{
			//ARP FAILED - REMOTE DEVICE NOT FOUND
		}
		break;
*/


//For further information please see the project technical manual






//*****************************
//*****************************
//********** DEFINES **********
//*****************************
//*****************************
#ifndef ARP_C_INIT		//(Do only once)
#define	ARP_C_INIT


#include "eth-main.h"


#define	ETHERNET_PROTOCOL_ARP		0x0800
#define ARP_OPCODE_REQUEST 			0x0001
#define ARP_OPCODE_RESPONSE 		0x0002


//----- ARP STATE MACHINE STATES -----
typedef enum _ARP_STATE
{
	SM_ARP_IDLE,
	SM_ARP_SEND_REPLY
} ARP_STATE;

//----- DATA TYPE DEFINITIONS -----
//ARP PACKET
typedef struct _ARP_PACKET
{
	WORD		hardware_type;
	WORD		protocol;
	BYTE		mac_addr_len;
	BYTE		protocol_len;
	WORD		op_code;
	MAC_ADDR	sender_mac_addr;
	IP_ADDR		sender_ip_addr;
	MAC_ADDR	target_mac_addr;
	IP_ADDR		target_ip_addr;
} ARP_PACKET;


#endif



//*******************************
//*******************************
//********** FUNCTIONS **********
//*******************************
//*******************************
#ifdef ARP_C				//(Defined only by associated C file)
//-----------------------------------
//----- INTERNAL ONLY FUNCTIONS -----
//-----------------------------------
void arp_tx_packet (DEVICE_INFO *remote_device_info, WORD op_code);


//-----------------------------------------
//----- INTERNAL & EXTERNAL FUNCTIONS -----
//-----------------------------------------
//(Also defined below as extern)
void arp_initialise (void);
BYTE arp_resolve_ip_address(IP_ADDR *ip_address_to_resolve);
BYTE arp_is_resolve_complete (IP_ADDR *ip_address_being_resolved, MAC_ADDR *resolved_mac_address);
BYTE arp_process_rx (void);


#else
//------------------------------
//----- EXTERNAL FUNCTIONS -----
//------------------------------
extern void arp_initialise (void);
extern BYTE arp_resolve_ip_address(IP_ADDR *ip_address_to_resolve);
extern BYTE arp_is_resolve_complete (IP_ADDR *ip_address_being_resolved, MAC_ADDR *resolved_mac_address);
extern BYTE arp_process_rx (void);


#endif




//****************************
//****************************
//********** MEMORY **********
//****************************
//****************************
#ifdef ARP_C				//(Defined only by associated C file)
//--------------------------------------------
//----- INTERNAL ONLY MEMORY DEFINITIONS -----
//--------------------------------------------
BYTE sm_arp;


//--------------------------------------------------
//----- INTERNAL & EXTERNAL MEMORY DEFINITIONS -----
//--------------------------------------------------
//(Also defined below as extern)
DEVICE_INFO arp_last_received_response;


#else
//---------------------------------------
//----- EXTERNAL MEMORY DEFINITIONS -----
//---------------------------------------
extern DEVICE_INFO arp_last_received_response;


#endif










