//****************************************************************************************
//****************************************************************************************
//  Project Name:		TCP/IP DRIVER
//						DHCP (DYNAMIC HOST CONFIGURATION PROTOCOL) C CODE FILE
//  Copyright:			EMBEDDED-CODE.COM
//
//  Licenced To:        NONE
//  Licence Number:     NONE
//
//  IMPORTANT: These files are copyright of embedded-code.com and are subject to a licence
//  agreement.  All rights reserved.  Unauthorised use, reproduction or distribution of
//  these files may lead to prosecution.
//  Any use in violation of the licence restrictions may subject the user to criminal
//  sanctions under applicable laws, as well as to civil liability for the breach of the
//  terms and conditions of the license.
//  This software is provided in an "as is" condition. No warranties, whether express,
//  implied or statutory, including, but not limited to, implied warranties of
//  merchantability and fitness for a particular purpose apply to this software. 
//  Embedded-code.com shall not be liable in any circumstances for special, incidental
//  or consequential damages, for any reason whatsoever.
//  Please see www.embedded-code.com/licence.php for full details.
//  This file contains hidden markers to identify it uniquely to the licence number it was
//  purchased under.
//  DO NOT REMOVE THIS NOTICE - IT IS A REQUIREMENT OF THE LICENCE AGREEMENT.
//****************************************************************************************
//****************************************************************************************


#include "main.h"
#define DHCP_C
#include "eth-dhcp.h"
#undef DHCP_C

#include "eth-main.h"
#include "eth-nic.h"
#include "eth-udp.h"		//UDP is requried for DHCP



//--------------------------------------------------------------------------
//----- GENERATE COMPILE ERROR IF DHCP HAS NOT BEEN DEFINED TO BE USED -----
//--------------------------------------------------------------------------
#ifndef STACK_USE_DHCP
#error DHCP file is included in project but not defined to be used - remove file from project to reduce code size.
#endif





//*************************************
//*************************************
//********** DHCP INITIALISE **********
//*************************************
//*************************************
//Called from the tcp_ip_initialise function
void dhcp_initialise (void)
{
	sm_dhcp = DHCP_INIT;
	nic_linked_and_ip_address_valid = 0;

	//If configured to use DHCP then initialise the IP settings
	if (eth_dhcp_using_manual_settings == 0)
	{
		our_ip_address.val = 0;
		our_subnet_mask.val = 0;
		######################.### = #;
	}
}




//**********************************
//**********************************
//********** PROCESS DHCP **********
//**********************************
//**********************************
void process_dhcp (void)
{
	DEVICE_INFO remote_device_info;



	//----------------------------------------------
	//----- CHECK FOR USING MANUAL IP SETTINGS -----
	//----------------------------------------------
	if (eth_dhcp_using_manual_settings)
	{
		//----- WE ARE CURRENTLY CONFIGURED TO USE MANUAL SETTINGS, NOT DHCP -----
		sm_dhcp = DHCP_BOUND;
		eth_dhcp_1sec_renewal_timer = 1000;				//If our application changes our state to use DHCP cause a request to happen quickly
		eth_dhcp_1sec_lease_timer = 2;
	}


	//------------------------------------
	//----- CHECK FOR NIC NOT LINKED -----
	//------------------------------------
	## (############## == #)
	{
		//----- WE HAVE NO ETHERNET LINK - PUT DHCP INTO INITIALISE STATE WAITING FOR NIC TO BE LINKED -----
		####### = #########;
	}


	//------------------------
	//----- PROCESS DHCP -----
	//------------------------
	###### (#######)
	{
	#### #########:
		//----------------------
		//----- INITIALISE -----
		//----------------------
		
		##################### = #;				//Flag that we do not have valid IP settings
		
		//ENSURE OUR SOCKET IS NOT OPEN
		## (########### != ####################)
			##################(&###########);
		
		## (##################### == #)
		{
			//----- WE ARE CURRENTLY CONFIGURED TO USE DHCP (NOT MANUAL SETTINGS) -----
			###############.### = #;
			#################.### = #;
			#####################.### = #;
			##################### = ###;			//Set a value so that the discover stage doesn't immediatly return to here

			//GENERATE A RANDOM DELAY OF UP TO 10 SECONDS BEFORE CONTACTING THE DHCP SERVER
			//(Good practice to avoid lots of devices hitting a server at once when connectivety returns)
			###### #####################
				#################### = (((####)#################.#[#]) << #) + (####)#################.#[#];		//Use our mac address as the random variable
				##### (#################### > #####)			//Limit to 10 seconds
					#################### #= #####;
				## (#################### < ####)				//Force to at least 1 second
					#################### += ####;
			#####
					#################### = ###;
			######

			//Do discovery next
			####### = ##############;
		}
		#####;



	#### ##############:
		//-------------------------
		//----- DHCP DISCOVER -----
		//-------------------------
		//Send a DHCP discover request

		//Wait to do action (if coming from another state that set a delay to occur first)
		## (####################)
			#####;

		## (##################### == #)
		{
			//----- OUR LEASE HAS EXPIRED - IP SETTINGS NO LONGER VALID -----
			####### = #########;
			#####;
		}


		//TRY AND OPEN A UDP SOCKET
		####################.##########.### = ##########;			//Set to broadcast
		####################.###########.#[#] = ####;
		####################.###########.#[#] = ####;
		####################.###########.#[#] = ####;
		####################.###########.#[#] = ####;
		####################.###########.#[#] = ####;
		####################.###########.#[#] = ####;
		########### = #################(&####################, #################, #################);

		## (########### != ####################)
		{
			//WE HAVE OBTAINED A UDP SOCKET - SETUP FOR TX
			## (#############(###########) == #)
			{
				//TX IS NOT CURRENTLY AVAILABLE
				//Try again next time
				##################(&###########);
				#####;
			}
			//----- SEND DISCOVER PACKET -----
			###############(#####################);

			#################### = #####################;
			####### = #####################;
			#####;
		}
		//CANNOT OPEN A UDP SOCKET
		//There is no free UDP socket currently - Try again next time
		#####;



	#### #####################:
		//-----------------------------------
		//----- WAIT FOR OFFER RESPONSE -----
		//-----------------------------------
		## (#####################(###########))
		{
			//----- RESPONSE RECEVIED -----
			
			//Process the response packet
			## (###############())
			{
				//PACKET PROCESSED AND OUR STATE MACHINE HAS BEEN UPDATED - DO NOTHING
				##################(&###########);
				#####;
			}
			####
			{
				//PACKET WAS NOT VALID FOR SOME REASON
				//Try again after a short delay
				##################(&###########);
				#################### = ####;
				####### = ##############;
				#####;				
			}

		}
		#### ## (#################### == #)
		{
			//------ TIMED OUT - NO RESPONSE FROM DHCP SERVER -----
			//Setup to try again after a delay
			#################### = (((####)#################.#[#]) << #) + (####)#################.#[#];		//Set timer to a random delay based on our mac address
			##### (#################### > ####)
				#################### #= ####;
			## (#################### < ####)
				#################### += ####;

			##################(&###########);
			####### = ##############;
		}
		#####;



	#### #############:		//(These 2 states do the same thing)
	#### ##############:
		//-----------------------------------------
		//----- REQUEST OR RENEW DHCP ADDRESS -----
		//-----------------------------------------
		//Wait to do action
		## (####################)
			#####;

		//TRY AND OPEN A UDP SOCKET
		//Address must be broadcast for a request (after discover) as a DHCP server will not reply if addressed by its IP address until it has ACK'ed the first request
		//When renewing the individual IP address can be used but there isn't any reason to so we always send broadcast
		####################.##########.### = ##########;			//Set to broadcast
		####################.###########.#[#] = ####;
		####################.###########.#[#] = ####;
		####################.###########.#[#] = ####;
		####################.###########.#[#] = ####;
		####################.###########.#[#] = ####;
		####################.###########.#[#] = ####;

		########### = #################(&####################, #################, #################);

		## (########### != ####################)
		{
			//WE HAVE OBTAINED A UDP SOCKET - SETUP FOR TX
			## (#############(###########) == #)
			{
				//TX IS NOT CURRENTLY AVAILABLE
				//Try again next time
				##################(&###########);
				#####;
			}
			//----- SEND REQUEST PACKET -----
			###############(#####################);

			#################### = #####################;
			####### = #####################;
			#####;
		}
		//CANNOT OPEN A UDP SOCKET
		//There is no free UDP socket currently - Try again next time
		#####;



	#### #####################:
		//-------------------------------------
		//----- WAIT FOR REQUEST RESPONSE -----
		//-------------------------------------

		## (#####################(###########))
		{
			//----- RESPONSE RECEVIED -----
			
			//Process the response packet
			## (###############())
			{
				//PACKET PROCESSED AND OUR STATE MACHINE HAS BEEN UPDATED - DO NOTHING
				##################(&###########);
				#####;
			}
			####
			{
				//PACKET WAS NOT VALID FOR SOME REASON
				//Try again after a short delay
				##################(&###########);
				#################### = ####;
				####### = ##############;
				#####;				
			}

		}
		#### ## (#################### == #)
		{
			//------ TIMED OUT - NO RESPONSE FROM DHCP SERVER -----
			//Setup to try again after a delay
			#################### = (((####)#################.#[#]) << #) + (####)#################.#[#];		//Set timer for a retry if this doesn't work
			##### (#################### > ####)
				#################### #= ####;
			## (#################### < ####)
				#################### += ####;

			##################(&###########);
			####### = ##############;
		}
		#####;




	#### ##########:
		//----------------------
		//----- DHCP BOUND -----
		//----------------------
		//We have our IP settings - do we need to renew with the DHCP server?

		##################### = #;				//Flag that we have valid IP settings

		## (##################### == #)
		{
			//----- WE NEED TO RENEW OUR ADDRES -----
			####### = ##############;			
			#####;
		}

		## (##################### == #)
		{
			//----- OUR LEASE HAS EXPIRED - IP SETTINGS NO LONGER VALID -----
			####### = #########;
			#####;
		}

		#####;

	//-------------------------
	//----- END OF SWITCH -----
	//-------------------------
	}	//switch (sm_dhcp)



}





//**************************************
//**************************************
//********** SEND DHCP PACKET **********
//**************************************
//**************************************
void dhcp_tx_packet (BYTE message_type)
{
	WORD w_temp;


	//----- A UDP PACKET HAS BEEN SETUP AND WE ARE READY TO WRITE THE UDP DATA AREA -----

	//Send Boot Request
	udp_write_next_byte(1);

	//Send Hardware Address Type = Ethernet
	udp_write_next_byte(1);

	//Address Length
	udp_write_next_byte(6);

	//Hops
	udp_write_next_byte(0);

	//Transaction ID (Random number - use our mac address)
	udp_write_next_byte(our_mac_address.v[2]);
	###################(###############.#[#]);
	###################(###############.#[#]);
	###################(###############.#[#]);

	//Elapsed Time
	#####################(#);
	#####################(#);

	//Flags
	#####################(#);
	#####################(#);

	//Client IP Address
	#####################(###############.#[#]);
	#####################(###############.#[#]);
	#####################(###############.#[#]);
	#####################(###############.#[#]);

	//Your (Client) IP Address
	#####################(#);
	#####################(#);
	#####################(#);
	#####################(#);

	//Server IP Address
	#####################(#);
	#####################(#);
	#####################(#);
	#####################(#);

	//Relay agent IP
	#####################(#);
	#####################(#);
	#####################(#);
	#####################(#);

	//Our MAC address
	#####################(#################.#[#]);
	#####################(#################.#[#]);
	#####################(#################.#[#]);
	#####################(#################.#[#]);
	#####################(#################.#[#]);
	#####################(#################.#[#]);

	//Unused bytes
	### (###### = #; ###### < ###; ######++)
		#####################(#);

	//DHCP Magic Cookie
	#####################(####);
	#####################(####);
	#####################(####);
	#####################(####);

	//----- BELOW HERE ARE THE DHCP OPTIONS -----
	//(These may be changed if required)

	//Send Option - DHCP Message Type
	#####################(##);					//Option
	#####################(#);						//Length
	#####################(#############);			//Data

	//Send Option - Client ID
	#####################(##);					//Option
	#####################(#);						//Length
	#####################(#);						//Data - Hardware type (Ethernet)
	#####################(#################.#[#]);	//Data - our client ID (MAC address)
	#####################(#################.#[#]);
	#####################(#################.#[#]);
	#####################(#################.#[#]);
	#####################(#################.#[#]);
	#####################(#################.#[#]);


	//If doing DHCP request then send option - dhcp server IP, and send option - our requested IP
	## (####### == #############)
	{
		#####################(##);						//Option
		#####################(#);							//Length
		#####################(#####################.#[#]);	//Data
		#####################(#####################.#[#]);
		#####################(#####################.#[#]);
		#####################(#####################.#[#]);

		#####################(##);						//Option
		#####################(#);							//Length
		#####################(####################.#[#]);	//Data
		#####################(####################.#[#]);
		#####################(####################.#[#]);
		#####################(####################.#[#]);
    }

	//Send our name if we have one assigned to the pointer variable
	## (#####################)
	{
		#####################(##);					//Option
		#####################(##);					//Length
		#####################(#####################[#]);					//Data - our ascii name
		#####################(#####################[#]);
		#####################(#####################[#]);
		#####################(#####################[#]);
		#####################(#####################[#]);
		#####################(#####################[#]);
		#####################(#####################[#]);
		#####################(#####################[#]);
		#####################(#####################[#]);
		#####################(#####################[#]);
		#####################(#####################[##]);
		#####################(#####################[##]);
		#####################(#####################[##]);
		#####################(#####################[##]);
		#####################(#####################[##]);
		#####################(####);
	}


	//End of options marker
	#####################(####);


	//----- SEND THE PACKET -----
	##############();
}



//*****************************************
//*****************************************
//********** RECEIVE DHCP PACKET **********
//*****************************************
//*****************************************
//Returns 1 if a DHCP packet was sucessfully processed, 0 if not
//If return value is 1 then the DHCP state machine has been updated
BYTE dhcp_rx_packet (void)
{
	BYTE data;
	BYTE w_temp;
	IP_ADDR received_ip;
	BYTE option_code;
	BYTE option_len;
	DWORD option_data;
	BYTE option_message_type;
	DWORD option_lease_time;
	IP_ADDR option_subnet_mask;
	IP_ADDR option_gateway;
	//IP_ADDR option_server_ip_addr;

	received_ip.val = 0;
	option_subnet_mask.val = 0;
	##############.### = #;

	//GET OPCODE
	#####################(&####);
	## (#### != #)
		#### #####################;

	//GET ADDRESS TYPE
	#####################(&####);

	//GET ADDRESS LENGTH
	#####################(&####);
	## (#### != #)
		#### #####################;

	//GET HOPS [1]
	//GET TRANSACTION ID [4]
	//GET ELAPSED TIME [2]
	//GET FLAGS [2]
	//GET CLIENT IP [4]
	###(###### = #; ###### < ##; ######++)
		#####################(&####);

	//GET YOUR (CLIENT) IP ADDRESS (The IP address being assigned to us)
	#####################(&###########.#[#]);
	#####################(&###########.#[#]);
	#####################(&###########.#[#]);
	#####################(&###########.#[#]);

	//GET SERVER IP [4]
	//GET RELAY AGENT IP [4]
	###(###### = #; ###### < #; ######++)
		#####################(&####);

	//GET MAC ADDRESS
	#####################(&####);
	## (#### != #################.#[#])
		#### #####################;
		
	#####################(&####);
	## (#### != #################.#[#])
		#### #####################;
		
	#####################(&####);
	## (#### != #################.#[#])
		#### #####################;
		
	#####################(&####);
	## (#### != #################.#[#])
		#### #####################;
		
	#####################(&####);
	## (#### != #################.#[#])
		#### #####################;
		
	#####################(&####);
	## (#### != #################.#[#])
		#### #####################;

	//UNUSED BYTES
	###(###### = #; ###### < (## + ## + ###); ######++)
		#####################(&####);

	//GET MAGIC COOKIE
	#####################(&####);
	## (#### != ####)
		#### #####################;
		
	#####################(&####);
	## (#### != ####)
		#### #####################;
		
	#####################(&####);
	## (#### != ####)
		#### #####################;
		
	#####################(&####);
	## (#### != ####)
		#### #####################;


	//----- BELOW HERE ARE THE DHCP OPTIONS -----

	//GET EACH OPTION
	##### (
			#####################(&###########) && 
			(########### != ####) &&
			#####################(&##########)
			)
	{
		//----- GET THE OPTION DATA -----
		########### = #;
		### (###### = #; ###### < ##########; ######++)
		{
			########### <<= #;
			#####################(&####);
			########### += (#####)####;
		}

		//----- PROCESS THE OPTION -----
		

		## ((########### == ##) && (########## == #))
		{
			//----- MESSAGE TYPE -----
			##################### = (####)###########;
		}	
		#### ## ((########### == ##) && (########## == #))
		{
			//----- LEASE TIME -----
			################### = ###########;
		}	
		#### ## ((########### == #) && (########## == #) && (##################### == ##################))
		{
			//----- SUBNET MASK -----
			####################.#[#] = (####)(########### >> ##);
			####################.#[#] = (####)((########### & ##########) >> ##);
			####################.#[#] = (####)((########### & ##########) >> #);
			####################.#[#] = (####)(########### & ##########);
		}
		#### ## ((########### == #) && (########## == #) && (##################### == ##################))
		{
			//----- GATEWAY -----
			###############.#[#] = (####)(########### >> ##);
			###############.#[#] = (####)((########### & ##########) >> ##);
			###############.#[#] = (####)((########### & ##########) >> #);
			###############.#[#] = (####)(########### & ##########);
		}
		//else if ((option_code == 3) && (option_len == 4) && (option_message_type == DHCP_MESSAGE_ACK))
		//{
		//	//----- DHCP SERVER IP ADDRESS ----- (Enable this if you want it for some reason - we get the server IP address from the packet header)
		//	option_server_ip_addr = option_data;
		//}


		//<<<< ADD ANY OTHER OPTIONS REQURIED HERE


	}
	

	//----- STORE THE DHCP SERVER IP ADDRESS -----
	#####################.### = ##########[###########].####################.##########.###;


	//----- PROCESS PACKET -----
	## (##################### == ##################)
	{
		//----- NAK RECEIVED -----
		//This should not happen - restart DHCP in case the DHCP server want's to give us a new address

		#################### = #;
		####### = #########;
	}
	#### ## ((####### == #####################) && (##################### == ####################))
	{
		//----- WE ARE WAITING FOR AN OFFER AND IT HAS BEEN RECEIVED -----
		//Store the DHCP server mac address
		#####################.#[#] = ##########[###########].####################.###########.#[#];
		#####################.#[#] = ##########[###########].####################.###########.#[#];
		#####################.#[#] = ##########[###########].####################.###########.#[#];
		#####################.#[#] = ##########[###########].####################.###########.#[#];
		#####################.#[#] = ##########[###########].####################.###########.#[#];
		#####################.#[#] = ##########[###########].####################.###########.#[#];

		//Store the offered IP address
		####################.### = ###########.###;

		//Set DHCP state machine to new state
		#################### = #;
		####### = #############;
	}
	#### ## (
			(####### == #####################) &&
			(##################### == ##################)
			)
	{
		//----- WE HAVE SENT A REQUEST OR RENEWAL REQUEST AND WE HAVE RECEIVED THE RESPONSE -----

		//Get the lease time	
		## (################### > #####################)
			##################### = #####################;
		#### ## (################### < #####################)
			##################### = #####################;
		####
			##################### = ###################;

		##################### = (##################### >> #);			//We will ask to renew the lease half way through our lease time

		//Store our new IP address
		###############.### = ###########.###;

		//Store our new subnet mask
		## (####################.###)
			#################.### = ####################.###;
		
		//Store our new gateway address
		## (###############.###)
			#####################.### = ###############.###;

		//Set DHCP state machine to new state
		####### = ##########;

	}
	####
	{
		#### #####################;
	}

	//-----------------------------------------------------
	//----- GOOD DHCP RESPONSE RECEIVED AND PROCESSED -----
	//-----------------------------------------------------
	####################();
	######(#);

dhcp_rx_packet_dump:
	//-------------------------------------------
	//----- RESPONSE RECEIVED BUT NOT VALID -----
	//-------------------------------------------
	udp_dump_rx_packet();
	return(0);
}





