//****************************************************************************************
//****************************************************************************************
//  Project Name:		TCP/IP DRIVER
//						DNS (DOMAIN NAME SYSTEM) C CODE FILE
//  Copyright:			EMBEDDED-CODE.COM
//
//  Licenced To:        NONE
//  Licence Number:     NONE
//
//  IMPORTANT: These files are copyright of embedded-code.com and are subject to a licence
//  agreement.  All rights reserved.  Unauthorised use, reproduction or distribution of
//  these files may lead to prosecution.
//  Any use in violation of the licence restrictions may subject the user to criminal
//  sanctions under applicable laws, as well as to civil liability for the breach of the
//  terms and conditions of the license.
//  This software is provided in an "as is" condition. No warranties, whether express,
//  implied or statutory, including, but not limited to, implied warranties of
//  merchantability and fitness for a particular purpose apply to this software. 
//  Embedded-code.com shall not be liable in any circumstances for special, incidental
//  or consequential damages, for any reason whatsoever.
//  Please see www.embedded-code.com/licence.php for full details.
//  This file contains hidden markers to identify it uniquely to the licence number it was
//  purchased under.
//  DO NOT REMOVE THIS NOTICE - IT IS A REQUIREMENT OF THE LICENCE AGREEMENT.
//****************************************************************************************
//****************************************************************************************


#include "main.h"
#define	ETH_DNS_C
#include "eth-dns.h"
#undef	ETH_DNS_C

#include "eth-main.h"
#include "eth-udp.h"
#include "eth-arp.h"
#include "eth-nic.h"




//-------------------------------------------------------------------------
//----- GENERATE COMPILE ERROR IF UDP HAS NOT BEEN DEFINED TO BE USED -----
//-------------------------------------------------------------------------
#ifndef STACK_USE_DNS
#error DNS file is included in project but not defined to be used - remove file from project to reduce code size.
#endif




//**********************************
//**********************************
//********** DO DNS QUERY **********
//**********************************
//**********************************
//url_pointer
//	= 'www.###.com' etc for a web address (non case sensitive)
//	= ###.com etc for a mail server mx address
//dns_type
//	= 1 for host (www lookup)
//	= 15 for mx mailserver (email lookup)
//Returns:
//	1 = DNS query started
//	0 = unable to start query currently - try again later
BYTE do_dns_query (BYTE *url_pointer, BYTE dns_type)
{
	BYTE count;

	//----- CHECK WE ARE CONNECTED -----
	if (!nic_linked_and_ip_address_valid)
		return(0);

	//----- CHECK WE ARE NOT ALREADY DOIGN A QUERY -----
	if ((dns_state != SM_DNS_IDLE) && (dns_state != SM_DNS_FAILED))
	{
		//We can't currently do a new dns query as DNS is already active
		return (0);
	}
	
	//----- START NEW DNS QUERY -----
	//Copy the name
	for (count = 0; count < DNS_MAX_URL_LENGTH; count++)
		#################[#####] = ###########[#####];

	//Store the type
	##################### = ########;

	######### = #####################;			//The 'process_dns' state machine will do the dns query
	###### (#);
}



//****************************************
//****************************************
//********** CHECK DNS RESPONSE **********
//****************************************
//****************************************
//Returns IP address
//	0 				= dns not yet complete
//	0xFFFFFFFF		= dns failed1
//	Anything else	= IP address of requested URL
IP_ADDR check_dns_response (void)
{
	IP_ADDR return_ip;


	if (dns_state == SM_DNS_IDLE)
	{
		//----- DNS IS COMPLETE AND WAS SUCESSFUL -----
		return (dns_resolved_ip_address);
	}
	else if (dns_state == SM_DNS_FAILED)
	{
		//----- DNS IS COMPLETE BUT FAILED -----
		dns_resolved_ip_address.val = 0xffffffff;
		###### (#######################);
	}
	
	//----- DNS IS NOT YET COMPLETE -----
	#########.### = #;
	###### (#########);
}



//*********************************
//*********************************
//********** PROCESS DNS **********
//*********************************
//*********************************
//This function is called reguarly by tcp_ip_process_stack
void process_dns (void)
{
	static BYTE eth_dns_10ms_clock_timer_last;


	//-----------------------------------
	//----- CHECK FOR UPDATE TIMERS -----
	//-----------------------------------
	if ((BYTE)((BYTE)(ethernet_10ms_clock_timer & 0x000000ff) - eth_dns_10ms_clock_timer_last) >=	10)
	{
		eth_dns_10ms_clock_timer_last = (BYTE)(ethernet_10ms_clock_timer & 0x000000ff);
		
		//TIMEOUT TIMER
		if (dns_100ms_timeout_timer)
			dns_100ms_timeout_timer--;
	}



	switch(dns_state)
	{
	#### ###########:
		//--------------------------------------------
		//----- IDLE - NO DNS REQUEST IS PENDING -----
		//--------------------------------------------
		
		#####;


	#### #####################:
		//------------------------------------
		//----- A DNS REQUEST IS PENDING -----
		//------------------------------------

		//----- USE ARP TO GET THE MAC ADDRESS OF OUR GATEWAY DEVICE -----
		## (#####################(&#####################))
		{
			//REQUEST WAS SENT
			##################### = #####################;
		
			######### = #####################;
			#####;
		}
		//Request cannot be sent right now - try again next time
		#####;


	#### #####################:
		//-------------------------------------
		//----- WAIT FOR THE ARP RESPONSE -----
		//-------------------------------------
		//Check for timeout
		## (##################### == #)
			#### ####################;

		//Wait for the ARP response
		## (#####################(&#####################, &##############.###########) )
		{
			//----- ARP RESPONSE RECEVIED FROM GATEWAY -----
			
			//DNSServerNode.MACAddr has been set in the function above
	        
			//Set the IP address
			##############.##########.### = #####################.###;

			//Send DNS request
			######### = #####################;
		}
		#####;


	#### #####################:
		//----------------------------
		//----- SEND DNS REQUEST -----
		//----------------------------
		//Check for timeout
		## (##################### == #)
			#### ####################;

		//Open a UDP socket
		############### = ################# (&##############, #################, #################);
		
		//Check a socket was allocated
		## (############### == ####################)
			#####;							//No UDP socket currently available - try again next time

		//Start the TX
		## (#############(###############))
		{
			//TX setup - send the packet
			##################(###################, #####################);
			##################### = #####################;
			######### = #####################;
		}
		####
		{
			//Can't tx right now - close the socket ready to try again next time
			##################(&###############);
		}
        #####;


	#### #####################:
		//----------------------------------------------------------------
		//----- A DNS REQUEST HAS BEEN SENT - WAITING FOR A RESPONSE -----
		//----------------------------------------------------------------
		//Check for timeout
		## (##################### == #)
		{
			//TIMED OUT - DNS FAILED
			##################(&###############);
			#### ####################;
		}

		## (#####################(###############))
		{
			## (####################(###################, #####################, &#####################))
			{
				//A VALID DNS RESPONSE HAS BEEN RECEIVED
				######### = ###########;
			}
			####
			{
				//AN INVALID DNS RESPONSE HAS BEEN RECEIVED
				######### = ##############;
			}
			##################(&###############);				//Close the UDP socket
		}
		#####;


	#### ##############:
		//-----------------------------------
		//----- THE LAST REQUEST FAILED -----
		//-----------------------------------
		//We stay in this state until the next request so that the failed state can be determined whenever the calling function wants to know
		##################(&###############);				//Close the UDP socket   
		#####;

	}

	//----------------
	//----- EXIT -----
	//----------------
	######;

//----------------------
//----- DNS FAILED -----
//----------------------
process_dns_failed:
	dns_state = SM_DNS_FAILED;
	return;
}



//**************************************
//**************************************
//********** SEND DNS REQUEST **********
//**************************************
//**************************************
//requested_domain_name
//	= 'www.###.com' etc for a web address (non case sensitive)
//	= ###.com etc for a mail server mx address
//qtype
//	= 1 for host (www lookup)
//	= 15 for mx mailserver (email lookup)
void dns_send_request (BYTE *requested_domain_name, BYTE qtype)
{

	BYTE name_character_number;
	BYTE name_length;
	BYTE name_string[64];
	BYTE loop_count;


	//UDP PACKET HAS ALREADY BEEN SETUP

	//---------------------------
	//----- SEND THE HEADER -----
	//---------------------------
	
	//----- ID [2] -----
	//(Message ID number)
	udp_write_next_byte(0x12);
	udp_write_next_byte(0x23);

	//----- FLAGS [2] -----
	//QR (Bit 15)
	//	0 = Query 
	//	1 = response
	//OPCODE (Bits 14:11)
	//	0 = standard query (QUERY)
	//	1 = inverse query (IQUERY)
	//	2 = server status request (STATUS)
	//	3-15 =unused
	//AA (Bit 10)
	//	Authoritative Answer - valid in responses - specifies that the responding name server is an authority for the domain name in question section
	//	(May have multiple owner names because of aliases.  The AA bit corresponds to the name which matches the query name, or the first owner name
	//	in the answer section)
	//TC (Bit 9)
	//	TrunCation - specifies that this message was truncated due to length greater than that permitted on the transmission channel
	//RD (Bit 8)
	//	Recursion Desired - may be set in a query and is copied into the response. 
	//	1 = pursue the query recursively (carry out DNS and return the IP address instead of simply responding with a list of name servers if IP is unknown)
	udp_write_next_byte(0x01);		//Recursion desired - we rely on the gateway will carry out recursion for us

	//RA (Bit 7)
	//	Recursion Available - set or cleared in a response, and denotes whether recursive query support is available in the name server
	//Z (Bits 6:4)
	//	Reserved for future use.  Must be zero in all queries and responses
	//RCODE (Bits 3:0)
	//	Response code
	//	0 = No error condition
	//	1 = Format error - The name server was unable to interpret the query
	//	2 = Server failure
	//	3 = Name Error (Meaningful only for responses from an authoritative name server, this code signifies that the domain name referenced in the
	//	query does not exist)
	//	4 = Not Implemented (The name server does not support the requested kind of query)
	//	5 = Refused
	//	6-15 = unused
	udp_write_next_byte(0x00);


	//----- QDCOUNT [2] -----
	//(The number of entries in the question section)
	udp_write_next_byte(0x00);
	###################(####);

	//----- ANCOUNT [2] -----
	//(The number of resource records in the answer section)
	#####################(####);
	#####################(####);

	//----- NSCOUNT [2] -----
	//(The number of name server resource records in the authority records section)
	#####################(####);
	#####################(####);

	//----- ARCOUNT [2] -----
	//(The number of resource records in the additional records section)
	#####################(####);
	#####################(####);


	//-----------------------------
	//----- SEND THE QUESTION -----
	//-----------------------------

	//----- QNAME [#] -----
	//A domain name represented as a sequence of labels, where each label consists of a length octet followed by that
	//number of bytes.  The domain name terminates with the zero length octet for the null label of the root.  Note
	//that this field may be an odd number of bytes and no padding is used.
	//An example:-
	//0x03, 'W', 'W', 'W',
	//0x05, 'Y', 'A', 'H', 'O', 'O',
	//0x03, 'C', 'O', 'M',
	//0x00

	##################### = #;
	##### (#)
	{
		//GET THE NEXT LABEL SECTION OF THE NAME
		########### = #;
		##### (#)
		{
			###########[###########] = ###################[#####################];
			## (###########[###########] == '.')		//Period name seperator?
				#####;
			## (###########[###########] == ####)		//Null end of name?
				#####;
			## (########### >= ##)						//Check for overflow
				#####;
				
			###########++;
			#####################++;
		}
		#####################++;

		//Write the next label section of the name
		#####################(###########);
		### (########## = #; ########## < ###########; ##########++)
			#####################(###########[##########]);

		## (###########[###########] == ####)		//Exit if reached null end of name?
			#####;
	}
	#####################(#);		//Null termination

	//----- QTYPE [2] -----
	//A two octet code which specifies the type of the query.  The values for this field include all codes valid for a
	//TYPE field, together with some more general codes which can match more than one type of RR
	//Value 1 = a host address (IP v4 address), 15 = mail mx
	#####################(#);
	#####################(#####);

	//----- QCLASS [2] -----
	//A two octet code that specifies the class of the query.  For example, the QCLASS field is IN for the Internet
	#####################(#);			//Value 1 = The internet
	#####################(#);


	//------------------
	//----- ANSWER -----
	//------------------
	//----- NAME [#] -----
	//A domain name to which this resource record pertains

	//----- TYPE [2] -----
	//Two bytes containing one of the RR type codes.  This field specifies the meaning of the data in the RDATA field

	//----- CLASS [2] -----
	//Two bytes which specify the class of the data in the RDATA field

	//----- TTL [4] -----
	//The time interval in seconds that the resource record may be cached before it should be discarded.  Zero values are
	//interpreted to mean that the RR can only be used for the transaction in progress, and should not be cached

	//----- RDLENGTH [2] -----
	//The length in bytes of the RDATA field

	//----- RDATA [#] -----
	//A variable length string of bytes that describes the resource.  The format of this information varies
	//according to the TYPE and CLASS of the resource record.  For example, the if the TYPE is A and the CLASS is IN,
	//the RDATA field is a 4 octet ARPA Internet address


	//---------------------
	//----- AUTHORITY -----
	//---------------------
	//----- NAME [#] -----
	//A domain name to which this resource record pertains

	//----- TYPE [2] -----
	//Two bytes containing one of the RR type codes.  This field specifies the meaning of the data in the RDATA field

	//----- CLASS [2] -----
	//Two bytes which specify the class of the data in the RDATA field

	//----- TTL [4] -----
	//The time interval (in seconds) that the resource record may be cached before it should be discarded.  Zero values are
	//interpreted to mean that the RR can only be used for the transaction in progress, and should not be cached

	//----- RDLENGTH [2] -----
	//The length in bytes of the RDATA field

	//----- RDATA [#] -----
	//A variable length string of bytes that describes the resource.  The format of this information varies
	//according to the TYPE and CLASS of the resource record.  For example, the if the TYPE is A and the CLASS is IN,
	//the RDATA field is a 4 octet ARPA Internet address


	//----------------------
	//----- ADDITIONAL -----
	//----------------------
	
	//----- NAME [#] -----
	//A domain name to which this resource record pertains

	//----- TYPE [2] -----
	//Two bytes containing one of the RR type codes.  This field specifies the meaning of the data in the RDATA field

	//----- CLASS [2] -----
	//Two bytes which specify the class of the data in the RDATA field

	//----- TTL [4] -----
	//The time interval (in seconds) that the resource record may be cached before it should be discarded.  Zero values are
	//interpreted to mean that the RR can only be used for the transaction in progress, and should not be cached

	//----- RDLENGTH [2] -----
	//The length in bytes of the RDATA field

	//----- RDATA [#] -----
	//A variable length string of bytes that describes the resource.  The format of this information varies
	//according to the TYPE and CLASS of the resource record.  For example, the if the TYPE is A and the CLASS is IN,
	//the RDATA field is a 4 octet ARPA Internet address


	//---------------------------
	//----- SEND THE PACKET -----
	//---------------------------
	##############();

}



//*********************************************
//*********************************************
//********** GET RECEIVED DNS PACKET **********
//*********************************************
//*********************************************
//We request that the network gateway device does a recursive search (i.e. it goes through the tree of name servers to find the IP address).  As
//the DNS response can use compression (compression of names by repeated names or sections of names being re-used later on in the packet) we don't
//check the response names (as this is unecessary and would involve a great deal of complexity and ram space).  Instead we check the answer and
//additional sections looking for an IP address.
//DNS may return several IP addresses (for instance for large sites) and in this case we just use the first one.
//
//Returns
//	1 = good response received and resolved_ip_address has been loaded
//	0 = DNS failed
BYTE dns_check_response(BYTE *requested_domain_name, BYTE qtype, IP_ADDR *resolved_ip_address)
{
	BYTE loop_count;
	BYTE received_byte;
	WORD_VAL qd_count;
	WORD_VAL an_count;
	WORD_VAL ns_count;
	WORD_VAL ar_count;
	BYTE name_character_number = 0;
	BYTE label_length;
	WORD data_length;
	BYTE response_is_good;


	//---------------------------
	//----- GET THE HEADER -----
	//---------------------------
	
	//----- ID [2] -----
	//(Message ID number)
	udp_read_next_rx_byte(&received_byte);
	if (received_byte != 0x12)				//Should match the number we sent
		goto dns_check_response_failed;
	udp_read_next_rx_byte(&received_byte);
	if (received_byte != 0x23)
		#### #########################;

	//----- FLAGS [2] -----
	//QR (Bit 15)
	//	0 = Query 
	//	1 = response
	//OPCODE (Bits 14:11)
	//	0 = standard query (QUERY)
	//	1 = inverse query (IQUERY)
	//	2 = server status request (STATUS)
	//	3-15 =unused
	//AA (Bit 10)
	//	Authoritative Answer - valid in responses - specifies that the responding name server is an authority for the domain name in question section
	//	(May have multiple owner names because of aliases.  The AA bit corresponds to the name which matches the query name, or the first owner name
	//	in the answer section)
	//TC (Bit 9)
	//	TrunCation - specifies that this message was truncated due to length greater than that permitted on the transmission channel
	//RD (Bit 8)
	//	Recursion Desired - may be set in a query and is copied into the response. 
	//	1 = pursue the query recursively (carry out DNS and return the IP address instead of simply responding with a list of name servers if IP is unknown)
	#####################(&##############);
	## ((############## & ####)  == #)		//Check message is a response
		#### #####################;

	//RA (Bit 7)
	//	Recursion Available - set or cleared in a response, and denotes whether recursive query support is available in the name server
	//Z (Bits 6:4)
	//	Reserved for future use.  Must be zero in all queries and responses
	//RCODE (Bits 3:0)
	//	Response code
	//	0 = No error condition
	//	1 = Format error - The name server was unable to interpret the query
	//	2 = Server failure
	//	3 = Name Error (Meaningful only for responses from an authoritative name server, this code signifies that the domain name referenced in the
	//	query does not exist)
	//	4 = Not Implemented (The name server does not support the requested kind of query)
	//	5 = Refused
	//	6-15 = unused
	#####################(&##############);
	## ((############## & ####)  == #)		//Check server supports recursion
		#### #####################;

	## ((############## & ####)  != #)		//Response code should be zero for no error
		#### #####################;

	//----- QDCOUNT [2] -----
	//(The number of entries in the question section)
	#####################(&########.#[#]);
	#####################(&########.#[#]);

	//----- ANCOUNT [2] -----
	//(The number of resource records in the answer section)
	#####################(&########.#[#]);
	#####################(&########.#[#]);

	//----- NSCOUNT [2] -----
	//(The number of name server resource records in the authority records section)
	#####################(&########.#[#]);
	#####################(&########.#[#]);

	//----- ARCOUNT [2] -----
	//(The number of resource records in the additional records section)
	#####################(&########.#[#]);
	#####################(&########.#[#]);


	//------------------------------------
	//----- GET THE QUESTION ENTRIES -----
	//------------------------------------

	//----- QNAME [#] -----
	//(A domain name represented as a sequence of labels, where each label consists of a length octet followed by that
	//number of bytes.  The domain name terminates with the zero length octet for the null label of the root.  Note
	//that this field may be an odd number of bytes; no padding is used)

	## (########.### !=#)				//There should only be 1 question entry as we only asked 1 question
		#### #####################;

	##################### = #;
	#####(#)
	{
		//GET THE LENGTH OF THE NEXT LABEL
		#####################(&#############);
		## ((############# == #) && (##################### > #))				//If the length is 0 then name is complete
			#####;

		//THE NEXT CHARACTER IN THE REQUESTED NAME SHOULD BE '.'
		## (#####################)
		{
			## (*(##################### + #####################) != '.')
				#### #####################;
			
			#####################++;
		}

		//GET AND CHECK EACH CHARACTER MATCHES OUR QUESTION
		### (########## = #; ########## < #############; ##########++)
		{
			#####################(&##############);
			## (#####################(##############) != #####################(*(##################### + #####################)))
				#### #####################;
			#####################++;
		}
	}

	//----- QTYPE [2] -----
	//(A two octet code which specifies the type of the query.  The values for this field include all codes valid for a
	//TYPE field, together with some more general codes which can match more than one type of RR)
	//Should match the request we sent
	#####################(&##############);
	## (############## != #)					//Check high byte
		#### #####################;

	#####################(&##############);
	## (############## != #####)				//Check low byte
		#### #####################;

	//----- QCLASS [2] -----
	//(A two octet code that specifies the class of the query. Value 1 = The internet
	//Should match the request we sent
	#####################(&##############);
	## (############## != #)					//Check high byte
		#### #####################;

	#####################(&##############);
	## (############## != #)					//Check low byte
		#### #####################;


	//------------------
	//----- ANSWER -----
	//------------------
	// THIS SECTION MAY CONTAIN THE IP ADDRESS WE REQUESTED
	//If doing a type 1 'host address' dns then this section will contain the IP address
	//If doing a type 15 'mx' mailserver address the additional section will contain the IP address if its not here
	##### (########.###)
	{
		################## = #;		//Default to good record

		//NAME [#]
		//(A domain name to which this resource record pertains)
		//As we're not buffering the entire packet to deal with the possible compression we don't bother to check this
		##### (#)
		{
			#####################(&##############);

			## (############## == ####)
			{
				//Null - name is done
				#####;
			}

			## (############## & ####)
			{
				//Using a compressed name
				//Get the next byte of the name location offset and exit
				#####################(&##############);
				#####;
			}
			####
			{
				//Using a normal name
				//Get all of the name characters
				############# = ##############;
				##### (#############)
				{
					#####################(&##############);
					###############;
				}
			}
		}

		//----- TYPE [2] -----
		//(Two bytes containing one of the RR type codes.  This field specifies the meaning of the data in the RDATA field)
		//Should be type 'A' = host address (value = 0x0001)
		#####################(&##############);		//Get and check high byte
		## (############## != ####)
			################## = #;

		#####################(&##############);		//Get and check low byte
		## (############## != ####)
			################## = #;

		//----- CLASS [2] -----
		//(Two bytes which specify the class of the data in the RDATA field)
		//Should be class 'IN' = internet (value = 0x0001)
		#####################(&##############);		//Get and check high byte
		## (############## != ####)
			################## = #;

		#####################(&##############);		//Get and check low byte
		## (############## != ####)
			################## = #;

		//----- TTL [4] -----
		//(The time interval (in seconds) that the resource record may be cached before it should be discarded.  Zero values are
		//interpreted to mean that the RR can only be used for the transaction in progress, and should not be cached)
		#####################(&##############);
		#####################(&##############);
		#####################(&##############);
		#####################(&##############);

		//----- RDLENGTH [2] -----
		//(The length in bytes of the RDATA field)
		#####################(&##############);		//Get and check high byte
		## (############## != ####)
			#### #####################;

		#####################(&##############);		//Get and check low byte
		############# = ##############;
		## (############## != ####)
			################## = #;

		//----- RDATA [#] -----
		//(A variable length string of bytes that describes the resource.  The format of this information varies
		//according to the TYPE and CLASS of the resource record.  For example, the if the TYPE is A and the CLASS is IN,
		//the RDATA field is a 4 octet ARPA Internet address)
		### (########## = #; ########## < #############; ##########++)
		{
			#####################(&##############);
			## (##################)
				#####################>#[##########] = ##############;
		}

		## (##################)
			#### #####################;

		########.#####;
	}


	//---------------------
	//----- AUTHORITY -----
	//---------------------
	// THIS SECTION WON'T CONTAIN THE IP ADDRESS WE REQUESTED - MOVE GET PAST IT
	##### (########.###)
	{
		//----- NAME [#] -----
		//(A domain name to which this resource record pertains)
		##### (#)
		{
			#####################(&##############);

			## (############## == ####)
			{
				//Null - name is done
				#####;
			}

			## (############## & ####)
			{
				//Using a compressed name
				//Get the next byte of the name location offset and exit
				#####################(&##############);
				#####;
			}
			####
			{
				//Using a normal name
				//Get all of the name characters
				############# = ##############;
				##### (#############)
				{
					#####################(&##############);
					###############;
				}
			}
		}

		//----- TYPE [2] -----
		//(Two bytes containing one of the RR type codes.  This field specifies the meaning of the data in the RDATA field)
		#####################(&##############);
		#####################(&##############);

		//----- CLASS [2] -----
		//(Two bytes which specify the class of the data in the RDATA field)
		#####################(&##############);
		#####################(&##############);

		//----- TTL [4] -----
		//(The time interval (in seconds) that the resource record may be cached before it should be discarded.  Zero values are
		//interpreted to mean that the RR can only be used for the transaction in progress, and should not be cached)
		#####################(&##############);
		#####################(&##############);
		#####################(&##############);
		#####################(&##############);

		//----- RDLENGTH [2] -----
		//(The length in bytes of the RDATA field)
		#####################(&##############);
		########### = (############## << #);
		#####################(&##############);
		########### += ##############;

		//----- RDATA [#] -----
		//(A variable length string of bytes that describes the resource.  The format of this information varies
		//according to the TYPE and CLASS of the resource record.  For example, the if the TYPE is A and the CLASS is IN,
		//the RDATA field is a 4 octet ARPA Internet address)
		##### (###########)
		{
			#####################(&##############);
			##############;
		}
		
		########.#####;
	}


	//----------------------
	//----- ADDITIONAL -----
	//----------------------
	// THIS SECTION MAY CONTAIN THE IP ADDRESS WE REQUESTED
	##### (########.###)
	{
		################## = #;		//Default to good record

		//----- NAME [#] -----
		//(A domain name to which this resource record pertains)
		//As we're not buffering the entire packet to deal with the possible compression we don't bother to check this
		##### (#)
		{
			#####################(&##############);

			## (############## == ####)
			{
				//Null - name is done
				#####;
			}

			## (############## & ####)
			{
				//Using a compressed name
				//Get the next byte of the name location offset and exit
				#####################(&##############);
				#####;
			}
			####
			{
				//Using a normal name
				//Get all of the name characters
				############# = ##############;
				##### (#############)
				{
					#####################(&##############);
					###############;
				}
			}
		}

		//----- TYPE [2] -----
		//(Two bytes containing one of the RR type codes.  This field specifies the meaning of the data in the RDATA field)
		//Should be type 'A' = host address (value = 0x0001)
		#####################(&##############);		//Get and check high byte
		## (############## != ####)
			################## = #;

		#####################(&##############);		//Get and check low byte
		## (############## != ####)
			################## = #;

		//----- CLASS [2] -----
		//(Two bytes which specify the class of the data in the RDATA field)
		//Should be class 'IN' = internet (value = 0x0001)
		#####################(&##############);		//Get and check high byte
		## (############## != ####)
			################## = #;

		#####################(&##############);		//Get and check low byte
		## (############## != ####)
			################## = #;

		//----- TTL [4] -----
		//(The time interval (in seconds) that the resource record may be cached before it should be discarded.  Zero values are
		//interpreted to mean that the RR can only be used for the transaction in progress, and should not be cached)
		#####################(&##############);			//We don't care as we're not caching responses
		#####################(&##############);
		#####################(&##############);
		#####################(&##############);

		//----- RDLENGTH [2] -----
		//(The length in bytes of the RDATA field)
		//Should be 4
		#####################(&##############);		//Get and check high byte
		## (############## != ####)
			#### #####################;

		#####################(&##############);		//Get and check low byte
		############# = ##############;
		## (############## != ####)
			################## = #;

		//----- RDATA [#] -----
		//(A variable length string of bytes that describes the resource.  The format of this information varies
		//according to the TYPE and CLASS of the resource record.  For example, the if the TYPE is A and the CLASS is IN,
		//the RDATA field is a 4 octet ARPA Internet address)
		### (########## = #; ########## < #############; ##########++)
		{
			#####################(&##############);
			## (##################)
				#####################>#[##########] = ##############;
		}

		## (##################)
			#### #####################;

		########.#####;
	}


	//------------------------
	//----- BAD RESPONSE -----
	//------------------------
dns_check_response_failed:
	udp_dump_rx_packet();                             //We are done with this packet

	return (0);


	//-------------------------
	//----- GOOD RESPONSE -----
	//-------------------------
dns_check_response_good:
	udp_dump_rx_packet();                             //We are done with this packet

	return (1);
}






