//****************************************************************************************
//****************************************************************************************
//  Project Name:		TCP/IP DRIVER
//						ICMP (INTERNET CONTROL MESSAGE PROTOCOL) C CODE FILE
//  Copyright:			EMBEDDED-CODE.COM
//
//  Licenced To:        NONE
//  Licence Number:     NONE
//
//  IMPORTANT: These files are copyright of embedded-code.com and are subject to a licence
//  agreement.  All rights reserved.  Unauthorised use, reproduction or distribution of
//  these files may lead to prosecution.
//  Any use in violation of the licence restrictions may subject the user to criminal
//  sanctions under applicable laws, as well as to civil liability for the breach of the
//  terms and conditions of the license.
//  This software is provided in an "as is" condition. No warranties, whether express,
//  implied or statutory, including, but not limited to, implied warranties of
//  merchantability and fitness for a particular purpose apply to this software. 
//  Embedded-code.com shall not be liable in any circumstances for special, incidental
//  or consequential damages, for any reason whatsoever.
//  Please see www.embedded-code.com/licence.php for full details.
//  This file contains hidden markers to identify it uniquely to the licence number it was
//  purchased under.
//  DO NOT REMOVE THIS NOTICE - IT IS A REQUIREMENT OF THE LICENCE AGREEMENT.
//****************************************************************************************
//****************************************************************************************



#include "main.h"
#define ICMP_C
#include "eth-icmp.h"
#undef ICMP_C

#include "eth-main.h"
#include "eth-nic.h"
#include "eth-ip.h"


//--------------------------------------------------------------------------
//----- GENERATE COMPILE ERROR IF ICMP HAS NOT BEEN DEFINED TO BE USED -----
//--------------------------------------------------------------------------
#ifndef STACK_USE_ICMP
#error ICMP file is included in project but not defined to be used - remove file from project to reduce code size.
#endif






//***************************************************************
//***************************************************************
//********** PROCESS RECEIVED ICMP ECHO REQUEST PACKET **********
//***************************************************************
//***************************************************************
//Returns 1 if a valid ICMP ECHO request packet is received, 0 if not
BYTE icmp_process_received_echo_request (WORD *icmp_id, WORD *icmp_sequence, BYTE *icmp_data_buffer, WORD data_remaining_bytes)
{
	ICMP_HEADER icmp_header;
	WORD received_checksum;
	WORD calculated_checksum;
	BYTE calculate_checksum_next_byte_is_low;


	//----- GET THE ICMP HEADER -----
	if (nic_read_array((BYTE*)&icmp_header, sizeof(icmp_header)) == 0)
		return (0);								//Error - packet was too small - dump

	//----- GET THE ICMP DATA -----
	if (nic_read_array((BYTE*)icmp_data_buffer, (data_remaining_bytes - sizeof(icmp_header))) == 0)
		return (0);								//Error - packet was too small - dump

	//----- CALCULATE THE CHECKSUM AND CHECK IT MATCHES -----
	//Store the received checksum and then set to zero in the header ready for checksum calculation
    ################### = ###########.########;
    ###########.######## = #;

	//Calculate the checksum for the header
	##################### = #;
	##################### = #;
	##################### (&#####################, &#####################, (####*)&###########, ######(###########));

	//Calculate the checksum for the data
	##################### (&#####################, &#####################, (####*)##################, (##################### # ######(###########)));
	
	//Ensure checksums match
	################### = ~###################;
	################### = #################(###################);

	## (################### != #####################)
		######(#);

	//SWAP IDENTIFIER, SEQ NUMBER & CHECKSUM WORDS
	###########.########## = #################(###########.##########);
	###########.################# = #################(###########.#################);
	//icmp_header.checksum = swap_word_bytes(icmp_header.checksum);		//Don't need to

	//----- CHECK CODE IS ECHO REQUEST -----
	## (###########.#### != ###################)
		######(#);


	//STORE THE RETURN VALUES TO BE USED IN THE REPLY
	*####### = ###########.##########;
	*############## = ###########.#################;

	######(#);
}


//**************************************
//**************************************
//********** SEND ICMP PACKET **********
//**************************************
//**************************************
//nic_setup_tx() must have been called first
void icmp_send_packet(DEVICE_INFO *remote_device_info,BYTE icmp_packet_type, BYTE *data_buffer, WORD data_length, WORD *icmp_id, WORD *icmp_sequence)
{
	ICMP_HEADER icmp_header;
	WORD calculated_checksum;
	BYTE calculated_checksum_next_byte_is_low;


	//----- SETUP THE HEADER -----
	icmp_header.type = icmp_packet_type;
	icmp_header.code = 0;
	icmp_header.checksum = 0;
	icmp_header.identifier = *icmp_id;
	###########.############### = *#############;

	//SWAP IDENTIFIER, SEQ NUMBER & CHECKSUM WORDS
	###########.########## = #################(###########.##########);
	###########.################# = #################(###########.#################);
	//icmp_header.checksum = swap_word_bytes(icmp_header.checksum);		//Done later


	//----- CALCULATE THE HEADER CHECKSUM -----
	##################### = #;
	##################### = #;
	##################### (&#####################, &#####################, (####*)&###########, ######(###########));

	//Add the data buffer to the checksum
	##################### (&#####################, &#####################, (####*)###########, ###########);

	//Add the calculated checksum to the header
	###########.######## = #################(~#####################);


	//----- WRITE THE IP HEADER -----
	#################(####################, ##################);


	//----- WRITE THE ICMP HEADER -----
	#################((####*)&###########, ######(###########));


	//----- WRITE THE ICMP DATA -----
	#################(###########, ###########);


	//----- SEND THE PACKET -----
	#############();
}






