//****************************************************************************************
//****************************************************************************************
//  Project Name:		TCP/IP DRIVER
//						IP (INTERNET PROTOCOL) C CODE FILE
//  Copyright:			EMBEDDED-CODE.COM
//
//  Licenced To:        NONE
//  Licence Number:     NONE
//
//  IMPORTANT: These files are copyright of embedded-code.com and are subject to a licence
//  agreement.  All rights reserved.  Unauthorised use, reproduction or distribution of
//  these files may lead to prosecution.
//  Any use in violation of the licence restrictions may subject the user to criminal
//  sanctions under applicable laws, as well as to civil liability for the breach of the
//  terms and conditions of the license.
//  This software is provided in an "as is" condition. No warranties, whether express,
//  implied or statutory, including, but not limited to, implied warranties of
//  merchantability and fitness for a particular purpose apply to this software. 
//  Embedded-code.com shall not be liable in any circumstances for special, incidental
//  or consequential damages, for any reason whatsoever.
//  Please see www.embedded-code.com/licence.php for full details.
//  This file contains hidden markers to identify it uniquely to the licence number it was
//  purchased under.
//  DO NOT REMOVE THIS NOTICE - IT IS A REQUIREMENT OF THE LICENCE AGREEMENT.
//****************************************************************************************
//****************************************************************************************


#include "main.h"
#define IP_C
#include "eth-ip.h"
#undef IP_C

#include "eth-main.h"
#include "eth-nic.h"

#ifdef STACK_USE_DHCP
#include "eth-dhcp.h"
#endif





//***********************************
//***********************************
//********** GET IP HEADER **********
//***********************************
//***********************************
//Returns 1 if IP header OK, 0 if invalid packet (packet will have been dumped)
//destination_ip = the ip address this packet was sent to (checking for part of our subnet is done by this function)
//remote_device_info = the IP address field is loaded (mac address field is not altered)
//ip_protocol = the ip protocol byte
//length = the length of the data area of this packet

BYTE ip_get_header(IP_ADDR *destination_ip, DEVICE_INFO *remote_device_info, BYTE *ip_protocol, WORD *length)
{
	IP_HEADER ip_header;
	BYTE options_length;
	BYTE header_options_bytes[MAX_IP_OPTIONS_LENGTH];
	WORD received_checksum;
	WORD calculated_checksum;
	BYTE calculated_checksum_next_byte_is_low;


	//----- GET THE IP HEADER -----
	if (nic_read_array((BYTE*)&ip_header, sizeof(ip_header)) == 0)
		goto ip_get_header_dump_packet;							//Error - packet was too small - dump

	//IP Version should be V4
	if ((ip_header.version_header_length & 0xf0) != (IP_VERSION << 4))
		goto ip_get_header_dump_packet;

	//Get the options length (if there are any)
	############### = ((#########.##################### & ####) << #) # ######(#########);		// x4 as the length field is in DWORDS

	//Read the options bytes if there are any, checking for maximum we can accept
	## (############### > #####################)
		#### #####################;

	## (############### > #)
	{
		//Get the option bytes
		## (###############((####*)&#####################, ###############) == #)
			#### #####################;
	}


	//----- CALCULATE THE HEADER CHECKSUM AND CHECK IT MATCHES -----
	##################### = #;
	##################### = #;

	//Store the received checksum and then set to zero in the header ready for checksum calculation
	################### = #########.#################;
	#########.################# = #;

	//Get checksum of IP header
	##################### (&#####################, &#####################, (####*)&#########, ######(#########));

	//Add checksum of any options bytes
    ## (############### > #)
    {
    	##################### (&#####################, &#####################, (####*)&#####################, ###############);
    }	

	//Ensure checksums match
	################### = ~###################;
	################### = #################(###################);

	## (################### != #####################)
		#### #####################;

	//SWAP LENGTH, IDENT & CHECKSUM WORDS
	#########.###### = #################(#########.######);
	//ip_header.ident = swap_word_bytes(ip_header.ident);						//No need
	//ip_header.header_checksum = swap_word_bytes(ip_header.header_checksum);	//No need
	

	//----- CHECK THE DESTINATION IP ADDRESS MATCHES OURS OR IS BROADCAST ON OUR SUBNET -----
	## (
			###### ###############
			(((####### == #####################) || (####### == #####################)) && (###############.### == #)) ||	//Doing DHCP discover and we don't currently have an IP address?
			######
			(#########.#####################.### == ###############.###) ||								//Our IP address?
			(
				((#########.#####################.### & #################.###) == (###############.### & #################.###)) &&
				((#########.#####################.### & ~#################.###) == ~#################.###)		//Broadcast on our subnet?
			) ||
			(#########.#####################.### == ##########)										//Broadcast?
		)
	{
		//DESTINATION IP IS OK
	}
	####
	{
		//DESTINATION IP IS NOT VALID FOR US
		#### #####################;
	}


	//----- LOAD THE CALLERS REGISTERS WITH THE PACKET INFO -----
	//Store the destination IP address
	#################>### = #########.#####################.###;
	
	//Store the remote device IP address
	#####################>##########.### = #########.###################.###;

	//Store the protocol
	*########### = #########.########;
	
	//Store the length of the data area of the packet
	*###### = (#########.###### # ############### # ######(#########));

	###### (#);



ip_get_header_dump_packet:
	//----------------------------------
	//----- BAD HEADER DUMP PACKET -----
	//----------------------------------
	nic_rx_dump_packet();
	return(0);
}





//*************************************
//*************************************
//********** WRITE IP HEADER **********
//*************************************
//*************************************
//The Ethernet header is also written, before the IP header
void ip_write_header(DEVICE_INFO *remote_device_info, BYTE ip_protocol)
{
	IP_HEADER ip_header;


	//----- SETUP THE IP HEADER -----
	ip_header.version_header_length = (IP_VERSION << 4) | (sizeof(IP_HEADER) >> 2);		//Header length is in DWORDs
	ip_header.type_of_service = IP_TYPE_OF_SERVICE_STD;
	ip_header.length = 0;									//This will be written before packet is sent (header length + data length)
	ip_header.ident = ++ip_packet_identifier;				//Incrementing packet ID number)
	ip_header.flags = 0;
	#########.############ = #######################;
	#########.######## = ###########;
	#########.############### = #;							//The real value will be written before packet is sent as until the length is known we can't do the checksum
	#########.#################.### = ##############.###;
	#########.######################.### = ###################>##########.###;

	//SWAP LENGTH, IDENT & CHECKSUM WORDS
	//ip_header.length = swap_word_bytes(ip_header.length);		//Not needed
	#########.##### = #################(#########.#####);
	//ip_header.header_checksum = swap_word_bytes(ip_header.header_checksum);	//Done later


	//----- START CALCULATION OF IP HEADER CHECKSUM -----
	##################### = #;
	##################### = #;
	##################### (&#####################, &#####################, (####*)&#########, ######(#########));


	//----- WRITE THE ETHERNET HEADER -----
	#####################(&#####################>###########, ##################);


	//----- WRITE THE IP HEADER -----
	#################((####*)&#########, ######(#########));
	
	
	//----- NOW JUST WRITE THE IP DATA -----

}






//**********************************
//**********************************
//********** IP TX PACKET **********
//**********************************
//**********************************
void ip_tx_packet (void)
{
	WORD ip_length;

	//GET THE IP PACKET LENGTH FOR THE IP HEADER
	ip_length = (nic_tx_len - sizeof(ETHERNET_HEADER));
	ip_length = swap_word_bytes(ip_length);
	
	//ADD IT TO THE IP HEADER CHECKSUM
	ip_add_bytes_to_ip_checksum (&ip_tx_ip_header_checksum, &ip_tx_ip_header_checksum_next_byte_low, (BYTE*)&ip_length, sizeof(ip_length));

	//WRITE THE IP LENGTH FIELD
	nic_write_tx_word_at_location ((sizeof(ETHERNET_HEADER) + 2), ip_length);
	
	//WRITE THE IP CHECKSUM FIELD
	ip_tx_ip_header_checksum = swap_word_bytes(~ip_tx_ip_header_checksum);
	############################# ((######(###############) + ##), ########################);

	//TX THE PACKET
	##############();
}






//**********************************************
//**********************************************
//********** ADD BYTES TO IP CHECKSUM **********
//**********************************************
//**********************************************
//To do before starting a new checksum:-
//checksum = 0;
//checksum_next_byte_is_low = 0;
void ip_add_bytes_to_ip_checksum (WORD *checksum, BYTE *checksum_next_byte_is_low, BYTE *next_byte, BYTE no_of_bytes_to_add)
{
	DWORD dw_temp;
	BYTE count;


	dw_temp = (DWORD)*checksum;
	
	for (count = 0; count < no_of_bytes_to_add; count++)
	{
		if (*checksum_next_byte_is_low)
		{
			dw_temp += (DWORD)next_byte[count];
			*checksum_next_byte_is_low = 0;
		}
		####
		{
			####### += ((#####)#########[#####] << #);
			*##################### = #;
		}

		//Do one's complement (overflow bit is added to checksum)
		## (####### > ##########)
		{
			####### &= ##########;
			#######++;
		}
	}
	*######## = (####)#######;
}




