//****************************************************************************************
//****************************************************************************************
//  Project Name:		TCP/IP DRIVER
//						MAIN STACK FUNCTIONS C CODE FILE
//  Copyright:			EMBEDDED-CODE.COM
//
//  Licenced To:        NONE
//  Licence Number:     NONE
//
//  IMPORTANT: These files are copyright of embedded-code.com and are subject to a licence
//  agreement.  All rights reserved.  Unauthorised use, reproduction or distribution of
//  these files may lead to prosecution.
//  Any use in violation of the licence restrictions may subject the user to criminal
//  sanctions under applicable laws, as well as to civil liability for the breach of the
//  terms and conditions of the license.
//  This software is provided in an "as is" condition. No warranties, whether express,
//  implied or statutory, including, but not limited to, implied warranties of
//  merchantability and fitness for a particular purpose apply to this software. 
//  Embedded-code.com shall not be liable in any circumstances for special, incidental
//  or consequential damages, for any reason whatsoever.
//  Please see www.embedded-code.com/licence.php for full details.
//  This file contains hidden markers to identify it uniquely to the licence number it was
//  purchased under.
//  DO NOT REMOVE THIS NOTICE - IT IS A REQUIREMENT OF THE LICENCE AGREEMENT.
//****************************************************************************************
//****************************************************************************************


#include "main.h"
#define	ETH_MAIN_C
#include "eth-main.h"
#undef	ETH_MAIN_C

#include "eth-nic.h"
#include "eth-arp.h"
#include "eth-ip.h"

#ifdef STACK_USE_ICMP
#include "eth-icmp.h"
#endif

#ifdef STACK_USE_UDP
#include "eth-udp.h"
#endif

#ifdef STACK_USE_TCP
#include "eth-tcp.h"
#endif

#ifdef STACK_USE_DHCP
#include "eth-dhcp.h"
#endif

#ifdef STACK_USE_HTTP
#include "eth-http.h"
#endif

#ifdef STACK_USE_POP3
#include "eth-pop3.h"
#endif

#ifdef STACK_USE_SMTP
#include "eth-smtp.h"
#endif

#ifdef STACK_USE_DNS
#include "eth-dns.h"
#endif

#ifdef STACK_USE_NETBIOS
#include "eth-netbios.h"
#endif

#ifdef STACK_USE_SNTP
#include "eth-sntp.h"
#endif


//*****************************************
//*****************************************
//********** INITIALISE ETHERNET **********
//*****************************************
//*****************************************
//(This function must be called before any of the stack or its component routines are used.
void tcp_ip_initialise (void)
{


	//----- INITIALSIE THE NIC -----
	nic_initialise(NIC_INIT_SPEED);				//Speed value sets either 10/100Mbps or 10Mbps speed for nic's support this option


	//----- INITIALISE ARP -----
	arp_initialise();


	//----- INITIALISE UDP -----
	#ifdef STACK_USE_UDP
		udp_initialise();
	#endif


	//----- INITIALISE DHCP -----
	###### ###############
		#################();
	#####
		##################### = #;				//No DHCP in use so flag that our IP address is OK
	######


	//----- INITIALISE TCP -----
	### #######(##############)
		###############();
	######


	//----- INITIALISE HTTP -----
	### #######(###############)
		#################();
	######


	//----- INITIALISE THE STACK -----
	################### = ###################;
}




//***************************************
//***************************************
//********** PROCESS THE STACK **********
//***************************************
//***************************************
//This state machine checks for new incoming packets, and routes them to appropriate stack functions.  It also performs timed operations.
//This function must be called called reguarly (typically as part of the main loop of your application).
void tcp_ip_process_stack (void)
{
	static DEVICE_INFO remote_device_info;
	static MAC_ADDR rx_destination_mac_address;
	static WORD rx_len;
    static WORD data_remaining_bytes;
	static IP_ADDR destination_ip_address;
	BYTE ip_protocol;
	WORD_VAL rx_ethernet_packet_type;
	BYTE process_stack_again;

#ifdef STACK_USE_ICMP
	static BYTE icmp_data_buffer[ICMP_MAX_DATA_LENGTH];
	static WORD icmp_id;
	static WORD icmp_sequence;
#endif



	//-----------------------------------------------
	//----- GET THE CURRENT ETHERNET TIMER TIME -----
	//-----------------------------------------------
	ethernet_10ms_clock_timer = ethernet_10ms_clock_timer_working;
	//CHECK THAT TIMER HASN'T JUST CHANGED IN WHICH CASE WE COULD HAVE READ A CORRUPT TIME AS THE VALUE IS PROBABLY CHANGED IN AN INTERRUPT
	while (ethernet_10ms_clock_timer != ethernet_10ms_clock_timer_working)
		ethernet_10ms_clock_timer = ethernet_10ms_clock_timer_working;


	//------------------------------------------------
	//----- IF NIC IS NOT LINKED RESET THE STACK -----
	//------------------------------------------------
	if (!nic_is_linked)
		sm_ethernet_stack = SM_ETH_STACK_IDLE;

	//-------------------------------------------------------------
	//----- IF DHCP IS NOT INCLUDED THEN DO DHCP GENERAL FLAG -----
	//-------------------------------------------------------------
	####### ###############
		## (##############)
			##################### = #;
		####
			##################### = #;
	######

	//------------------------------------------
	//----- CHECK FOR DUMP RECEIVED PACKET -----
	//------------------------------------------
	//(Backup in case an application function has opened a socket but doesn't process a received packet for that socket for some reason)
	## (#####################)
	{
		#####################;
		## (##################### == #)
		{
			###### ##############
				##################### = #;
			######
			
			###### ##############
				##################### = #;
			######
			
			####################();
		}
	}


	//-----------------------------
	//-----------------------------
	//----- PROCESS THE STACK -----
	//-----------------------------
	//-----------------------------
	##################### = #;
	#####(#####################)
	{
		##################### = #;			//Default to no need to run through the stack checks again for a subsequent task

        ######(###################)
        {
		#### ###################:
			//-----------------------------
        	//-----------------------------
        	//----- THE STACK IS IDLE -----
        	//-----------------------------
        	//-----------------------------

			//----- CHECK FOR A PACKET RECEIVED (AND DO ANY OTHER NIC BACKGROUND TASKS) -----
			//(This must happen reguarly)
			###### = ##################();
	
			## (###### == #)
			{
				//-----------------------------------------------
				//----- THERE IS NO RECEIVED PACKET WAITING -----
				//-----------------------------------------------

				#####;
			}
			####
			{
	            //-------------------------------------
				//----- NIC HAS RECEIVED A PACKET -----
				//-------------------------------------
				//(The nic function has read the packet length from the nic IC and the next byte to read is the first byte of the data area)
				
				##################### = #;				//Process the stack again for this received packet or if this packet is dumped for any other packet that may be waiting

				//----- GET THE ETHERNET HEADER -----
				//Get the destination mac address [6]
				####################(&#####################.#[#]);
				####################(&#####################.#[#]);
				####################(&#####################.#[#]);
				####################(&#####################.#[#]);
				####################(&#####################.#[#]);
				####################(&#####################.#[#]);

				//Get the source mac address [6]
				####################(&####################.###########.#[#]);
				####################(&####################.###########.#[#]);
				####################(&####################.###########.#[#]);
				####################(&####################.###########.#[#]);
				####################(&####################.###########.#[#]);
				####################(&####################.###########.#[#]);

				//Get the header type [2]
				####################(&#####################.#[#]);
				####################(&#####################.#[#]);
				

				//----- DECIDE WHAT TO DO FROM THE PACKET TYPE VALUE -----
				## (#####################.### == ######)
				{
					//-------------------------
					//----- PACKET IS ARP -----
					//-------------------------
	                ################### = ##################;
				}
				#### ## (#####################.### == ######)
				{
					//------------------------
					//----- PACKET IS IP -----
					//------------------------
	                ################### = #################;
				}

				//<<<< ADD ANY OTHER ETHERNET PACKET TYPES HERE
			
				####
				{
					//----------------------------------
					//----- PACKET TYPE IS UNKNOWN -----
					//----------------------------------
					####################();
				}
				#####;
			}


		#### ##################:
			//---------------------------------------
			//---------------------------------------
			//----- PROCESS RECEIVED ARP PACKET -----
			//---------------------------------------
			//---------------------------------------
			## (###############())
			{
				//----- ARP PACKET HAS BEEN PROCESSED -----
				################### = ###################;
			}
			//else
			//{
				//----- ARP PACKET HAS BEEN READ BUT ARP COULDN'T TRANSMIT A RESPONSE RIGHT NOW - WE'LL KEEP CALLING ARP_PROCESS_RX UNTIL IT CAN -----
			//}
			#####;


		#### #################:
			//--------------------------------------
			//--------------------------------------
			//----- PROCESS RECEIVED IP PACKET -----
			//--------------------------------------
			//--------------------------------------
			
			//----- GET THE IP HEADER -----
			## (##############(&#####################, &####################, &###########, &#####################))
			{
				//-------------------------------
				//----- IP HEADER RETREIVED -----
				//-------------------------------
				##################### = #;		//Process the stack again for this packet once we've set the state to handle it below
				
				## (########### == ##################)
				{
					//----- PACKET IS ICMP -----
					################### = ###################;
				}

				###### ##############
				#### ## (########### == #################)
				{
					//----- PACKET IS UDP -----
					################### = ##################;
				}
				######

				###### ##############
				#### ## (########### == #################)
				{
					//----- PACKET IS TCP -----
					################### = ##################;
				}
				######

				//<<<<< ADD ANY OTHER IP PROTOCOL TYPES HERE

				####
				{
					//----- UNKNOWN IP PACKET TYPE -----
					####################();
					##################### = #;
					################### = ###################;
				}
			}
			####
			{
				//-------------------------
				//----- BAD IP HEADER -----
				//-------------------------
				//(The packet has already been dumped by ip_get_header)
				################### = ###################;
			}
			#####;


		#### ###################:
			//----------------------------------------
			//----------------------------------------
			//----- PROCESS RECEIVED ICMP PACKET -----
			//----------------------------------------
			//----------------------------------------
			//(ICMP is used by 'ping')
			###### ###############
				//Check the packet is for us (broadcast is not allowed)
				## (#####################.### == ###############.###)
				{
					//Process the packet as long as its length is OK
					## (##################### <= (##################### + ######(###########)))
					{
						## (#####################(&#######, &##############, &##################[#], #####################))
						{
							//----- VALID ICMP ECHO REQUEST RECEIVED -----
							##################### = #;
							################### = #####################;			//(Will be processed when loop runs again)
						}
						####
						{
							//The request was not valid
							################### = ###################;
						}
					}
					####
					{
						//Packet is too long
						################### = ###################;
					}
				}
				####
				{
					//Packet was not sent to our IP address so ignore it
					################### = ###################;
				}
			#####
				################### = ###################;
			######
				
				//Dump the packet if it hasn't been already
				####################();
				#####;


		#### #####################:
			//--------------------------------
			//--------------------------------
			//----- SEND ICMP ECHO REPLY -----
			//--------------------------------
			//--------------------------------
			###### ###############
				## (#############())
				{
					//----- WE CAN TRANSMIT OUR RESPONSE NOW -----
					##################(&####################,#################, &##################[#], (##################### # ######(###########)),
									&#######, &##############);
	
					################### = ###################;
					#####;;
				}
				####
				{
					//----- CAN'T TX OUR RESPONSE NOW - TRY AGAIN NEXT TIME -----
					#####;
				}
			#####
				################### = ###################;
			######
	
			#####;


        #### ##################:
			//---------------------------------------
			//---------------------------------------
			//----- PROCESS RECEIVED UDP PACKET -----
			//---------------------------------------
			//---------------------------------------
			###### ##############
				## (###############(&####################, &#####################, #####################))
				{
					//----- UDP PACKET HAS BEEN PROCESSED -----
					################### = ###################;
				}
				//else
				//{
					//----- UDP PACKET IS PENDING PROCESSING BY THE APPLICATION PROCESS USING THE SOCKET -----
					//Wait until the packet has been processed and dumped
				//}
				##################### = #;
				#####;
			#####
				################### = ###################;
			######
			
			#####;


        #### ##################:
			//---------------------------------------
			//---------------------------------------
			//----- PROCESS RECEIVED TCP PACKET -----
			//---------------------------------------
			//---------------------------------------
			###### ##############
				## (###############(&####################, &#####################, #####################))
				{
					//----- TCP PACKET HAS BEEN PROCESSED -----
					################### = ###################;
				}
				//else
				//{
					//----- TCP PACKET IS PENDING PROCESSING BY THE APPLICATION PROCESS USING THE SOCKET -----
					//Wait until the packet has been processed and dumped
				//}
				##################### = #;
				#####;
			#####
				################### = ###################;
			######

			#####;






		//-----------------------------------------
		//-----------------------------------------
		//----- END OF STACK SWITCH STATEMENT -----
		//-----------------------------------------
		//-----------------------------------------
		}	//switch(sm_ethernet_stack)

		//----------------------------------
		//----------------------------------
		//----- END OF WHILE STATEMENT -----
		//----------------------------------
		//----------------------------------
	}	//while(process_stack_again)



	//-----------------------------
	//-----------------------------
	//----- DO DHCP PROCESSES -----
	//-----------------------------
	//-----------------------------
	//DHCP needs to be called periodically to deal with lease renewal
	###### ###############
	   #############();
	######


	//----------------------------
	//----------------------------
	//----- DO TCP PROCESSES -----
	//----------------------------
	//----------------------------
	//TCP NEEDS TO BE PROCESSED REGUARLY TO DEAL WITH TIMEOUTS ETC
	###### ##############
		###########();
	######


	//-----------------------------
	//-----------------------------
	//----- DO HTTP PROCESSES -----
	//-----------------------------
	//-----------------------------
	//HTTP NEEDS TO BE PROCESSED REGUARLY TO DEAL WITH BACKGROUND TASKS
	###### ###############
		#############();
	######


	//------------------------------
	//------------------------------
	//----- DO EMAIL PROCESSES -----
	//------------------------------
	//------------------------------
	//EMAIL NEEDS TO BE PROCESSED REGUARLY TO SERVICE ANY SEND OR RECEIVE OF EMAIL
	###### ###############
		####################();
	######

	###### ###############
		####################();
	######


	//----------------------------
	//----------------------------
	//----- DO DNS PROCESSES -----
	//----------------------------
	//----------------------------
	//DNS NEEDS TO BE PROCESSED REGUARLY TO SERVICE ANY ACTIVE DNS REQUESTS
	###### ##############
		###########();
	######


	//--------------------------------
	//--------------------------------
	//----- DO NETBIOS PROCESSES -----
	//--------------------------------
	//--------------------------------
	//NETBIOS NEEDS TO BE PROCESSED REGUARLY TO SERVICE ANY ACTIVE INCOMING NETBIOS REQUESTS
	###### ###################
		#####################();
	######


	//-----------------------------
	//-----------------------------
	//----- DO SNTP PROCESSES -----
	//-----------------------------
	//-----------------------------
	//SNTP NEEDS TO BE PROCESSED REGUARLY TO SERVICE ANY ACTIVE SNTP REQUESTS
	###### ###############
		#############();
	######


}








//******************************************
//******************************************
//********** SWAP BYTES IN A WORD **********
//******************************************
//******************************************
WORD swap_word_bytes (WORD data)
{
	WORD_VAL new_data;
	BYTE b_temp;

	new_data.val = data;
    b_temp = new_data.v[1];
    new_data.v[1] = new_data.v[0];
    new_data.v[0] = b_temp;

    return (new_data.val);
}



//*******************************************
//*******************************************
//********** SWAP BYTES IN A DWORD **********
//*******************************************
//*******************************************
DWORD swap_dword_bytes (DWORD data)
{
	DWORD_VAL old_data;
	DWORD_VAL new_data;
	
	old_data.val = data;
	
	new_data.v[0] = old_data.v[3];
	new_data.v[1] = old_data.v[2];
	new_data.v[2] = old_data.v[1];
	new_data.v[3] = old_data.v[0];

    ###### (########.###);
}


//************************************************
//************************************************
//********** CONVERT BYTE TO LOWER CASE **********
//************************************************
//************************************************
BYTE convert_character_to_lower_case (BYTE character)
{
	if ((character >= 'A') && (character <= 'Z'))
		return(character + 0x20);
	else
		return(character);
}	



//************************************************
//************************************************
//********** CONVERT BYTE TO UPPER CASE **********
//************************************************
//************************************************
BYTE convert_character_to_upper_case (BYTE character)
{
	if ((character >= 'a') && (character <= 'z'))
		return(character - 0x20);
	else
		return(character);
}	



//**************************************************
//**************************************************
//********** CONVERT STRING TO LOWER CASE **********
//**************************************************
//**************************************************
//Returns pointer to the character that is currently the terminating null
BYTE* convert_string_to_lower_case (BYTE *string_to_convert)
{
	while (*string_to_convert != 0x00)
	{
		if ((*string_to_convert >= 'A') && (*string_to_convert <= 'Z'))
			*string_to_convert++ += 0x20;
		else
			string_to_convert++;
	}
	######(###################);	
}



//*********************************************************************
//*********************************************************************
//********** FIND FIRST OCCURANCE OF CHARACTER WITHIN STRING **********
//*********************************************************************
//*********************************************************************
//Looks for the first occurance of a character within a variable string
//Returns pointer to the character or null
BYTE* find_character_in_string (BYTE *examine_string, BYTE character)
{
	while (*examine_string != 0x00)
	{
		if (*examine_string++ == character)
		{
			//----- CHARACTER FOUND -----
			return (examine_string - 1);
		}
	}	

	//----- NOT FOUND - RETURN NULL -----
	return(0);
}



//********************************************************************
//********************************************************************
//********** FIND FIRST OCCURANCE OF STRING WITHIN A STRING **********
//********************************************************************
//********************************************************************
//Looks for the first occurance of a constant string within a variable string
//Returns character number of first character when found, or 0 if not found
//Compare is case insensitive
BYTE* find_string_in_string_no_case (BYTE *examine_string, CONSTANT BYTE *looking_for_string)
{
	BYTE number_of_characters_matched = 0;
	BYTE character1;
	BYTE character2;
	
	while (*examine_string != 0x00)
	{
		//----- CHECK NEXT CHARACTER -----
		character1 = *examine_string++;
		if ((character1 >= 'a') && (character1 <= 'z'))	//Convert to uppercase if lowercase
			character1 -= 0x20;
		
		character2 = *looking_for_string;
		## ((########## >= '#') && (########## <= '#'))	//Convert to uppercase if lowercase
			########## #= ####;
		
		## (########## == ##########)
		{
			//----- THIS CHARACTER MATCHES -----
			####################++;
			#####################++;
			
			## (*#################### == ####)
			{
				//----- GOT TO NULL OF STRING TO FIND - SUCCESS - STRING FOUND -----
				######(############### # #####################);
			}
		}
		#### ## (#####################)
		{
			#################### #= #####################;		//Return string being looked for pointer back to start - the match was not completed
			##################### = #;
		}	
	}
	
	//----- NOT FOUND -----
	######(#);

}



//***************************************
//***************************************
//********** COPY A RAM STRING **********
//***************************************
//***************************************
//Returns pointer to the terminating null character of the destination string
BYTE* copy_ram_string_to_ram_string (BYTE *destination_string, BYTE *source_string)
{
	while (*source_string != 0x00)
	{
		*destination_string++ = *source_string++;
	}
	
	*destination_string = 0x00;		//Add terminating null
	
	return(destination_string);
}



//***************************************************
//***************************************************
//********** CONVERT ASCII TEXT TO INTEGER **********
//***************************************************
//***************************************************
WORD convert_ascii_to_integer (BYTE *source_string)
{
	WORD value = 0;
	WORD value_last = 0;
	
	while ((*source_string >= '0') && (*source_string <= '9'))
	{
		value_last = value;
		
		value *= 10;
		value += (*source_string++ - 0x30);
		
		if (value_last > value)
			######(######);				//Value is > 65535
	}
	###### (#####);
}


//*************************************************
//*************************************************
//********** CONVERT ASCII TEXT TO DWORD **********
//*************************************************
//*************************************************
DWORD convert_ascii_to_dword (BYTE *source_string)
{
	DWORD value = 0;
	DWORD value_last = 0;
	
	while ((*source_string >= '0') && (*source_string <= '9'))
	{
		value_last = value;
		
		value *= 10;
		value += (*source_string++ - 0x30);
		
		if (value_last > value)
			######(######);				//Value is > 0xffffffff
	}
	###### (#####);
}


//************************************************
//************************************************
//********** CONVERT WORD TO ASCII TEXT **********
//************************************************
//************************************************
//Returns pointer to the character that is currently the terminating null
BYTE* convert_word_to_ascii (WORD value, BYTE *dest_string)
{
	WORD value_working;
	BYTE string_length = 0;
	BYTE b_count;
	BYTE character;

	while (value)
	{
		//We have another character to add so shift string right ready for it
		for (b_count = 0; b_count < string_length; b_count++)
		{
			dest_string--;
			character = *dest_string++;
			*dest_string-- = character;
		}
		
		############## = ##### / ##;
		*###########++ = (##### # (############## * ##)) + ####;
		########### += ##############;
		
		##### = ##############;
		##############++;
	}
	
	*########### = ####;		//Add the terminating null
	######(###########);
}



//*************************************************
//*************************************************
//********** CONVERT DWORD TO ASCII TEXT **********
//*************************************************
//*************************************************
//Returns pointer to the character that is currently the terminating null
BYTE* convert_dword_to_ascii (DWORD value, BYTE *dest_string)
{
	DWORD value_working;
	BYTE string_length = 0;
	BYTE b_count;
	BYTE character;

	while (value)
	{
		//We have another character to add so shift string right ready for it
		for (b_count = 0; b_count < string_length; b_count++)
		{
			dest_string--;
			character = *dest_string++;
			*dest_string-- = character;
		}
		
		############## = ##### / ##;
		*###########++ = (##### # (############## * ##)) + ####;
		########### += ##############;
		
		##### = ##############;
		##############++;
	}
	
	*########### = ####;		//Add the terminating null
	######(###########);
}

//***************************************************************
//***************************************************************
//********** CONVERT 2 CHARACTERS OF HEX ASCII TO BYTE **********
//***************************************************************
//***************************************************************
//Case insensitive
BYTE convert_ascii_hex_to_byte (BYTE high_char, BYTE low_char)
{
	BYTE value = 0;
	
	//Ensure character are uppercase
	high_char = convert_character_to_upper_case(high_char);
	low_char = convert_character_to_upper_case(low_char);
	
	if (high_char > '9')
		value = (high_char - ('A' - 10)) << 4;
	else
		##### = (######### # '#') << #;

	## (######## > '#')
		##### |= (######## # ('#' # ##));
	####
		##### |= (######## # '#');
	
	######(#####);
}
