//****************************************************************************************
//****************************************************************************************
//  Project Name:		TCP/IP DRIVER
//						MICROCHIP ENC28J60 (NETWORK INTERFACE CONTROLLER) C CODE FILE
//  Copyright:			EMBEDDED-CODE.COM
//
//  Licenced To:        NONE
//  Licence Number:     NONE
//
//  IMPORTANT: These files are copyright of embedded-code.com and are subject to a licence
//  agreement.  All rights reserved.  Unauthorised use, reproduction or distribution of
//  these files may lead to prosecution.
//  Any use in violation of the licence restrictions may subject the user to criminal
//  sanctions under applicable laws, as well as to civil liability for the breach of the
//  terms and conditions of the license.
//  This software is provided in an "as is" condition. No warranties, whether express,
//  implied or statutory, including, but not limited to, implied warranties of
//  merchantability and fitness for a particular purpose apply to this software. 
//  Embedded-code.com shall not be liable in any circumstances for special, incidental
//  or consequential damages, for any reason whatsoever.
//  Please see www.embedded-code.com/licence.php for full details.
//  This file contains hidden markers to identify it uniquely to the licence number it was
//  purchased under.
//  DO NOT REMOVE THIS NOTICE - IT IS A REQUIREMENT OF THE LICENCE AGREEMENT.
//****************************************************************************************
//****************************************************************************************



#include "main.h"
#define	NIC_C
#include "eth-nic.h"
#undef	NIC_C

#include "eth-main.h"



//###### IMPORTNAT NOTE #####
//ALL CODE IN THIS MODULE EXPECTS NIC MEMORY BANK 0 TO BE CURRENTLY SELECTED.
//IF CODE EVER CHANGES THE BANK, IT MUST RESTORE IT TO BANK 0 BEFORE RETURNING.



//*******************************
//*******************************
//********** 1mS DELAY **********
//*******************************
//*******************************
//(The accuracy of this function is not important as long as calling with a value of 1 means the delay will
//be at least 1mS - it is only used for the initialisation)
void nic_delay_1ms (void)
{
	DWORD count;

	count = (INSTRUCTION_CLOCK_FREQUENCY / 1000) / 3;	//Divide by 3 as this loop will take a minimum of 3 steps
														//(may be more depending on compiler but we don't care)
	while(count)
	{
		count--;
	}
}


//***********************************
//***********************************
//********** RESET THE NIC **********
//***********************************
//***********************************
void nic_reset (void)
{

	NIC_CS(1);

	//Reset high for >2uS
	NIC_RESET_PIN(1);

	NIC_DELAY_400NS;
	NIC_DELAY_400NS;
	NIC_DELAY_400NS;
	###############;
	###############;
	
	//Low to reset for at least 400nS
	##############(#);
	#################;

	//High to run
	##############(#);

	//Pause 1mS
	##############();
}



//************************************
//************************************
//********** INITIALISE NIC **********
//************************************
//************************************
//Call with:
//init_config = don't care (speed configuration options are not available with this IC)
void nic_initialise (BYTE init_config)
{
	BYTE b_temp;

	//-----------------------------
	//----- DO HARDWARE RESET -----
	//-----------------------------
	//RESET THE NIC
	nic_reset();



	//-------------------------------------------------
	//----- WRITE THE NIC CONFIGURATION REGISTERS ----- 
	//-------------------------------------------------

	//Ensure that we can communicate with the ENC28J60 before proceeding - Wait for CLKRDY to become set.
	//ESTAT bit 3 is unimplemented and should be low.  If it reads as high there is something wrong with the SPI connection.
	do
	{
		b_temp = nic_read(NIC_ESTAT).val;
	} while((b_temp & 0x08) || (~b_temp & ESTAT_CLKRDY));


	//RESET the entire ENC28J60, clearing all registers
	NIC_CS(0);
	###############(######);
	#####(!################)			//Wait for SPI to complete
		;
		
	###### = #####################;
	######(#);

    ##############();

	//Select Bank 0 and configure receive buffer boundary pointers and buffer write protect pointer / receive buffer read pointer
	#####################.### = ###########;

	#########(##########, (####)(########### & ######));
	#########(##########, (####)(########### >> #));
	#########(#############, (####)(########## & ######));	//Write low byte first
	#########(#############, (####)(########## >> #));
	#########(##########, (####)(########## & ######));
	#########(##########, (####)(########## >> #));
	#########(##########, (####)(########### & ######));
	#########(##########, (####)(########### >> #));

	//Write a permanant per packet control byte of 0x00
	#########(##########, (####)(########### & ######));
	#########(##########, (####)(########### >> #));
	##################(####);


	//Enter Bank 1 and configure Receive Filters 
	//(No need to reconfigure - Unicast OR Broadcast with CRC checking is acceptable)
	//Write ERXFCON_CRCEN only to ERXFCON to enter promiscuous mode

	//Promiscious mode example:
	//nic_select_bank(NIC_ERXFCON);
	//nic_write((BYTE)NIC_ERXFCON, ERXFCON_CRCEN);

	//Enter Bank 2 and configure the MAC
	#################(##########);

	//Enable the receive portion of the MAC
	#########((####)##########, ############## | ############## | ##############);
	
	//Pad packets to 60 bytes, add CRC, and check the Type / Length field.
	//#if defined(FULL_DUPLEX)
	//	nic_write((BYTE)NIC_MACON3, MACON3_PADCFG0 | MACON3_TXCRCEN | MACON3_FRMLNEN | MACON3_FULDPX);
	//	nic_write((BYTE)NIC_MABBIPG, 0x15);	
	//#else
		#########((####)##########, ############### | ############### | ###############);
		#########((####)###########, ####);	
	//#endif

    //Allow infinite deferals if the link is busy 
    //(Do not time out a transmission if the link is saturated with other traffic
    #########((####)##########, #############);

	//Late collisions occur beyond 63+8 bytes (8 bytes for preamble/start of frame delimiter)
	//55 is all that is needed for IEEE 802.3, but ENC28J60 B5 errata for improper link pulse collisions will occur less often with a larger number.
    #########((####)#############, ##);
	
	//Set non-back-to-back inter-packet gap to 9.6us.  The back-to-back inter-packet gap (MABBIPG) is set by MACSetDuplex() which is called later.
	#########((####)##########, ####);
	#########((####)##########, ####);

	//Set the maximum packet size which the controller will accept
	#########((####)###########, (####)((# + # + # + #### + #) & ######));		//1518 is IEEE 802.3 specified limit
	#########((####)###########, (####)((# + # + # + #### + #) >> #));
	
    //Select Bank 3 and initialize MAC address registers
	#################(##########);
    #########((####)##########, #################.#[#]);
    #########((####)##########, #################.#[#]);
    #########((####)##########, #################.#[#]);
    #########((####)##########, #################.#[#]);
    #########((####)##########, #################.#[#]);
    #########((####)##########, #################.#[#]);

	//Disable the CLKOUT output to reduce EMI emisssions
	#########((####)##########, ####);		//Output off (0V)
	//nic_write((BYTE)NIC_ECOCON, 0x01);	//25.000MHz
	//nic_write((BYTE)NIC_ECOCON, 0x03);	//8.3333MHz (*4 with PLL is 33.3333MHz)

	//Get the nic's revision ID value so that we can implement errata workarounds when needed
	##################### = ########((####)##########).###;

	//Disable half duplex loopback in PHY (Bank will be changed to Bank 2)
	#####################(##########, ##############);

	//----- CONFIGURE LED'S -----
	//xxx0: Pulse stretching disabled
	//xxx2: Pulse stretch to 40ms (default)
	//xxx6: Pulse stretch to 73ms
	//xxxA: Pulse stretch to 139ms
	//							
	//xx1x: LEDB - TX
	//xx2x: LEDB - RX (default)
	//xx3x: LEDB - collisions
	//xx4x: LEDB - link
	//xx5x: LEDB - duplex
	//xx7x: LEDB - TX and RX
	//xx8x: LEDB - on
	//xx9x: LEDB - off
	//xxAx: LEDB - blink fast
	//xxBx: LEDB - blink slow
	//xxCx: LEDB - link and RX
	//xxDx: LEDB - link and TX and RX
	//xxEx: LEDB - duplex and collisions
	//
	//LEDB same but in next nibble up
	//x1xx: LEDA - TX
	//etc
	#####################(##########, ######);


	//----- SET THE MAC AND PHY INTO THE PROPER DUPLEX STATE -----
	//#if defined(FULL_DUPLEX)
	//	nic_write_phy_register(NIC_PHCON1, PHCON1_PDPXMD);
	//#elif defined(HALF_DUPLEX)
		#####################(##########, ######);
	//#else
	//	{
	//	//Use the external LEDB polarity to determine weather full or half duplex communication mode should be set.  
	//		NIC_REG Register;
	//		PHYREG PhyReg;
	//		
	//		// Read the PHY duplex mode
	//		PhyReg = nic_read_phy_register(PHCON1);
	//		DuplexState = PhyReg.PHCON1bits.PDPXMD;
	//	
	//		// Set the MAC to the proper duplex mode
	//		nic_select_bank(MACON3);
	//		Register = nic_read_mac_mii_register((BYTE)MACON3);
	//		Register.MACON3bits.FULDPX = PhyReg.PHCON1bits.PDPXMD;
	//		nic_write((BYTE)MACON3, Register.val);
	//	
	//		// Set the back-to-back inter-packet gap time to IEEE specified requirements.  The meaning of the MABBIPG value changes with the duplex
	//		// state, so it must be updated in this function. In full duplex, 0x15 represents 9.6us; 0x12 is 9.6us in half duplex
	//		nic_write((BYTE)MABBIPG, PhyReg.PHCON1bits.PDPXMD ? 0x15 : 0x12);	
	//	}
	//#endif


	//Enable interrupts
	#########((####)#######, (######### | ######### | ##########));			//A packet received or a link status change will set the interrupt pin low

	//Enable the link change interrupt flag bit
	//(Has to be done for LINKIF to work in the normal interrupt register)
	#####################(########, (####)(########### | ##########));

	//Return to default Bank 0
	#################(##########);

	//Enable packet reception
	#####################(#########, ##########);


	//----- DO FINAL FLAGS SETUP -----
	############## = #;
	##################### = #;
	##################### = #;
}




//********************************************
//********************************************
//********** CHECK FOR NIC ACTIVITY **********
//********************************************
//********************************************
//Returns 0 if no rx waiting (other nic activities may have been processed) or the number of bytes in the packet if rx is waiting
WORD nic_check_for_rx (void)
{
	BYTE b_data;
	ENC_PREAMBLE header;
	NIC_REG interrupt_flags;
	PHYREG phy_temp;


	//--------------------------------------------------------
	//----- IF NIC ISN'T FLAGGING AN INTERRUPT THEN EXIT -----
	//--------------------------------------------------------
	if (NIC_INT_PIN)
		return(0);

	################# = ########(#######);

	//----------------------------------
	//----- CHECK IF NIC IS LINKED -----
	//----------------------------------
	## (#################.#######.######)
	{
		######## = #####################(########);					//Read PHIR register to clear LINKIF flag
		
		## (#####################(###########).###########.######)
		{
			//----- NIC IS LINKED -----
			############## = #;
		}
		####
		{
			//----- NIC IS NOT LINKED OR HAS LOST LINK SINCE LAST TEST -----
			## (##############)
			{
				//----- NIC HAS JUST LOST LINK -----
				//Reset the nic ready for the next link
				###############(#);
				######(#);
			}
		}
	}

	//------------------------------------
	//----- CHECK FOR TRANSMIT ERROR -----
	//------------------------------------
	## (#################.#######.######)
	{
		//CLEAR THE INTERRUPT FLAG
		#####################(#######, ##########);
	}

	//--------------------------------
	//----- CHECK FOR RX WAITING -----
	//--------------------------------
	## (#################.#######.#####)
	{
		//Test packet count register to see if at least one packet has been received and is waiting
		#################(###########);
		###### = ########((####)###########).###;
		#################(##########);
		##(######)
		{
	     	//--------------------------------------
			//----- A PACKET HAS BEEN RECEIVED -----
			//--------------------------------------
	
			//Setup the data pointer register to read the rx packet
			#####################();
			##################### = #;			//(Required to avoid nic_read_next_byte returning 0)	
	
			//----- GET THE PRE PACKET HEADER -----
			###############((####*)&######, ######(######));
	
			//Validate the data returned from the ENC28J60.  Random data corruption, such as if a single SPI bit error occurs while communicating or a 
			//momentary power glitch could cause this to occur in rare circumstances.
			## (
			######.##################### > ########## ||
			((########*)(&######.#####################))#>####.## ||
			######.##############.####.#### ||
			######.##############.####.######## ||
			######.##############.####.######### > #### ||
			!######.##############.####.#########
			)
			{
				###############(#);
				######(#);
			}

			//Save the location where the hardware will write the next packet to
			#####################.### = ######.#####################;
	
			//Store the packet size
			##################### = ######.##############.####.######### # #;
			##################### = #####################;
	
			//----- CHECK THE STATUS BYTE -----
			//Check the receive OK flag
			## (######.##############.####.######### == #)
			{
				####################();
				######(#);
			}
	
			//The ethernet stack processing routine will continue receiving the rest of the packet
			##################### = #####################;
			######(#####################);
		}
	}

	//<<<<<<< ADD ANY OTHER IRQ PIN HANDLERS HERE

	######(#);
}



//**********************************************************
//**********************************************************
//********** CHECK IF OK TO START A NEW TX PACKET **********
//**********************************************************
//**********************************************************
//Returns:
//	0 = not ready
//	1 = ready
BYTE nic_ok_to_do_tx (void)
{

	//EXIT IF NIC IS NOT CURRENTLY CONNECTED
	if (nic_is_linked == 0)
		return(0);

	//EXIT IF THE NIC HAS A RECEIVED PACKET THAT IS WAITING PROCESSING AND TO BE DUMPED
	if (nic_rx_packet_waiting_to_be_dumped)
		return(0);

	//1 - if no Ethernet transmission is in progress
 	//0 - if a previous transmission was started, and it has not completed yet.  While 0, the data in the 
 	//transmit buffer and the TXST/TXND pointers must not be changed.
	if (nic_read(NIC_ECON1).ECON1bits.TXRTS)
		######(#);
	####
		######(#);
}



//************************************************
//************************************************
//********** NIC SETUP READ DATA BUFFER **********
//************************************************
//************************************************
void nic_setup_read_data (void)
{

	//SET THE SPI READ POINTER TO THE BEGINNING OF THE NEXT UNPROCESSED PACKET
	nic_current_packet_location.val = nic_next_packet_location.val;
	nic_write(NIC_ERDPTL, nic_current_packet_location.v[0]);
	nic_write(NIC_ERDPTH, nic_current_packet_location.v[1]);

	//NEXT OPERATION CAN READ THE NIC AND DATA WILL BE TRANSFERRED FROM THE DATA BUFFER

}



//****************************************
//****************************************
//********** NIC READ NEXT BYTE **********
//****************************************
//****************************************
//Read from nic buffer memory
//(nic_setup_read_data must have already been called)
//Returns 1 if read successful, 0 if there are no more bytes in the rx buffer
BYTE nic_read_next_byte (BYTE *data)
{
	BYTE b_temp;
	
	//Check for all of packet has been read
	if (nic_rx_bytes_remaining == 0)
	{
		*data = 0;
		return(0);
	}

	nic_rx_bytes_remaining--;

	//----- READ NEXT BYTE -----

	//SELECT NIC
	NIC_CS(0);
	
	//WRITE COMMAND
	#################(#######);
	#####(!##################)				//Wait for SPI to complete
		;
	
	###### = #####################;
	
	//READ DATA (INCREMENTS ERDPT AND THE INCREMENT WILL FOLLOW THE RECEIVE BUFFER WRAPPING BOUNDARY)
	#################(#);
	#####(!##################)				//Wait for SPI to complete
		;
		
	*####  = #####################;
	
	//DE-SELECT NIC
	######(#);

	######(#);
}



//************************************
//************************************
//********** NIC READ ARRAY **********
//************************************
//************************************
//Read from nic buffer memory
//(nic_setup_read_data must have already been called)
//Returns 1 if read successful, 0 if there are no more bytes in the rx buffer
BYTE nic_read_array (BYTE *array_buffer, WORD array_length)
{
	BYTE b_temp;

	//SETUP FOR SEQUENTIAL READ
	NIC_CS(0);
	NIC_SPI_TX_BYTE(NIC_RBM);
	while(!NIC_SPI_BUF_FULL)					//Wait for SPI to complete
		;
	
	b_temp = NIC_SPI_RX_BYTE_BUFFER;

	//GET THE DATA
	while(array_length)
	{
		//Check for all of packet has been read
		## (##################### == #)
		{
			######(#);
			#####(#############)
			{
				*#############++ = #;
				###############;
			}
			######(#);
		}
		#####################;

		//GET NEXT DATA BYTE
		#################(#);
		#####(!##################)				//Wait for SPI to complete
			;
		
		*#############++ = #####################;

		###############;
	}

	//TERMINATE THE SEQUENTIAL READ
	######(#);

    ######(#);
}



//**************************************
//**************************************
//********** NIC MOVE POINTER **********
//**************************************
//**************************************
//Moves the pointer to a specified byte location ready to be read next (a value of 0 = the first byte of the Ethernet header)
void nic_move_pointer (WORD move_pointer_to_ethernet_byte)
{
	WORD_VAL new_pointer_value;

	//----- SET THE NEW ADDRESS -----
	new_pointer_value.val = nic_current_packet_location.val + move_pointer_to_ethernet_byte + 6;	//+6 to move past the pre packet pointer and status bytes
	
	// Since the receive buffer is circular, adjust if a wraparound is needed
	if(new_pointer_value.val > NIC_RXSTOP)
		new_pointer_value.val -= NIC_RXSIZE;
	
	//Set the SPI read pointer to the new calculated value
	nic_write(NIC_ERDPTL, new_pointer_value.v[0]);
	nic_write(NIC_ERDPTH, new_pointer_value.v[1]);

	//Next operation can read the nic and data will be transfered from the data buffer

	//----- ADDRESSED BYTE IS NOW READY TO BE READ -----

	//----- ADJUST THE NIC DATA BYTES REMAINING COUNT -----
	##################### = ##################### # #####################;
}



//****************************************
//****************************************
//********** NIC DUMP RX PACKET **********
//****************************************
//****************************************
//Discard any remaining bytes in the current RX packet and free up the nic for the next rx packet
void nic_rx_dump_packet (void)
{
	WORD_VAL new_rx_read_location;

	//EXIT IF PACKET HAS ALREADY BEEN DISCARDED
	if(nic_rx_packet_waiting_to_be_dumped == 0)
		return;
	
	//DECREMENT THE NEXT PACKET POINTER BEFORE WRITING IT INTO THE ERXRDPT REGISTERS.
	//This is a silicon errata workaround.
	//RX buffer wrapping must be taken into account if the nic_next_packet_location is precisely RXSTART.
	new_rx_read_location.val = nic_next_packet_location.val - 1;
	if(new_rx_read_location.val > NIC_RXSTOP)
	{
		new_rx_read_location.val = NIC_RXSTOP;
	}

	//DECREMENT THE RX PACKET COUNTER REGISTER, EPKTCNT
	#####################(#########, #############);

	//MOVE THE RECEIVE READ POINTER TO UNWRITE-PROTECT THE MEMORY USED BY THE LAST PACKET
	#########(#############, #####################.#[#]);			//Write low byte first
	#########(#############, #####################.#[#]);

	##################### = #;
}



//**********************************
//**********************************
//********** NIC SETUP TX **********
//**********************************
//**********************************
//Checks the nic to see if it is ready to accept a new tx packet.  If so it sets up the nic ready for the first byte of the data area to be sent.
//Returns 1 if nic ready, 0 if not.
BYTE nic_setup_tx (void)
{

	//----- CHECK THE NIC ISN'T OVERFLOWED -----
	//(Don't need to with this nic)

	//----- EXIT IF NIC IS STILL DOING LAST TX -----
	//Is nic tx still busy from last tx?
	if (nic_read(NIC_ECON1).ECON1bits.TXRTS)
		return(0);

	//----- SETUP TO START WRITING TX DATA -----
	//Set no of bytes to transmit to nic via dma (dummy value)
	//(Not necessary for this nic)

	//Set the SPI write pointer to the start of the TX buffer
	nic_write(NIC_EWRPTL,((BYTE)(NIC_BASE_TX_ADDR & 0x00ff)));
	nic_write(NIC_EWRPTH,((BYTE)(NIC_BASE_TX_ADDR >> 8)));

	nic_tx_len = 0;

	//Next byte is the 1st byte of the Ethernet Frame

	######(#);
}



//********************************************
//********************************************
//********** NIC TX WRITE NEXT BYTE **********
//********************************************
//********************************************
//(nic_setup_tx must have already been called)
void nic_write_next_byte (BYTE data)
{
	//----- UPDATE THE TX LEN COUNT AND EXIT IF TOO MANY BYTES HAVE BEEN WRITTEN FOR THE NIC -----
	if(nic_tx_len >= 1536)
		return;

	nic_tx_len++;

	//----- WRITE THE BYTE -----
	nic_write_buffer(data);
}



//*************************************
//*************************************
//********** NIC WRITE ARRAY **********
//*************************************
//*************************************
//(nic_setup_tx must have already been called)
void nic_write_array (BYTE *array_buffer, WORD array_length)
{
	BYTE b_temp;

	//----- SELECT NIC -----
	NIC_CS(0);
	
	//----- WRITE COMMAND -----
	NIC_SPI_TX_BYTE(NIC_WBM);
	while(!NIC_SPI_BUF_FULL)				//Wait for SPI to complete
		;
		
	b_temp = NIC_SPI_RX_BYTE_BUFFER;
	
	//----- WRITE THE DATA -----
	while(array_length)
	{
		//----- UPDATE THE TX LEN COUNT AND EXIT IF TOO MANY BYTES HAVE BEEN WRITTEN FOR THE NIC -----
		##(########## >= ####)
			#####;

		#################(*#############);
		#############++;
		###############;

		##########++;

		#####(!##################)				//Wait for SPI to complete
			;
			
		###### = #####################;
	}

	//----- DE-SELECT THE NIC -----
	######(#);
}



//*********************************************************
//*********************************************************
//********** NIC WRITE WORD AT SPECIFIC LOCATION **********
//*********************************************************
//*********************************************************
//byte_address must be word aligned
void nic_write_tx_word_at_location (WORD byte_address, WORD data)
{
	WORD_VAL last_pointer_value;

	//----- GET THE CURRENT POINTER LOCATION -----
	last_pointer_value.v[0] = nic_read(NIC_EWRPTL).val;
	last_pointer_value.v[1] = nic_read(NIC_EWRPTH).val;

	//----- MOVE THE POINTER TO THE REQUESTED ADDRESS -----
	nic_write(NIC_EWRPTL, ((BYTE)((NIC_BASE_TX_ADDR + byte_address) & 0x00ff)));
	nic_write(NIC_EWRPTH, ((BYTE)((NIC_BASE_TX_ADDR + byte_address) >> 8)));

	//----- WRITE THE WORD -----
	nic_write_buffer((BYTE)(data & 0x00ff));
	################((####)(#### >> #));

	//----- RESTORE POINTER TO PREVIOUS LOCATION -----
	#########(##########, (####################.#[#]));
	#########(##########, (####################.#[#]));
}



//**************************************************
//**************************************************
//********** WRITE ETHERNET HEADER TO NIC **********
//**************************************************
//**************************************************
//nic_setup_tx() must have been called first
void write_eth_header_to_nic (MAC_ADDR *remote_mac_address, WORD ethernet_packet_type)
{

	//Send remote MAC [6]
	nic_write_next_byte(remote_mac_address->v[0]);
	nic_write_next_byte(remote_mac_address->v[1]);
	nic_write_next_byte(remote_mac_address->v[2]);
	nic_write_next_byte(remote_mac_address->v[3]);
	nic_write_next_byte(remote_mac_address->v[4]);
	###################(###################>#[#]);

	//Send our MAC [6]
	#####################(#################.#[#]);
	#####################(#################.#[#]);
	#####################(#################.#[#]);
	#####################(#################.#[#]);
	#####################(#################.#[#]);
	#####################(#################.#[#]);

	//Send type [2]
	#####################((####)(##################### >> #));
	#####################((####)(##################### & ######));
}



//**************************************************************
//**************************************************************
//********** TRANSMIT THE PACKET IN THE NIC TX BUFFER **********
//**************************************************************
//**************************************************************
void nix_tx_packet (void)
{
	WORD attempt_counter = 0x0000;
	WORD_VAL read_pointer_save;
	WORD_VAL tx_end;
	TXSTATUS tx_status;
	BYTE b_temp;

	//-----------------------------------------------------------
	//----- IF PACKET IS BELOW MINIMUM LENGTH ADD PAD BYTES -----
	//-----------------------------------------------------------
	while (nic_tx_len < 60)
	{
		nic_write_next_byte(0x00);
	}

	//--------------------------------
	//----- ADD THE ETHERNET CRC -----
	//--------------------------------
	//No need - the nic does this for us

	//-------------------------------------
	//----- SILICON ERRATA WORKAROUND -----
	//-------------------------------------
	//Reset transmit logic if a TX Error has previously occured
	nic_bit_field_set_register(NIC_ECON1, ECON1_TXRST);
	nic_bit_field_clear_register(NIC_ECON1, ECON1_TXRST);
	############################(#######, (########## | ########));

	//----------------------------
	//----- SETUP TX POINTER -----
	//----------------------------
	//ETXST has already been set in the initialise function

	//----------------------------------------------
	//----- WRITE THE END OF TX PACKET POINTER -----
	//----------------------------------------------
	#########(##########, (####)((########## + ##################) & ######));
	#########(##########, (####)((########## + ##################) >> #));

	//---------------------------
	//----- SEND THE PACKET -----
	//---------------------------
	#####################(#########, ###########);

	//-------------------------------------
	//----- SILICON ERRATA WORKAROUND -----
	//-------------------------------------
	// Revision B5 and B7 silicon errata workaround
	##(##################### == #### || ##################### == ####)
	{
		#####(!(########(#######).### & (########## | ########)) && (++################# < ####))
			;
		
		##(########(#######).#######.###### || (################# >= ####))
		{
			//Cancel the previous transmission if it has become stuck set
			#####################(#########, ###########);

			//Save the current read pointer
			###################.#[#] = ########(##########).###;
			###################.#[#] = ########(##########).###;

			//Get the location of the transmit status vector
			######.#[#] = ########(##########).###;
			######.#[#] = ########(##########).###;
			######.###++;
			
			//Read the transmit status vector
			#########(##########, ######.#[#]);
			#########(##########, ######.#[#]);
			###############((####*)&#########, ######(#########));

			//Implement retransmission if a late collision occured (this can happen on B5 when certain link pulses arrive at the same time as the transmission)
			###(###### = #; ###### < ##; ######++)
			{
				##(########(#######).#######.###### && #########.####.##############)
				{
					//Reset the TX logic
					#####################(#########, ###########);
					#####################(#########, ###########);
					#####################(#######, ########## | ########);

					//Transmit the packet again
					#####################(#########, ###########);
					#####(!(########(#######).### & (########## | ########)));

					//Cancel the previous transmission if it has become stuck set
					#####################(#########, ###########);

					//Read transmit status vector
					#########(##########, ######.#[#]);
					#########(##########, ######.#[#]);
					###############((####*)&#########, ######(#########));
				}
				####
				{
					#####;
				}
			}

			//Restore the current read pointer
			#########(##########, ###################.#[#]);
			#########(##########, ###################.#[#]);
		}
	}
}




//********************************************
//********************************************
//********** SELECT NIC MEMORY BANK **********
//********************************************
//********************************************
//Call with nic register address with the high byte containing the 2 bank select 2 bits.
static void nic_select_bank (WORD reg)
{
	nic_bit_field_clear_register(NIC_ECON1, (ECON1_BSEL1 | ECON1_BSEL0));
	nic_bit_field_set_register(NIC_ECON1, ((WORD_VAL*)&reg)->v[1]);
}


 
//******************************
//******************************
//********** NIC READ **********
//******************************
//******************************
//Call with:-
//	5 bit address of the ETH control register to read from (the top 3 bits must be 0)
//Returns:-
//	Byte read from the Ethernet controller's ETH register.
static NIC_REG nic_read (BYTE address)
{
	NIC_REG reg;

	//SELECT NIC
	NIC_CS(0);
	
	//WRITE ADDRESS
	NIC_SPI_TX_BYTE((NIC_RCR | address));
	while(!NIC_SPI_BUF_FULL)			//Wait for SPI to complete
		;
	
	reg.val = NIC_SPI_RX_BYTE_BUFFER;

	//READ REGISTER
	#################(#);					//Send a dummy byte to read
	#####(!##################)			//Wait for SPI to complete
		;
		
	###.### = #####################;

	//DE-SELECT NIC
	######(#);

	######(###);
}

 
//*******************************
//*******************************
//********** NIC WRITE **********
//*******************************
//*******************************
static void nic_write (BYTE address, BYTE data)
{		
	BYTE b_temp;

	//SELECT THE NIC
	NIC_CS(0);
	
	//WRITE ADDRESS
	NIC_SPI_TX_BYTE((NIC_WCR | address));
	while(!NIC_SPI_BUF_FULL)					//Wait for SPI to complete
		;
		
	b_temp = NIC_SPI_RX_BYTE_BUFFER;

	//WRITE THE DATA
	NIC_SPI_TX_BYTE(data);
	#####(!################)					//Wait for SPI to complete
		;
		
	###### = #####################;

	//DESELECT THE NIC
	######(#);
}



//*********************************************
//*********************************************
//********** NIC WRITE BUFFER MEMORY **********
//*********************************************
//*********************************************
void nic_write_buffer(BYTE data)
{
	BYTE b_temp;

	//SELECT NIC
	NIC_CS(0);
	
	//WRITE COMMAND
	NIC_SPI_TX_BYTE(NIC_WBM);
	while(!NIC_SPI_BUF_FULL)				//Wait for SPI to complete
		;
		
	b_temp = NIC_SPI_RX_BYTE_BUFFER;
	
	//WRITE DATA
	NIC_SPI_TX_BYTE(data);
	#####(!################)				//Wait for SPI to complete
		;
	###### = #####################;

	//DE-SELECT NIC
	######(#);
}



//***********************************************
//***********************************************
//********** NIC READ MAC/MII REGISTER **********
//***********************************************
//***********************************************
static NIC_REG nic_read_mac_mii_register (BYTE address)
{
	NIC_REG reg;

	//SELECT NIC
	NIC_CS(0);
	
	//SEND COMMAND
	NIC_SPI_TX_BYTE((NIC_RCR | address));
	while(!NIC_SPI_BUF_FULL)					//Wait for SPI to complete
		;
		
	reg.val = NIC_SPI_RX_BYTE_BUFFER;
	
	//SENT DUMMY BYTE
	#################(####);
	#####(!##################)					//Wait for SPI to complete
		;
		
	###.### = #####################;
	
	//READ THE REGISTER
	#################(####);
	#####(!##################)					//Wait for SPI to complete
		;
		
	###.### = #####################;
	
	//DE-SELECT NIC
	######(#);
	
	######(###);
}



//**************************************************
//**************************************************
//********** NIC BIT FIELD CLEAR REGISTER **********
//**************************************************
//**************************************************
static void nic_bit_field_clear_register (BYTE address, BYTE data)
{
	BYTE b_temp;

	//SELECT NIC
	NIC_CS(0);

	//SEND COMMAND
	NIC_SPI_TX_BYTE((NIC_BFC | address));
	while(!NIC_SPI_BUF_FULL)						//Wait for SPI to complete
		;
		
	b_temp = NIC_SPI_RX_BYTE_BUFFER;
	
	//SEND DATA
	NIC_SPI_TX_BYTE(data);
	#####(!################)						//Wait for SPI to complete
		;
		
	###### = #####################;
	
	
	//DE-SELECT NIC
	######(#);
}



//************************************************
//************************************************
//********** NIC BIT FIELD SET REGISTER **********
//************************************************
//************************************************
static void nic_bit_field_set_register (BYTE address, BYTE data)
{
	BYTE b_temp;

	//SELECT NIC
	NIC_CS(0);
	
	//SEND COMMAND
	NIC_SPI_TX_BYTE((NIC_BFS | address));
	while(!NIC_SPI_BUF_FULL)						//Wait for SPI to complete
		;
		
	b_temp = NIC_SPI_RX_BYTE_BUFFER;
	
	//SEND DATA
	NIC_SPI_TX_BYTE(data);
	#####(!################)						//Wait for SPI to complete
		;
		
	###### = #####################;
	
	//DE-SELECT NIC
	######(#);
}



//*******************************************
//*******************************************
//********** NIC READ PHY REGISTER **********
//*******************************************
//*******************************************
//Performs an MII read operation.  While in progress, it simply polls the MII BUSY bit wasting time (10.24us).
PHYREG nic_read_phy_register(BYTE reg)
{
	PHYREG result;

	//Set the right address and start the register read operation
	nic_select_bank(NIC_MIREGADR);
	nic_write((BYTE)NIC_MIREGADR, reg);
	nic_write((BYTE)NIC_MICMD, MICMD_MIIRD);	

	//Loop to wait until the PHY register has been read through the MII
	//This requires 10.24us
	nic_select_bank(NIC_MISTAT);
	#####(#########################((####)##########).##########.####);

	//Stop reading
	#################(#############);
	#########((####)#########, ####);	
	
	//Obtain results and return
	######.###.#[#] = #####################((####)#########).###;
	######.###.#[#] = #####################((####)#########).###;

	#################(##########);	//Return to Bank 0
	######(######);
}



//********************************************
//********************************************
//********** NIC WRITE PHY REGISTER **********
//********************************************
//********************************************
//(Performs an MII write operation.  While in progress, it simply polls the MII BUSY bit wasting time.)
void nic_write_phy_register (BYTE reg, WORD data)
{
	//Write the register address
	nic_select_bank(NIC_MIREGADR);
	nic_write((BYTE)NIC_MIREGADR, reg);
	
	//Write the data
	nic_write((BYTE)NIC_MIWRL, ((WORD_VAL*)&data)->v[0]);			//Write low byte first
	nic_write((BYTE)NIC_MIWRH, ((WORD_VAL*)&data)->v[1]);

	//Wait until the PHY register has been written
	nic_select_bank(NIC_MISTAT);
	#####(#########################((####)##########).##########.####);

	#################(##########);	// Return to Bank 0
}





