//****************************************************************************************
//****************************************************************************************
//  Project Name:		TCP/IP DRIVER
//						POP3 (POST OFFICE PROTOCOL) C CODE FILE
//  Copyright:			EMBEDDED-CODE.COM
//
//  Licenced To:        NONE
//  Licence Number:     NONE
//
//  IMPORTANT: These files are copyright of embedded-code.com and are subject to a licence
//  agreement.  All rights reserved.  Unauthorised use, reproduction or distribution of
//  these files may lead to prosecution.
//  Any use in violation of the licence restrictions may subject the user to criminal
//  sanctions under applicable laws, as well as to civil liability for the breach of the
//  terms and conditions of the license.
//  This software is provided in an "as is" condition. No warranties, whether express,
//  implied or statutory, including, but not limited to, implied warranties of
//  merchantability and fitness for a particular purpose apply to this software. 
//  Embedded-code.com shall not be liable in any circumstances for special, incidental
//  or consequential damages, for any reason whatsoever.
//  Please see www.embedded-code.com/licence.php for full details.
//  This file contains hidden markers to identify it uniquely to the licence number it was
//  purchased under.
//  DO NOT REMOVE THIS NOTICE - IT IS A REQUIREMENT OF THE LICENCE AGREEMENT.
//****************************************************************************************
//****************************************************************************************


#include "main.h"
#define	ETH_POP3_C
#include "eth-pop3.h"
#undef	ETH_POP3_C

#include "eth-main.h"
#include "eth-tcp.h"
#include "eth-arp.h"
#include "eth-nic.h"

#ifdef STACK_USE_DNS
#include "eth-dns.h"
#endif

#ifdef STACK_USE_SMTP
#include "eth-smtp.h"
#endif


#include "ap-main.h"			//Your application file that includes the definition for the function used to process
								//received pop3 emails and the variables for the POP3 server strings if used


//--------------------------------------------------------------------------
//----- GENERATE COMPILE ERROR IF POP3 HAS NOT BEEN DEFINED TO BE USED -----
//--------------------------------------------------------------------------
#ifndef STACK_USE_POP3
#error POP3 file is included in project but not defined to be used - remove file from project to reduce code size.
#endif






//*******************************
//*******************************
//********** GET EMAIL **********
//*******************************
//*******************************
//Returns:-
//	0 = unable to start process at the current time
//	1 = mail receive process started
BYTE email_start_receive (void)
{
	
	//----- CHECK WE ARE CONNECTED -----
	if (!nic_linked_and_ip_address_valid)
		return(0);

	//----- DON'T GET EMAIL WHILE WE'RE SENDING EMAIL -----
	#ifdef STACK_USE_SMTP
		if (email_is_send_active())
			return(0);
	######

	//----- CHECK WE'RE NOT ALREADY RECEIVING EMAIL -----
	## (#####################())
		######(#);

	//----- START RECEIVE EMAIL PROCESS -----
	##################### = #####################;
	####### = ###################;						//Trigger state machine to do the get email process

	//UPDATE USER DISPLAYED STRING IF IN USE
	###### #####################
		##################### = #####################;
		##################### = #;
	######

	######(#);
}



//********************************************************
//********************************************************
//********** ARE WE CURRRENTLY RECEIVING EMAIL? **********
//********************************************************
//********************************************************
//Returns 1 is receive is active, 0 if not
BYTE email_is_receive_active (void)
{

	if (sm_pop3 == SM_POP3_IDLE)
		return(0);
	else
		return(1);
}



//***************************************
//***************************************
//********** PROCESS GET EMAIL **********
//***************************************
//***************************************
//This function is called reguarly by tcp_ip_process_stack
void email_process_pop3 (void)
{
	static WORD number_of_emails_in_pop3_box;
	static WORD get_emails_next_email_number;
	static BYTE pop3_10ms_clock_timer_last;
	static BYTE get_email_all_fields_done;
	BYTE received_byte;
	BYTE loop_count;
	BYTE temp_string[64];
	BYTE b_temp;
	BYTE *string_pointer;
	IP_ADDR dns_resolved_ip_address;


	//-----------------------------------
	//----- CHECK FOR UPDATE TIMERS -----
	//-----------------------------------
	if ((BYTE)((BYTE)(ethernet_10ms_clock_timer & 0x000000ff) - pop3_10ms_clock_timer_last) >= 10)
	{
		pop3_10ms_clock_timer_last = (BYTE)(ethernet_10ms_clock_timer & 0x000000ff);
		
		//OUR TIMEOUT TIMER
		if (pop3_100ms_timeout_timer)
			pop3_100ms_timeout_timer--;
	}

	
	## ((####### != #############) && (####### != ###############))
	{
		//----------------------------------------------------
		//----- RECEIVE IS ACTIVE - DO BACKGROUND CHECKS -----
		//----------------------------------------------------
		
		//CHECK FOR RESPONSE TIMEOUT
		## (##################### == #)
			#### #####################;
		
		//CHECK FOR LOST ETHERNET CONNECTION
		## (!##############)
			#### #####################;
	}

	

	######(#######)
	{
	#### #############:
		//----------------
		//----------------
		//----- IDLE -----
		//----------------
		//----------------

		#####;


	#### ###################:
		//---------------------
		//---------------------
		//----- GET EMAIL -----
		//---------------------
		//---------------------

		//GET THE IP ADDRESS OF THE POP3 SERVER
		#####################(&###########[#], ######(###########));

		## (!#############(###########, #####################))
		{
			//DNS query not currently available - probably already doing a query - try again next time
			#####;
		}
		
		//WAIT FOR DNS TO COMPLETE OUR QUERY
		####### = #####################;
		##################### = #####################;

		//UPDATE USER DISPLAYED STRING IF IN USE
		###### #####################
			##################### = #####################;
			##################### = #;
		######

		#####;


	#### #####################:
		//------------------------------------------------
		//------------------------------------------------
		//----- WAITING FOR POP3 SERVER DNS RESPONSE -----
		//------------------------------------------------
		//------------------------------------------------

		//SEE IF DNS RESPONSE HAS BEEN RECEIVED
		##################### = ####################();

		## (#####################.### == ##########)
		{
			//----- DNS QUERY FAILED -----
			####### = ###############;
			#####;
		}

		## (#####################.###)
		{
			//----- DNS QUERY SUCESSFUL -----
			//Store the IP address
			##################.##########.### = #####################.###;

			####### = #####################;
		}
		#####;	
			

	#### #####################:
		//------------------------------------------------------------
		//------------------------------------------------------------
		//----- SEND ARP REQUEST FOR THE DNS RETURNED IP ADDRESS -----
		//------------------------------------------------------------
		//------------------------------------------------------------
		
		//Do ARP query to get the MAC address of the server or our gateway
		## (!#####################(&##################.##########))
		{
			//ARP request cannot be sent right now - try again next time
			#####;
		}
		##################### = #####################;
		####### = #####################;

		//UPDATE USER DISPLAYED STRING IF IN USE
		###### #####################
			##################### = #####################;
			##################### = #;
		######

		#####;


	#### #####################:
		//---------------------------------
		//---------------------------------
		//----- WAIT FOR ARP RESPONSE -----
		//---------------------------------
		//---------------------------------
		//Wait for the ARP response
		## (#####################(&##################.##########, &##################.###########))
		{
			//----- ARP RESPONSE RECEVIED -----
			####### = #####################;
		}
		#####;


	#### #####################:
		//-------------------------------
		//-------------------------------
		//----- OPEN TCP CONNECTION -----
		//-------------------------------
		//-------------------------------

		//----- CREATE A TCP CONNECTION TO THE POP3 SERVER AND WAIT FOR THE GREETING MESSAGE -----
		## (################# != ####################)			//We shouldn't have a socket currently, but make sure
			##################(#################);

		################# = ####################(&##################, ##################);
		
		## (################# == ####################)
		{
			//No TCP sockets currently available - try again next time
			#####;
		}

		##################### = #####################;
		####### = #####################;

		//UPDATE USER DISPLAYED STRING IF IN USE
		###### #####################
			##################### = #####################;
			##################### = #;
		######

		#####;


	#### #####################:
		//--------------------------------------
		//--------------------------------------
		//----- WAITING FOR TCP CONNECTION -----
		//--------------------------------------
		//--------------------------------------
		//Has connection been made?
		## (#####################(#################))
		{
			####### = #####################;
			##################### = #####################;

			//UPDATE USER DISPLAYED STRING IF IN USE
			###### #####################
				##################### = #####################;
				##################### = #;
			######
		}

		#####;


	#### #####################:
		//-----------------------------------------------------------
		//-----------------------------------------------------------
		//----- WAITING FOR POP3 SERVER GREETING TO BE RECEIVED -----
		//-----------------------------------------------------------
		//-----------------------------------------------------------
		
		//Check for packet received
		## (#####################(#################))
		{
			#####################(&##############);
			## (############## == '+')
			{
				//RESPONSE IS '+OK'
				####################();				//Dump the remainder of the recevied packet

				####### = #####################;
			}
			####
			{
				//RESPONSE IS NOT '+OK' - ERROR
				#### #####################;
			}
	    }
		#####;


	#### #####################:
		//------------------------------
		//------------------------------
		//----- SEND POP3 USERNAME -----
		//------------------------------
		//------------------------------
		## (!#####################(#################))
		{
			//Can't send currently - try again next time
			#####;
		}
		//Send the username
		#####################('#');
		#####################('#');
		#####################('#');
		#####################('#');
		#####################(' ');

		//Get the pop3 username
		#####################(&###########[#], ######(###########));

		### (########## = #; ###########[##########] != ####; ##########++)
			#####################(###########[##########]);	

		#####################('\#');		//CR
		#####################('\#');		//LF

		#####################(#################);				//Send the packet

		####### = #####################;
		##################### = #####################;

		//UPDATE USER DISPLAYED STRING IF IN USE
		###### #####################
			##################### = #####################;
			##################### = #;
		######

		#####;


	#### #####################:
		//--------------------------------------------
		//--------------------------------------------
		//----- WAITING FOR RESPONSE TO USERNAME -----
		//--------------------------------------------
		//--------------------------------------------

		//DO WE NEED TO RE-SEND THE LAST PACKET?
		//(TCP requires resending of packets if they are not acknowledged and to avoid requiring a large RAM buffer the application needs to remember the last packet sent on a socket
		//so it can be resent if requried - If your application has ram available to store a copy of the last sent packet then the tcp driver could be modified to use it instead).
		## (#####################(#################))
		{
			####### = #####################;
			#####;
		}

		//CHECK FOR PACKET RECEIVED
		## (#####################(#################))
		{
			#####################(&##############);
			## (############## == '+')
			{
				//RESPONSE IS '+OK'
				####################();				//Dump the remainder of the received packet

				####### = #####################;
			}
			####
			{
				//RESPONSE IS NOT '+OK' - ERROR
				#### #####################;
			}
	    }
		#####;


	#### #####################:
		//------------------------------
		//------------------------------
		//----- SEND POP3 PASSWORD -----
		//------------------------------
		//------------------------------
		## (!#####################(#################))
		{
			//Can't send currently - try again next time
			#####;
		}

		//Send the username
		#####################('#');
		#####################('#');
		#####################('#');
		#####################('#');
		#####################(' ');

		//Get the pop3 password
		#####################(&###########[#], ######(###########));

		### (########## = #; ###########[##########] != ####; ##########++)
			#####################(###########[##########]);	

		#####################('\#');		//CR
		#####################('\#');		//LF

		#####################(#################);				//Send the packet

		####### = #####################;
		##################### = #####################;

		//UPDATE USER DISPLAYED STRING IF IN USE
		###### #####################
			##################### = #####################;
			##################### = #;
		######

		#####;


	#### #####################:
		//--------------------------------------------
		//--------------------------------------------
		//----- WAITING FOR RESPONSE TO PASSWORD -----
		//--------------------------------------------
		//--------------------------------------------

		//DO WE NEED TO RE-SEND THE LAST PACKET?
		## (#####################(#################))
		{
			####### = #####################;
			#####;
		}

		//CHECK FOR PACKET RECEIVED
		## (#####################(#################))
		{
			#####################(&##############);
			## (############## == '+')
			{
				//RESPONSE IS '+OK' - SEND STAT TO GET NUMBER OF EMAILS IN THE POP3 BOX
				####################();				//Dump the remainder of the recevied packet

				####### = #####################;
			}
			####
			{
				//RESPONSE IS NOT '+OK' - ERROR
				#### #####################;
			}
	    }
		#####;


	#### #####################:
		//----------------------------------
		//----------------------------------
		//----- SEND POP3 STAT REQUEST -----
		//----------------------------------
		//----------------------------------
		## (!#####################(#################))
		{
			//Can't send currently - try again next time
			#####;
		}

		//Send the command
		#####################('#');
		#####################('#');
		#####################('#');
		#####################('#');

		#####################('\#');		//CR
		#####################('\#');		//LF

		#####################(#################);				//Send the packet

		####### = #####################;
		##################### = #####################;

		//UPDATE USER DISPLAYED STRING IF IN USE
		###### #####################
			##################### = #####################;
			##################### = #;
		######

		#####;


	#### #####################:
		//--------------------------------------------------
		//--------------------------------------------------
		//----- WAITING FOR RESPONSE TO 'STAT' REQUEST -----
		//--------------------------------------------------
		//--------------------------------------------------

		//DO WE NEED TO RE-SEND THE LAST PACKET?
		## (#####################(#################))
		{
			####### = #####################;
			#####;
		}

		//CHECK FOR PACKET RECEIVED
		## (#####################(#################))
		{
			//Check for '+OK ' at start of line
			#####################(&##############);
			## (############## != '+')
				#### #####################;
			#####################(&##############);
			## (############## != '#')
				#### #####################;
			#####################(&##############);
			## (############## != '#')
				#### #####################;
			#####################(&##############);
			## (############## != ' ')
				#### #####################;

			//Get the number of messages waiting (up to 4 digits)
			
			#####################(&##############);
			###########[#] = ##############;
			###########[#] = ####;
			###########[#] = ####;
			###########[#] = ####;
			###########[#] = ####;

			#####################(&##############);
			## (############## != ' ')
			{
				###########[#] = ##############;

				#####################(&##############);
				## (############## != ' ')
				{
					###########[#] = ##############;

					#####################(&##############);
					## (############## != ' ')
					{
						###########[#] = ##############;
					}
				}
			}

			##################### = #####################(&###########[#]);

			####################();				//Dump the remainder of the recevied packet

			##################### = #;
			####### = #####################;
		}

		#####;


	#### #####################:
		//------------------------------
		//------------------------------
		//----- REQUEST NEXT EMAIL -----
		//------------------------------
		//------------------------------
		## (##################### > #####################)
		{
			//----- ALL EMAILS RETREIVED - QUIT -----
			####### = #################;
			#####;
		}

		//----- GET NEXT EMAIL -----
		## (!#####################(#################))
		{
			//Can't send currently - try again next time
			#####;
		}

		//Send the command
		#####################('#');
		#####################('#');
		#####################('#');
		#####################('#');
		#####################(' ');
		
		//Send the message number
		#####################(#####################, &###########[#]);
		### (########## = #; ########## < #; ##########++)
		{
			## (###########[##########] != ####)
				#####################(###########[##########]);
			####
				#####;
		}
		#####################('\#');		//CR
		#####################('\#');		//LF

		#####################(#################);				//Send the packet

		#####################++;
        ##################### = #;
        ##################### = #;
		##################### = #####################;
		####### = #####################;

		//UPDATE USER DISPLAYED STRING IF IN USE
		###### #####################
			##################### = #####################;
			##################### = #;
		######

		#####;


	#### #####################:
		//-----------------------------------------------------
		//-----------------------------------------------------
		//----- WAITING FOR RESPONSE TO GET EMAIL REQUEST -----
		//-----------------------------------------------------
		//-----------------------------------------------------

		//DO WE NEED TO RE-SEND THE LAST PACKET?
		## (#####################(#################))
		{
			#####################;
			####### = #####################;
			#####;
		}

		//CHECK FOR PACKET RECEIVED
		## (#####################(#################))
		{
			//Check for '+OK ' at start of line
			#####################(&##############);
			## (############## != '+')
				#### #####################;
			#####################(&##############);
			## (############## != '#')
				#### #####################;
			#####################(&##############);
			## (############## != '#')
				#### #####################;
			#####################(&##############);
			## (############## != ' ')
				#### #####################;

			//Don't dump the packet - leave it for the next state to process as this packet may contain lines from the header
			####### = #####################;
		}

		//break;	//Fall into next state


	#### #####################:
		//-------------------------------------
		//-------------------------------------
		//----- RECEIVE THE CURRENT EMAIL -----
		//-------------------------------------
		//-------------------------------------
		//Check for packet received
		## (#####################(#################))
		{
			//Remember that the response may span over more than 1 packet, especially if the header is large due to routing
			
			##### (#####################(&##############))			//Get next byte, returns 0 if end of packet
			{
				//----- RECEIVE EACH LINE -----
			    ## (############## != '\#')						//Do until we get the CR character			
			    {
				    //Store each character, truncating any lines that don't fit in our string array
			        ## ((##################### < (##################### # #)) && (############## != '\#'))		//Don't include the line feed character
						#####################[#####################++] = ##############;
			    }
			    ####
			    {
				    //----------------------------------------------------
				    //----- CR CHARACTER RECEIVED - PROCESS THE LINE -----
				    //----------------------------------------------------
			        #####################[#####################] = ####;			//Add null termination to the string

					//----- LOOK FOR THE 'FROM' FIELD -----
					## (#####################(&#####################[#], &#####################[#]))	//Does this line contain this string?
					{
						//--------------------------------
						//----- 'FROM' EMAIL ADDRESS -----
						//--------------------------------
						############### = #####################(&#####################[#], '<');
						## (############### != #)
						{
							###############++;
							### (###### = #; ###### < (##################### # #); ######++)
							{
								## (*############### == '>')
								{
									//ALL OF REPLY EMAIL ADDRESS HAS BEEN COPIED
									#####################[######] = ####;					//Add null termination

									##################### |= ####;						//Flag that we have the reply address
									#####;
								}
								//Store next character
								#####################[######] = *###############++;
							}
						}
					}

					//----- LOOK FOR THE 'SUBJECT' FIELD -----
					############### = #####################(&#####################[#], &#####################[#]);	//Does this line contain this string?
					## (###############)
					{
						//-------------------------
						//----- EMAIL SUBJECT -----
						//-------------------------
						
						//Pass to user application to process
						#####################(#, ############### + ######(#####################), &#####################[#]);
						##################### |= ####;						//Flag that we have the subject
					}
					//----- LOOK FOR THE END OF HEADERS MARKER -----
					## ((##################### == #))
					{
						//---------------------------------
						//----- END OF HEADERS MARKER -----
						//---------------------------------
						##################### |= ####;						//Flag that we have moved to the email body
					}
					//----- LOOK FOR EMAIL BODY -----
					#### ## (##################### == ####)
					{
						//-----------------------------------
						//----- NEXT LINE OF EMAIL BODY -----
						//-----------------------------------

						//Pass to user application to process
						#####################(#, &#####################[#], &#####################[#]);
					}

					//----- LOOK FOR THE END OF MESSAGE MARKER -----			        
					## ((##################### == #) && (#####################[#] == '.'))
					{
						//---------------------------------
						//----- END OF MESSAGE MARKER -----
						//---------------------------------

						//Flag end of email to user application
						## (#####################(#, ####, &#####################[#]))
						{
							//Delete email, which will return to the getting of the next email
							####### = ###################;
						}
						####
						{
							//User does not want this email deleted
							####### = #####################;
						}
					}

			        //Reset ready for the next line
			        ##################### = #;
			    }
			}

			####################();				//Dump the remainder of the recevied packet

		}

		#####;


	#### ###################:
		//------------------------------------
		//------------------------------------
		//----- DELETE THE CURRENT EMAIL -----
		//------------------------------------
		//------------------------------------
		//----- GET NEXT EMAIL HEADER -----
		## (!#####################(#################))
		{
			//Can't send currently - try again next time
			#####;
		}
		//Send the command
		#####################('#');
		#####################('#');
		#####################('#');
		#####################('#');
		#####################(' ');
			
		//Send the message number
		#####################((##################### # #), &###########[#]);
		### (########## = #; ########## < #; ##########++)
		{
			## (###########[##########] != ####)
				#####################(###########[##########]);
			####
				#####;
		}

		#####################('\#');		//CR
		#####################('\#');		//LF

		#####################(#################);				//Send the packet

		##################### = #####################;
		####### = #####################;

		//UPDATE USER DISPLAYED STRING IF IN USE
		###### #####################
			##################### = #####################;
			##################### = #;
		######

		#####;


	#### #####################:
		//------------------------------------
		//------------------------------------
		//----- WAIT FOR DELETE RESPONSE -----
		//------------------------------------
		//------------------------------------

		//DO WE NEED TO RE-SEND THE LAST PACKET?
		## (#####################(#################))
		{
			####### = ###################;
			#####;
		}

		//CHECK FOR PACKET RECEIVED
		## (#####################(#################))
		{
			//Check for '+OK ' at start of line
			#####################(&##############);
			## (############## != '+')
				#### #####################;
			#####################(&##############);
			## (############## != '#')
				#### #####################;
			#####################(&##############);
			## (############## != '#')
				#### #####################;
			#####################(&##############);
			## (############## != ' ')
				#### #####################;

			####################();				//Dump the remainder of the recevied packet

			##################### = #####################;
			
			####### = #####################;
		}
		#####;


	#### #################:
		//------------------------------------------
		//------------------------------------------
		//----- QUIT CONNECTION TO POP3 SERVER -----
		//------------------------------------------
		//------------------------------------------
		## (!#####################(#################))
		{
			//Can't send currently - try again next time
			#####;
		}
		
		//Send the username
		#####################('#');
		#####################('#');
		#####################('#');
		#####################('#');

		#####################('\#');		//CR
		#####################('\#');		//LF

		#####################(#################);				//Send the packet

		####### = #####################;
		##################### = #####################;

		//UPDATE USER DISPLAYED STRING IF IN USE
		###### #####################
			##################### = #####################;
			##################### = #;
		######

		#####;


	#### #####################:
		//----------------------------------
		//----------------------------------
		//----- WAIT FOR QUIT RESPONSE -----
		//----------------------------------
		//----------------------------------

		//DO WE NEED TO RE-SEND THE LAST PACKET?
		## (#####################(#################))
		{
			####### = #################;
			#####;
		}

		//CHECK FOR PACKET RECEIVED
		## (#####################(#################))
		{
			//----- ALL DONE - DUMP THE TCP CONNECTION AND RETURN TO IDLE MODE -----
			#####################(#################);
			####### = #############;

			//UPDATE USER DISPLAYED STRING IF IN USE
			###### #####################
				##################### = #####################;
				##################### = #;
			######
		}
		#####;

	#### ###############:
		//------------------------------------
		//------------------------------------
		//----- GET EMAIL PROCESS FAILED -----
		//------------------------------------
		//------------------------------------

		//UPDATE USER DISPLAYED STRING IF IN USE
		###### #####################
			##################### = #####################;
			##################### = #;
		######

		####### = #############;

		#####;
		
	}


	######;

process_get_email_tcp_failed:
//----- FAILED DURING TCP TRANSFER - CLOSE CONNECTION AND CHANGE STATE TO ERROR -----
	if ((sm_pop3 != SM_POP3_GET_EMAIL) && (sm_pop3 != SM_POP3_WAIT_FOR_ARP_RESPONSE) && (sm_pop3 != SM_POP3_WAITING_DNS_RESPONSE))
	{
		tcp_dump_rx_packet();								//Dump the remainder of any received packet
		tcp_request_disconnect_socket(pop3_tcp_socket);		//Close the connection
	}

	sm_pop3 = SM_POP3_FAILED;

	return;

}





//********************************************
//********************************************
//********** RETURN POP3 SERVER URL **********
//********************************************
//********************************************
//(e.g. pop3.domain.com)
void email_return_pop3_url (BYTE *string_pointer, BYTE max_length)
{
	BYTE count = 0;
	#ifdef POP3_USING_CONST_ROM_SETTINGS
	CONSTANT BYTE *p_source_string;				//Using hard coded constant strings
	#else
	BYTE *p_source_string;							//Using variable strings
	#endif

	p_source_string = &POP3_SERVER_STRING[0];
	do
	{
		//Check for overflow
		## (++##### == ##########)
		{
			*###############++ = ####;
			######;
		}

		//Copy next byte
		*###############++ = *#################;
	}
	##### (#################++ != ####);
	
}



//******************************************
//******************************************
//********** RETURN POP3 USERNAME **********
//******************************************
//******************************************
void email_return_pop3_username (BYTE *string_pointer, BYTE max_length)
{
	BYTE count = 0;
	#ifdef POP3_USING_CONST_ROM_SETTINGS
	CONSTANT BYTE *p_source_string;				//Using hard coded constant strings
	#else
	BYTE *p_source_string;							//Using variable strings
	#endif

	p_source_string = &POP3_USERNAME_STRING[0];
	do
	{
		//Check for overflow
		## (++##### == ##########)
		{
			*###############++ = ####;
			######;
		}
		
		//Copy next byte
		*###############++ = *#################;
	}
	##### (#################++ != ####);
	
}



//******************************************
//******************************************
//********** RETURN POP3 PASSWORD **********
//******************************************
//******************************************
void email_return_pop3_password (BYTE *string_pointer, BYTE max_length)
{
	BYTE count = 0;
	#ifdef POP3_USING_CONST_ROM_SETTINGS
	CONSTANT BYTE *p_source_string;				//Using hard coded constant strings
	#else
	BYTE *p_source_string;							//Using variable strings
	#endif

	p_source_string = &POP3_PASSWORD_STRING[0];
	do
	{
		//Check for overflow
		## (++##### == ##########)
		{
			*###############++ = ####;
			######;
		}

		//Copy next byte
		*###############++ = *#################;
	}
	##### (#################++ != ####);
	
}









