//****************************************************************************************
//****************************************************************************************
//  Project Name:		TCP/IP DRIVER
//						SMTP (SIMPLE MAIL TRANSFER PROTOCOL) C CODE FILE
//  Copyright:			EMBEDDED-CODE.COM
//
//  Licenced To:        NONE
//  Licence Number:     NONE
//
//  IMPORTANT: These files are copyright of embedded-code.com and are subject to a licence
//  agreement.  All rights reserved.  Unauthorised use, reproduction or distribution of
//  these files may lead to prosecution.
//  Any use in violation of the licence restrictions may subject the user to criminal
//  sanctions under applicable laws, as well as to civil liability for the breach of the
//  terms and conditions of the license.
//  This software is provided in an "as is" condition. No warranties, whether express,
//  implied or statutory, including, but not limited to, implied warranties of
//  merchantability and fitness for a particular purpose apply to this software. 
//  Embedded-code.com shall not be liable in any circumstances for special, incidental
//  or consequential damages, for any reason whatsoever.
//  Please see www.embedded-code.com/licence.php for full details.
//  This file contains hidden markers to identify it uniquely to the licence number it was
//  purchased under.
//  DO NOT REMOVE THIS NOTICE - IT IS A REQUIREMENT OF THE LICENCE AGREEMENT.
//****************************************************************************************
//****************************************************************************************


#include "main.h"
#define	ETH_SMTP_C
#include "eth-smtp.h"
#undef	ETH_SMTP_C

#include "eth-main.h"
#include "eth-tcp.h"
#include "eth-arp.h"
#include "eth-nic.h"

#ifdef STACK_USE_DNS
#include "eth-dns.h"
#endif

#ifdef STACK_USE_POP3
#include "eth-pop3.h"
#endif


#include "ap-main.h"			//Your application file that includes the definition for the function used to process
								//received pop3 emails and the variables for the SMTP server strings if used


//--------------------------------------------------------------------------
//----- GENERATE COMPILE ERROR IF SMTP HAS NOT BEEN DEFINED TO BE USED -----
//--------------------------------------------------------------------------
#ifndef STACK_USE_SMTP
#error SMTP file is included in project but not defined to be used - remove file from project to reduce code size.
#endif




//********************************
//********************************
//********** SEND EMAIL **********
//********************************
//********************************
//Returns:-
//	0 = unable to start process at the current time
//	1 = mail receive process started
BYTE email_start_send (BYTE use_authenticated_login, BYTE include_file_attachment)
{	
	
	//----- CHECK WE ARE CONNECTED -----
	if (!nic_linked_and_ip_address_valid)
		return(0);

	//----- DON'T SEND EMAIL WHILE WE'RE RECEIVING EMAIL -----
	#ifdef STACK_USE_SMTP
		if (email_is_receive_active())
			return(0);
	######

	//----- CHECK WE'RE NOT ALREADY SENDING EMAIL -----
	## (#####################())
		######(#);

	//----- FLAG IF WE'RE TO DO AN AUTHENTICATED LOGIN -----
	## (#####################)
		##################### = #;			//1 = use SMTP authenticated login (username and password requried), 0 = normal login
	####
		##################### = #;

	//----- FLAG IF WE'RE GOING TO INCLUDE A FILE ATTACHMENT -----
	## (#####################)
		##################### = #;			//1 = use SMTP authenticated login (username and password requried), 0 = normal login
	####
		##################### = #;


	//----- START SEND EMAIL PROCESS -----
	##################### = ######;
	#################### = #####################;
	##################### = #####################;
	####### = ####################;						//Trigger state machine to do the send email process

	//UPDATE USER DISPLAYED STRING IF IN USE
	###### #####################
		##################### = #####################;
		##################### = #;
	######

	######(#);
}



//******************************************************
//******************************************************
//********** ARE WE CURRRENTLY SENDING EMAIL? **********
//******************************************************
//******************************************************
//Returns 1 is receive is active, 0 if not
BYTE email_is_send_active (void)
{
	if (sm_smtp == SM_SMTP_IDLE)
		return(0);
	else
		return(1);
}



//****************************************
//****************************************
//********** PROCESS SEND EMAIL **********
//****************************************
//****************************************
//This function is called reguarly by tcp_ip_process_stack
void email_process_smtp (void)
{
	static BYTE smtp_10ms_clock_timer_last;
	BYTE received_byte;
	BYTE loop_count;
	WORD count;
	BYTE b_temp;
	BYTE b_temp1;
	BYTE base64_source[3];
	BYTE base64_converted[4];
	BYTE temp_string[64];
	IP_ADDR dns_resolved_ip_address;
	BYTE resend_flag;



	//-----------------------------------
	//----- CHECK FOR UPDATE TIMERS -----
	//-----------------------------------
	if ((BYTE)((BYTE)(ethernet_10ms_clock_timer & 0x000000ff) - smtp_10ms_clock_timer_last) >= 10)
	{
		smtp_10ms_clock_timer_last = (BYTE)(ethernet_10ms_clock_timer & 0x000000ff);
		
		//OUT TIMEOUT TIMER
		if (smtp_100ms_timeout_timer)
			smtp_100ms_timeout_timer--;
	}

	## ((####### != #############) && (####### != ###############))
	{
		//-------------------------------------------------
		//----- SEND IS ACTIVE - DO BACKGROUND CHECKS -----
		//-------------------------------------------------
		
		//CHECK FOR RESPONSE TIMEOUT
		## (##################### == #)
			#### #####################;

		//CHECK FOR LOST ETHERNET CONNECTION
		## (!##############)
			#### #####################;
	}

	######(#######)
	{
	#### #############:
		//----------------
		//----------------
		//----- IDLE -----
		//----------------
		//----------------
		
		#####;


	#### ####################:
		//----------------------
		//----------------------
		//----- SEND EMAIL -----
		//----------------------
		//----------------------

		//GET THE IP ADDRESS OF THE SMTP SERVER
		#####################(&###########[#], ######(###########));

		## (!#############(###########, #####################))
		{
			//DNS query not available - try again next time
			#####;
		}

		####### = #####################;
		##################### = #####################;

		//UPDATE USER DISPLAYED STRING IF IN USE
		###### #####################
			##################### = #####################;
			##################### = #;
		######

		#####;


	#### #####################:
		//------------------------------------------------
		//------------------------------------------------
		//----- WAITING FOR SMTP SERVER DNS RESPONSE -----
		//------------------------------------------------
		//------------------------------------------------

		//SEE IF DNS RESPONSE HAS BEEN RECEIVED
		##################### = ####################();

		## (#####################.### == ##########)
		{
			//----- DNS QUERY FAILED -----
			####### = ###############;
			#####;
		}

		## (#####################.###)
		{
			//----- DNS QUERY SUCESSFUL -----
			
			//Store the IP address
			##################.##########.### = #####################.###;
			
			####### = #####################;
		}
		
		#####;


	#### #####################:
		//------------------------------------------------------------
		//------------------------------------------------------------
		//----- SEND ARP REQUEST FOR THE DNS RETURNED IP ADDRESS -----
		//------------------------------------------------------------
		//------------------------------------------------------------

		//Do ARP query to get the MAC address of the server or our gateway
		## (!#####################(&##################.##########))
		{
			//ARP request cannot be sent right now - try again next time
			#####;
		}
	
		//ARP REQUEST WAS SENT
		##################### = #####################;
		####### = #####################;

		//UPDATE USER DISPLAYED STRING IF IN USE
		###### #####################
			##################### = #####################;
			##################### = #;
		######
		
		#####;


	#### #####################:
		//---------------------------------
		//---------------------------------
		//----- WAIT FOR ARP RESPONSE -----
		//---------------------------------
		//---------------------------------
		//Wait for the ARP response
		## (#####################(&##################.##########, &##################.###########))
		{
			//----- ARP RESPONSE RECEVIED -----
			####### = #####################;
		}

		#####;


	#### #####################:
		//-------------------------------
		//-------------------------------
		//----- OPEN TCP CONNECTION -----
		//-------------------------------
		//-------------------------------

		//CREATE A TCP CONNECTION TO THE SMTP SERVER AND WAIT FOR THE GREETING MESSAGE
		## (################# != ####################)			//We shouldn't have a socket currently, but make sure
			##################(#################);
		
		################# = ####################(&##################, ##################);
		
		## (################# == ####################)
		{
			//No socket available - try again next time
			#####;
		}

		##################### = #####################;
		####### = #####################;

		//UPDATE USER DISPLAYED STRING IF IN USE
		###### #####################
			##################### = #####################;
			##################### = #;
		######

		#####;


	#### #####################:
		//--------------------------------------
		//--------------------------------------
		//----- WAITING FOR TCP CONNECTION -----
		//--------------------------------------
		//--------------------------------------

		//Has connection been made?
		## (#####################(#################))
		{
			####### = #####################;
			##################### = #####################;

			//UPDATE USER DISPLAYED STRING IF IN USE
			###### #####################
				##################### = #####################;
				##################### = #;
			######

		}

		#####;


	#### #####################:
		//-----------------------------------------------------------
		//-----------------------------------------------------------
		//----- WAITING FOR SMTP SERVER GREETING TO BE RECEIVED -----
		//-----------------------------------------------------------
		//-----------------------------------------------------------
		//Check for packet received
		## (#####################(#################))
		{
			#####################(&##############);
			## (############## == '#')				//A leading 2 character indicates the server is happy (3 used for data transfers, 4 & 5 = error)
			{
				//RESPONSE IS GOOD
				####################();				//Dump the remainder of the recevied packet

				####### = #####################;
				
			}
			####
			{
				//RESPONSE IS NOT GOOD - ERROR
				#### #####################;
			}
	    }

		#####;


	#### #####################:
		//---------------------
		//---------------------
		//----- SEND HELO -----
		//---------------------
		//---------------------
		## (!#####################(#################))
		{
			//Can't send currently - try again next time
			#####;
		}
		
		//Send the hello
		## (#####################)
		{
			//Special command for authenticated login
			#####################('#');
			#####################('#');
		}
		####
		{
			//Normal command
			#####################('#');
			#####################('#');
		}
		#####################('#');
		#####################('#');
		#####################(' ');
		#####################('#');		//EMB is our computer name (can be anything)
		#####################('#');
		#####################('#');

		#####################('\#');		//CR
		#####################('\#');		//LF

		#####################(#################);			//Send the packet

		####### = #####################;
		##################### = #####################;

		//Reset ready for the next line of response
		##################### = #;

		//UPDATE USER DISPLAYED STRING IF IN USE
		###### #####################
			##################### = #####################;
			##################### = #;
		######

		#####;


	#### #####################:
		//----------------------------------------
		//----------------------------------------
		//----- WAITING FOR RESPONSE TO HELO -----
		//----------------------------------------
		//----------------------------------------

		//DO WE NEED TO RE-SEND THE LAST PACKET?
		//(TCP requires resending of packets if they are not acknowledged and to avoid requiring a large RAM buffer the application needs to remember the last packet sent on a socket
		//so it can be resent if requried - If your application has ram available to store a copy of the last sent packet then the tcp driver could be modified to use it instead).
		## (#####################(#################))
		{
			####### = #####################;
			#####;
		}

		//CHECK FOR PACKET RECEIVED
		## (#####################(#################))
		{
			//----- PACKET RECEIVED -----
			##### (#####################(&##############))			//Get next byte, returns 0 if end of packet
			{
				//----- RECEIVE EACH LINE -----
				//(EHLO EXTENDED SMTP responses are multiline)
			    ## (############## != '\#')							//Do until we get the CR character			
			    {
			        ## (##################### < ##################### # # && ############## != '\#')		//Don't include the line feed character
						#####################[#####################++] = ##############;
			    }
			    ####
			    {
				    //----------------------------------------------------
				    //----- CR CHARACTER RECEIVED - PROCESS THE LINE -----
				    //----------------------------------------------------
					//receive_message_string[receive_message_string_len] = 0x00;			//Add null termination to the string
					//strlwr(&receive_message_string[0]);									//Convert string to lower case

					## (#####################[#] == '#')				//A leading 2 character indicates the server is happy (3 used for data transfers, 4 & 5 = error)
					{
						//RESPONSE IS GOOD
						## (#####################)
						{
							//----- USING EXTENDED SMTP - CHECK FOR FURTHER RESPONSE DUE -----
							## (#####################[#] == '#')				//A trailing '-' instead of a space indicates that another response will follow
							{
								//GET ADDITIONAL RESPONSE(S)
								//(Either in this packet or in additional [ackets)
			
						        //Reset ready for the next line
						        ##################### = #;
			
								########;
							}
						}
				
						//----- IF NOT DOING AUTHENTICATED LOGIN THEN SETUP TO START SENDING EMAIL -----
						//----- IF DOING AUTHENTICATED LOGIN THEN SETUP TO DO LOGIN -----
						## (#####################)
						{
							//DOING AUTHENTICATED LOGIN
							####### = #####################;
						}
						####
						{
							//DOING NORMAL NON-AUTHENTICATED LOGIN
							####### = #####################;
						}
						
					}
					####
					{
						//----- RESPONSE IS NOT GOOD - ERROR -----
						#### #####################;
					}

			        //Reset ready for the next line
			        ##################### = #;
			    }

			}
			//Packet processed - dump it
			####################();
	    }

		#####;



	#### #####################:
		//------------------------------------------------------
		//------------------------------------------------------
		//----- SEND MAIL SMTP AUTHENTICATED LOGIN COMMAND -----
		//------------------------------------------------------
		//------------------------------------------------------
		//(We only support AUTH LOGIN - other AUTH methods are available and a server is not requried to support all types, however AUTH LOGIN is typical)
		## (!#####################(#################))
		{
			//Can't send currently - try again next time
			#####;
		}

		//Send MAIL FROM
		#####################('#');
		#####################('#');
		#####################('#');
		#####################('#');
		#####################(' ');
		#####################('#');
		#####################('#');
		#####################('#');
		#####################('#');
		#####################('#');
		#####################('\#');		//CR
		#####################('\#');		//LF

		#####################(#################);				//Send the packet

		####### = #####################;
		##################### = #####################;

		//UPDATE USER DISPLAYED STRING IF IN USE
		###### #####################
			##################### = #####################;
			##################### = #;
		######

		#####;


	#### #####################:
		//------------------------------------------------------
		//------------------------------------------------------
		//----- WAITING FOR RESPONSE TO AUTH LOGIN COMMAND -----
		//------------------------------------------------------
		//------------------------------------------------------

		//DO WE NEED TO RE-SEND THE LAST PACKET?
		## (#####################(#################))
		{
			####### = #####################;
			#####;
		}

		//CHECK FOR PACKET RECEIVED
		## (#####################(#################))
		{
			#####################(&##############);
			## (############## == '#')				//A leading 3 character indicates the server is in data transfer mode, 4 & 5 = error)
			{
				//RESPONSE IS GOOD - SEND THE LOGIN USERNAME
				####################();				//Dump the remainder of the recevied packet

				####### = #####################;
			}
			####
			{
				//RESPONSE IS NOT GOOD - ERROR
				#### #####################;
			}
	    }

		#####;


	#### #####################:
		//------------------------------
		//------------------------------
		//----- SEND AUTH USERNAME -----
		//------------------------------
		//------------------------------
		## (!#####################(#################))
		{
			//Can't send currently - try again next time
			#####;
		}

		//Get the username
		#####################(&###########[#], ######(###########));

		//Write the string
		###### = #;
		##### (###### < ##)
		{
			//Write the next 3 bytes as Base64 encoded
			## (###########[######] != #)
			{
				##############[#] = ###########[######++];
				####### = #;
			}
			####
			{
				#####;			//All characters done
			}

			## (###########[######] != #)
			{
				##############[#] = ###########[######++];
				####### = #;
			}

			## (###########[######] != #)
			{
				##############[#] = ###########[######++];
				####### = #;
			}

			//Convert to 4 bytes of base 64
			#####################(&##############[#], &##################[#], #######);

			#####################(##################[#]);
			#####################(##################[#]);
			#####################(##################[#]);
			#####################(##################[#]);
			## (####### < #)
				#####;			//All characters done
		}

		#####################('\#');		//CR
		#####################('\#');		//LF

		#####################(#################);				//Send the packet

		####### = #####################;
		##################### = #####################;

		//UPDATE USER DISPLAYED STRING IF IN USE
		###### #####################
			##################### = #####################;
			##################### = #;
		######

		#####;


	#### #####################:
		//---------------------------------------------------------
		//---------------------------------------------------------
		//----- WAITING FOR RESPONSE TO AUTH USERNAME COMMAND -----
		//---------------------------------------------------------
		//---------------------------------------------------------

		//DO WE NEED TO RE-SEND THE LAST PACKET?
		## (#####################(#################))
		{
			####### = #####################;
			#####;
		}

		//CHECK FOR PACKET RECEIVED
		## (#####################(#################))
		{
			#####################(&##############);
			## (############## == '#')				//A leading 3 character indicates the server is in data transfer mode, 4 & 5 = error)
			{
				//RESPONSE IS GOOD - SEND THE LOGIN PASSWORD
				####################();				//Dump the remainder of the recevied packet

				####### = #####################;
			}
			####
			{
				//RESPONSE IS NOT GOOD - ERROR
				#### #####################;
			}
	    }

		#####;


	#### #####################:
		//------------------------------
		//------------------------------
		//----- SEND AUTH PASSWORD -----
		//------------------------------
		//------------------------------
		## (!#####################(#################))
		{
			//Can't send currently - try again next time
			#####;
		}

		//Get the password
		#####################(&###########[#], ######(###########));
		
		//Write the string
		###### = #;
		##### (###### < ##)
		{
			//Write the next 3 bytes as Base64 encoded
			## (###########[######] != #)
			{
				##############[#] = ###########[######++];
				####### = #;
			}
			####
			{
				#####;			//All characters done
			}
			
			## (###########[######] != #)
			{
				##############[#] = ###########[######++];
				####### = #;
			}
			
			## (###########[######] != #)
			{
				##############[#] = ###########[######++];
				####### = #;
			}

			//Convert to 4 bytes of base 64
			#####################(&##############[#], &##################[#], #######);

			#####################(##################[#]);
			#####################(##################[#]);
			#####################(##################[#]);
			#####################(##################[#]);
			## (####### < #)
				#####;			//All characters done
		}

		#####################('\#');		//CR
		#####################('\#');		//LF

		#####################(#################);				//Send the packet

		####### = #####################;
		##################### = #####################;

		//UPDATE USER DISPLAYED STRING IF IN USE
		###### #####################
			##################### = #####################;
			##################### = #;
		######

		#####;


	#### #####################:
		//---------------------------------------------------------
		//---------------------------------------------------------
		//----- WAITING FOR RESPONSE TO AUTH PASSWORD COMMAND -----
		//---------------------------------------------------------
		//---------------------------------------------------------

		//DO WE NEED TO RE-SEND THE LAST PACKET?
		## (#####################(#################))
		{
			####### = #####################;
			#####;
		}

		//CHECK FOR PACKET RECEIVED
		## (#####################(#################))
		{
			#####################(&##############);
			## (############## == '#')				//A leading 2 character indicates the server is happy (3 used for data transfers, 4 & 5 = error)
			{
				//RESPONSE IS GOOD
				####################();				//Dump the remainder of the recevied packet
	
				//Login complete - now send email as normal
				####### = #####################;
			}
			####
			{
				//RESPONSE IS NOT GOOD - ERROR
				#### #####################;
			}
	    }

		#####;


	#### #####################:
		//----------------------------------
		//----------------------------------
		//----- SEND MAIL FROM COMMAND -----
		//----------------------------------
		//----------------------------------
		## (!#####################(#################))
		{
			//Can't send currently - try again next time
			#####;
		}

		//Send MAIL FROM
		#####################('#');
		#####################('#');
		#####################('#');
		#####################('#');
		#####################(' ');
		#####################('#');
		#####################('#');
		#####################('#');
		#####################('#');
		#####################(':');
		#####################(' ');
		#####################('<');

		//Get the from address
		#####################(&###########[#], ######(###########));

		### (########## = #; ###########[##########] != ####; ##########++)
		{
			#####################(###########[##########]);	
		}

		#####################('>');
		#####################('\#');		//CR
		#####################('\#');		//LF

		#####################(#################);				//Send the packet

		####### = #####################;
		##################### = #####################;

		//UPDATE USER DISPLAYED STRING IF IN USE
		###### #####################
			##################### = #####################;
			##################### = #;
		######

		#####;


	#### #####################:
		//---------------------------------------------
		//---------------------------------------------
		//----- WAITING FOR RESPONSE TO MAIL FROM -----
		//---------------------------------------------
		//---------------------------------------------

		//DO WE NEED TO RE-SEND THE LAST PACKET?
		## (#####################(#################))
		{
			####### = #####################;
			#####;
		}

		//CHECK FOR PACKET RECEIVED
		## (#####################(#################))
		{
			#####################(&##############);
			## (############## == '#')				//A leading 2 character indicates the server is happy (3 used for data transfers, 4 & 5 = error)
			{
				//RESPONSE IS GOOD - SEND THE RCPT TO FIELD
				####################();				//Dump the remainder of the recevied packet

				####### = #####################;
			}
			####
			{
				//RESPONSE IS NOT GOOD - ERROR
				#### #####################;
			}
	    }

		#####;


	#### #####################:
		//--------------------------------
		//--------------------------------
		//----- SEND MAIL TO COMMAND -----
		//--------------------------------
		//--------------------------------
		## (!#####################(#################))
		{
			//Can't send currently - try again next time
			#####;
		}
		
		//Send MAIL FROM
		#####################('#');
		#####################('#');
		#####################('#');
		#####################('#');
		#####################(' ');
		#####################('#');
		#####################('#');
		#####################(':');
		#####################(' ');
		#####################('<');
		

		//Get the to address
		#####################(&###########[#], ######(###########));

		### (########## = #; ###########[##########] != ####; ##########++)
		{
			#####################(###########[##########]);	
		}

		#####################('>');
		#####################('\#');		//CR
		#####################('\#');		//LF

		#####################(#################);				//Send the packet

		####### = #####################;
		##################### = #####################;

		//UPDATE USER DISPLAYED STRING IF IN USE
		###### #####################
			##################### = #####################;
			##################### = #;
		######

		#####;


	#### #####################:
		//--------------------------------------------
		//--------------------------------------------
		//----- WAITING FOR RESPONSE TO RCPT TO  -----
		//--------------------------------------------
		//--------------------------------------------
		//(Once the response is recevied we could continue sending RCPT TO fields for additional recipients if we wanted to)

		//DO WE NEED TO RE-SEND THE LAST PACKET?
		## (#####################(#################))
		{
			####### = #####################;
			#####;
		}

		//CHECK FOR PACKET RECEIVED
		## (#####################(#################))
		{
			#####################(&##############);
			## (############## == '#')				//A leading 2 character indicates the server is happy (3 used for data transfers, 4 & 5 = error)
			{
				//RESPONSE IS GOOD - SEND DATA COMMAND TO START THE EMAIL TRANSFER
				####################();				//Dump the remainder of the recevied packet

				####### = #####################;
			}
			####
			{
				//RESPONSE IS NOT GOOD - ERROR
				#### #####################;
			}
	    }

		#####;


	#### #####################:
		//-----------------------------
		//-----------------------------
		//----- SEND DATA COMMAND -----
		//-----------------------------
		//-----------------------------
		## (!#####################(#################))
		{
			//Can't send currently - try again next time
			#####;
		}

		//Send the command
		#####################('#');
		#####################('#');
		#####################('#');
		#####################('#');

		#####################('\#');		//CR
		#####################('\#');		//LF

		#####################(#################);				//Send the packet

		####### = #####################;
		##################### = #####################;

		//UPDATE USER DISPLAYED STRING IF IN USE
		###### #####################
			##################### = #####################;
			##################### = #;
		######

		#####;


	#### #####################:
		//------------------------------------------------
		//------------------------------------------------
		//----- WAITING FOR RESPONSE TO DATA REQUEST -----
		//------------------------------------------------
		//------------------------------------------------

		//DO WE NEED TO RE-SEND THE LAST PACKET?
		## (#####################(#################))
		{
			####### = #####################;
			#####;
		}

		//CHECK FOR PACKET RECEIVED
		## (#####################(#################))
		{
			#####################(&##############);
			## (############## == '#')				//A leading 3 character indicates the server is in data transfer mode, 4 & 5 = error)
			{
				//RESPONSE IS GOOD
				####################();				//Dump the remainder of the received packet

				####### = #####################;
			}
			####
			{
				//RESPONSE IS NOT GOOD - ERROR
				#### #####################;
			}
	    }
	    #####;


	#### #####################:
		//---------------------------------
		//---------------------------------
		//----- SEND THE EMAIL HEADER -----
		//---------------------------------
		//---------------------------------
		## (!#####################(#################))
		{
			//Can't send currently - try again next time
			#####;
		}
			
		//SEND THE 'FROM' FIELD ON ITS OWN LINE
		#####################('#');
		#####################('#');
		#####################('#');
		#####################('#');
		#####################(':');
		#####################(' ');
		#####################('<');

		//Get the from address
		#####################(&###########[#], ######(###########));

		### (########## = #; ###########[##########] != ####; ##########++)
		{
			#####################(###########[##########]);	
		}
		#####################('>');
		#####################('\#');		//CR
		#####################('\#');		//LF

		//SEND THE 'TO' FIELD ON ITS OWN LINE
		#####################('#');
		#####################('#');
		#####################(':');
		#####################(' ');
		#####################('<');
		
		//Get to address
		#####################(&###########[#], ######(###########));

		### (########## = #; ###########[##########] != ####; ##########++)
		{
			#####################(###########[##########]);	
		}
		
		#####################('>');
		#####################('\#');		//CR
		#####################('\#');		//LF

		//SEND THE 'SUBJECT' FIELD ON ITS OWN LINE
		#####################('#');
		#####################('#');
		#####################('#');
		#####################('#');
		#####################('#');
		#####################('#');
		#####################('#');
		#####################(':');
		#####################(' ');

		//Get the subject
		#####################(&###########[#], ######(###########));

		### (########## = #; ###########[##########] != ####; ##########++)
		{
			#####################(###########[##########]);	
		}

		#####################('\#');		//CR
		#####################('\#');		//LF

		#####################(#################);				//Send the packet

		####### = #####################;
		##################### = #####################;

		//UPDATE USER DISPLAYED STRING IF IN USE
		###### #####################
			##################### = #####################;
			##################### = #;
		######

		#####;


	#### #####################:
		//--------------------------------------------------------
		//--------------------------------------------------------
		//----- WAIT FOR TCP ACK FOR THE EMAIL HEADER PACKET -----
		//--------------------------------------------------------
		//--------------------------------------------------------

		//DO WE NEED TO RE-SEND THE LAST PACKET?
		## (#####################(#################))
		{
			####### = #####################;
			#####;
		}

		//CHECK FOR TCP ACK RETURNED BY SERVER
		## (#####################(#################))
		{
			####### = #####################;
		}
		#####;


	#### #####################:
		//-------------------------------
		//-------------------------------
		//----- SEND THE EMAIL BODY -----
		//-------------------------------
		//-------------------------------
		//(This state may be called more than once depending on the length of the email)

		//DO WE NEED TO RE-SEND THE LAST PACKET?
		//(TCP requires resending of packets if they are not acknowledged and to avoid requiring a large RAM buffer the application needs to remember the last packet sent on a socket
		//so it can be resent if requried - If your application has ram available to store a copy of the last sent packet then the tcp driver could be modified to use it instead).
		########### = ####;					//Default to this is first byte of a new packet - functions should copy working registers in case a resend is needed
		## (#####################(#################))
		{
			########### = ####;				//Flag that functions should restore the copies of working registers as this packet is going to be a re-send
		}

		//----------------------------------------
		//----------------------------------------
		//----- GET NEXT BLOCK OF EMAIL DATA -----
		//----------------------------------------
		//----------------------------------------
		## (!#####################(#################))
		{
			//Can't send currently - try again next time
			#####;
		}
		
		### (##### = #; ##### < ####; #####++)			//Max 1500 bytes in an IP datagram.  Min IP header is 20 bytes, min TCP header is 20 bytes.  Limit to 1400 bytes to provide some slack
		{
			//CALL SEPERATE FUNCTION TO GET THE NEXT DATA BYTE TO SEND
			###### = #####################(&##############, ###########);
			## (######)
			{
				//Send next byte
				#####################(##############);
				
				## (###### == #)
					#####;								//Moving from email body to file attachment - start a new packet to make life simple if we have to do a resend
			}
			####
			{	
				//NO MORE BYTES TO SEND
				//End of email marker
				#####################('\#');		//CR
				#####################('\#');		//LF
				#####################('.');
				#####################('\#');		//CR
				#####################('\#');		//LF
	
				##################### = #####################;
				####### = #####################;

				//UPDATE USER DISPLAYED STRING IF IN USE
				###### #####################
					##################### = #####################;
					##################### = #;
				######

				#####;
			}
			########### = #;
		}

		#####################(#################);				//Send the packet

		## (####### != #####################)
		{
			##################### = #####################;
			####### = #####################;					//Repeate this state until all of email body is sent (there is no SMTP server acknowledgement until the email body is complete)
		}

		//There is not response in this part of the transfer - just keep sending the data until we're done

		#####;


	#### #####################:
		//----------------------------------------------
		//----------------------------------------------
		//----- WAITING FOR RESPONSE TO EMAIL DATA -----
		//----------------------------------------------
		//----------------------------------------------
		//(A response from the server is only send once all of the email has been sent and the special <CR><LF>.<CR><LF> end of message has been sent)
		//Check for packet received
		## (#####################(#################))
		{
			#####################(&##############);
			## (############## == '#')				//A leading 2 character indicates the server is is happy (3 used for data transfers, 4 & 5 = error)
			{
				####################();				//Dump the remainder of the received packet
				####### = #################;

				//UPDATE USER DISPLAYED STRING IF IN USE
				###### #####################
					##################### = #####################;
					##################### = #;
				######

				#####;
			}
			####
			{
				//RESPONSE IS NOT GOOD - ERROR
				#### #####################;
			}
		}
		#####;


	#### #################:
		//------------------------------------------
		//------------------------------------------
		//----- QUIT CONNECTION TO SMTP SERVER -----
		//------------------------------------------
		//------------------------------------------
		## (!#####################(#################))
		{
			//Can't send currently - try again next time
			#####;
		}
		
		//Send the username
		#####################('#');
		#####################('#');
		#####################('#');
		#####################('#');

		#####################('\#');		//CR
		#####################('\#');		//LF

		#####################(#################);				//Send the packet

		####### = #####################;
		##################### = #####################;

		//UPDATE USER DISPLAYED STRING IF IN USE
		###### #####################
			##################### = #####################;
			##################### = #;
		######

		#####;


	#### #####################:
		//----------------------------------
		//----------------------------------
		//----- WAIT FOR QUIT RESPONSE -----
		//----------------------------------
		//----------------------------------

		//DO WE NEED TO RE-SEND THE LAST PACKET?
		## (#####################(#################))
		{
			####### = #################;
			#####;
		}

		//CHECK FOR PACKET RECEIVED
		## (#####################(#################))
		{
			//----- ALL DONE - DUMP THE TCP CONNECTION AND RETURN TO IDLE MODE -----
			#####################(#################);
			####### = #############;

			//UPDATE USER DISPLAYED STRING IF IN USE
			###### #####################
				##################### = #####################;
				##################### = #;
			######
		}

		#####;


	#### ###############:
		//-------------------------------------
		//-------------------------------------
		//----- SEND EMAIL PROCESS FAILED -----
		//-------------------------------------
		//-------------------------------------

		//UPDATE USER DISPLAYED STRING IF IN USE
		###### #####################
			##################### = #####################;
			##################### = #;
		######
		
		####### = #############;

		#####;
		
	}

	######;

process_send_email_tcp_failed:
//-----------------------------------------------------------------------------------
//----- FAILED DURING TCP TRANSFER - CLOSE CONNECTION AND CHANGE STATE TO ERROR -----
//-----------------------------------------------------------------------------------

	if ((sm_smtp != SM_SMTP_SEND_EMAIL) && (sm_smtp != SM_SMTP_WAIT_FOR_ARP_RESPONSE) && (sm_smtp != SM_SMTP_WAITING_DNS_RESPONSE))
	{
		tcp_dump_rx_packet();							//Dump the remainder of any received packet
		tcp_request_disconnect_socket(smtp_tcp_socket);	//Close connection
	}

	sm_smtp = SM_SMTP_FAILED;

	return;

}






//****************************************************************
//****************************************************************
//********** SEND EMAIL PROVIDE NEXT BYTE OF EMAIL BODY **********
//****************************************************************
//****************************************************************
//Once the connection has been establised to the smtp server and the email header sent this function will be called for each
//byte to be included in the email body.
//The first call will be the start of the line following the 'subject' field of the header.
//Additional header lines may be added if requried (i.e. MIME header if attaching a file to the email)
//To start sending the email body you must first send a blank line to indicate the end of the header and the beginning of the
//email body

//Returns:
//	2 = there are more bytes to send but start a new packet (we ensure a new packet is started before sending a file attachment for simplicity)
//	1 = there are more bytes to send
//	0 = the last byte has already been sent and there is no more data.
//The calling function will add the <CR><LF>.<CR><LF> end of email marker
//
//resend_flags:
//	0x00	Get next byte as normal.
//	0x01	The byte being requested is the first byte of a new TCP packet.  The previous packet was sucessfully sent.
//	0x02 	The byte being requested is the first byte of a re-send of the last packet.
//			This is requried as TCP requires resending of packets if they are not acknowledged and to avoid requiring a large RAM buffer the
//			application needs to remember the last packet sent on a socket so it can be re-sent if requried.
//			(If your application has ram available to store a copy of the last sent packet then the tcp driver could be modified to use it instead).

BYTE send_email_get_next_byte(BYTE *data, BYTE resend_flags)
{


	static BYTE mime_base64_byte[4];
	static BYTE mime_line_character_count;
	static BYTE sm_send_email_data_copy;
	static WORD send_email_body_byte_number_copy;
	static BYTE mime_line_character_count_copy;
	static BYTE mime_base64_byte_copy[4];
	static WORD resend_possible_move_back_bytes;
	static WORD resend_move_back_bytes;
	static BYTE file_attachment_last_byte_sent;
	static BYTE get_data_started_new_tcp_packet;
	BYTE b_temp;
	BYTE bytes_to_encode[3];
	BYTE bytes_to_encode_len;

	
	//TO SEND A BASIC ASCII EMAIL:-
	//Send a blank line "\r\n"
	//Send the email text, using "\r\n" on the end of each line, with a maximum of 1000 charcters permitted per line (RFC821)
	//Finally return with 0x00 so that the SMTP function adds the end of email marker and finishes sending the email

	//TO SEND A MIME EMAIL WITH AN ATTACHMENT:-
	//Send mime_header_for_text_string
	//Send email_body_message_string or any ascii text to be displayed in the email body
	//Send mime_body_for_file_string
	//Send the file encoded in base64 format
	//Send mime_body_end_of_mime_string
	//Finally return with 0x00 so that the SMTP function adds the end of email marker and finishes sending the email


	if (resend_flags == 0x01)
	{
		//-------------------------------------------------------------------------------
		//----- COPY ALL WORKING REGISTERS AS THIS IS THE START OF A NEW TCP PACKET -----
		//-------------------------------------------------------------------------------
		sm_send_email_data_copy = sm_send_email_data;
		send_email_body_byte_number_copy = send_email_body_byte_number;
		mime_line_character_count_copy = mime_line_character_count;
		mime_base64_byte_copy[0] = mime_base64_byte[0];
		#####################[#] = ################[#];
		#####################[#] = ################[#];
		#####################[#] = ################[#];
		############################### = #;
		###################### = #;
		############################### = #;

	}
	#### ## (############# == ####)
	{
		//-------------------------------------------------------------------------------------
		//----- RESTORE ALL WORKING REGISTERS AS THIS IS A RE-SEND OF THE LAST TCP PACKET -----
		//-------------------------------------------------------------------------------------
		#################### = #####################;
		##################### = #####################;
		##################### = #####################;
		##################[#] = #####################[#];
		##################[#] = #####################[#];
		##################[#] = #####################[#];
		##################[#] = #####################[#];
		##################### = #####################;
		##################### = #;
		##################### = #;
		##################### = #;
	}


send_email_get_next_byte_redo:
	send_email_body_byte_number++;

	switch(sm_send_email_data)
	{
	case SM_SEND_EMAIL_DATA_MIME_HEADER:
		//------------------------------------------------------------------------------
		//----- THE MIME HEADER USED JUST FOLLOWING THE STANDARD EMAIL BODY HEADER -----
		//------------------------------------------------------------------------------
		*data = mime_header_for_text_string[(WORD)send_email_body_byte_number];

		if (mime_header_for_text_string[(WORD)send_email_body_byte_number + 1] == 0x00)
		{
			##################### = ######;
			#################### = #####################;

			##################### = #;
		}
		######(#);

	#### #####################:
		//--------------------------------------------------------
		//----- ASCII TEXT TO BE DISPLAYED IN THE EMAIL BODY -----
		//--------------------------------------------------------

		//----- GET NEXT BYTE FROM USER APLICATION -----
		###### = #####################(#, #####################, #####################, ####);
		##################### = #;
		##################### = #;
		## (######)
		{
			//----- WE HAVE THE NEXT BYTE TO SEND -----
			#####################++;
			######(#);
		}
		####
		{
			//----- NO MORE BYTES TO SEND -----
			##################### = ######;

			###### #####################
				## (#####################)
					#################### = #####################;		//User flagged that a file would be included when send email was origianlly called
				####
					#################### = #####################;		//Not using send an attachment
			#####
				#################### = #####################;			//Not using send attachment functionality
			######
			
			#### #####################;
		}


#ifdef EMAIL_ATTACHMENT_FILENAME			//Don't include file attachment code if this functionality is not being used
	case SM_SEND_EMAIL_BODY_MIME_FILE_HEADER_1:
		//--------------------------------------------------------
		//----- START OF MIME HEADER FOR THE FILE ATTACHMENT -----
		//--------------------------------------------------------
		*data = mime_body_file_header_string_1[(WORD)send_email_body_byte_number];

		if (mime_body_file_header_string_1[(WORD)send_email_body_byte_number + 1] == 0x00)
		{
			send_email_body_byte_number = 0xffff;
			mime_line_character_count = 0xff;
			################## = #####################################;
		}
		######(#);


	#### #####################:
		//-----------------------------------------------------------
		//----- FILENAME OF MIME HEADER FOR THE FILE ATTACHMENT -----
		//-----------------------------------------------------------
		//(This includes the file name)
		*#### = #####################[(####)#####################];

		## (#####################[(####)##################### + #] == ####)
		{
			##################### = ######;
			##################### = ####;
			#################### = #####################;
		}
		######(#);


	#### #####################:
		//------------------------------------------------------
		//----- END OF MIME HEADER FOR THE FILE ATTACHMENT -----
		//------------------------------------------------------
		*#### = #####################[(####)#####################];

		## (#####################[(####)##################### + #] == ####)
		{
			##################### = ######;
			##################### = ####;
			#################### = #####################;

			##################### = #;
			##################### = #;
			######(#);
		}
		######(#);


	#### #####################:
		//-----------------------------------
		//----- THE FILE DATA IN BASE64 -----
		//-----------------------------------

		//MAXIMUM PERMITTED LINE LENGTH IS 76 CHARACTERS
		#####################++;
		## (##################### == ##)
		{
			#####################;
			*#### = '\#';
			###### (#);
		}
		#### ## (##################### == ##)
		{
			#####################;
			##################### = ####;
			*#### = '\#';
			###### (#);
		}

		
		## ((##################### & ######) == #)
		{
			//----- GET NEW BLOCK OF 3 BYTES READY TO BE ENCODED -----
			##################### = #;
			##### (##################### < #)
			{
				## (#####################)
					#####;

				//----- GET NEXT FILE BYTE FROM USER APLICATION -----
				###### = #####################(#, #####################, #####################, &#################[#####################]);
				##################### = #;
				##################### = #;
				## (######)
				{
					//------ WE HAVE NEXT BYTE TO SEND -----
					#####################++;
					#####################++;
				}
				####
				{
					//----- THERE ARE NO MORE BYTES TO SEND -----
					##################### = #;
				}
			}

			## (##################### && (##################### == #))
			{
				//----- NO MORE FILE BYTES TO SEND -----
				##################### = ######;
				#################### = #####################;
				#### #####################;
			}

			//----- CONVERT TO 4 BYTES OF BASE 64 -----
			#####################(&#################[#], &##################[#], #####################);
		}


		//----- RETURN WITH THE NEXT ENCODED BYTE -----
		## ((##################### & ######) == #)
		{
			*#### = ##################[#];
			######(#);
		}
		#### ## ((##################### & ######) == #)
		{
			*#### = ##################[#];
			######(#);
		}
		#### ## ((##################### & ######) == #)
		{
			*#### = ##################[#];
			######(#);
		}
		####
		{
			*#### = ##################[#];
			######(#);
		}
#endif //#ifdef EMAIL_ATTACHMENT_FILENAME

	case SM_SEND_EMAIL_BODY_MIME_END_OF_BODY:
		//------------------------------
		//----- END OF MIME MARKER -----
		//------------------------------
		*data = mime_body_end_of_mime_string[send_email_body_byte_number];

		if (mime_body_end_of_mime_string[send_email_body_byte_number + 1]  == 0x00)
		{
			send_email_body_byte_number = 0xffff;
			sm_send_email_data = SM_SEND_EMAIL_BODY_DONE;
		}
		######(#);

	#### #####################:
		//--------------------------------
		//----- NO MORE DATA TO SEND -----
		//--------------------------------
		###### (#);
	
	}

	######(#);
}



//***************************************************************
//***************************************************************
//********** CONVERT 3 BYTES TO 4 BYTE BASE 64 ENCODED **********
//***************************************************************
//***************************************************************
//source = start of 3 byte array containing the 3 bytes to be converted
//dest = start of 4 byte array that the encoded bytes will be written to
void email_convert_3_bytes_to_base64 (BYTE *source, BYTE *dest, BYTE len)
{

	//Base64 uses an efficient 8bit to 7 bit conversion but without the encoded data being human readable.
	//It uses a 65 character subset of US-ASCII, with 6 bits represented per printable character. The 65th
	//character '=' is used as a special marker. The characters are selected so as to be universally representable,
	//and they exclude characters with particular significance to SMTP and MIME (e.g. ".", CR, LF, "-").
	//Each 24 bits (3 bytes) of the data being encoded is represented
	//by 4 characters, resulting in approximately 33% larger size overall.
	//Working from left to right, the next 3 bytes to send are joined together to form 24 bits and this is then split up
	//into 4 groups of 6 bits, with each 6 bit group represented by a character from the base64 alphabet:
	//	0  'A'		16 'Q'		32 'g'		48 'w'		PAD '='
	//	1  'B'		17 'R'		33 'h'		49 'x'
	//	2  'C'		18 'S'		34 'i'		50 'y'
	//	3  'D'		19 'T'		35 'j'		51 'z'
	//	4  'E'		20 'U'		36 'k'		52 '0'
	//	5  'F'		21 'V'		37 'l'		53 '1'
	//	6  'G'		22 'W'		38 'm'		54 '2'
	//	7  'H'		23 'X'		39 'n'		55 '3'
	//	8  'I'		24 'Y'		40 'o'		56 '4'
	//	9  'J'		25 'Z'		41 'p'		57 '5'
	//	10 'K'		26 'a'		42 'q'		58 '6'
	//	11 'L'		27 'b'		43 'r'		59 '7'
	//	12 'M'		28 'c'		44 's'		60 '8'
	//	13 'N'		29 'd'		45 't'		61 '9'
	//	14 'O'		30 'e'		46 'u'		62 '+'
	//	15 'P'		31 'f'		47 'v'		63 '/'
	//Rules:
	//When decoding any character not included in the base64 alphabet must be ignored.
	//The encoded output must have line lengths of no more than 76 characters.
	//If there are less than 24 bits available at the end of the encoded data then zero bits must be added to the right
	//of the final bits to reach the next 6 bit group boundary.  The '=' character is then added to make up any
	//remaining characters of the last 4 character group.  This allows 3 posibilities.  Where there are 24 bits to make
	//up the final 4 character output there will be no '=' padding character.  Where there are only 16 bits there will
	//be 3 characters followed by 1 '=' padding character.  Where there are only 8 bits there will be 2 characters
	//followed by 2 '=' padding characters.

	dest[0] = 0;
	dest[1] = 0;
	dest[2] = 0;
	dest[3] = 0;

	dest[0] |= ((source[0] >> 2) & 0x3f);
	####[#] |= ((######[#] << #) & ####);

	## (### == #)
	{
		//ONLY 1 BYTE TO BE ENCODED
		####[#] = ####;				//Flag to set byte to '=' (for null)
		####[#] = ####;				//Flag to set byte to '=' (for null)
	}
	####
	{
		// > 1 BYTE TO BE ENCODED
		####[#] |= ((######[#] >> #) & ####);
		####[#] |= ((######[#] << #) & ####);

		## (### == #)
		{
			//ONLY 2 BYTES TO BE ENCODED
			####[#] = ####;			//Flag to set byte to '=' (for null)
		}
		####
		{
			//ALL 3 BYTES TO BE ENCODED
			####[#] |= ((######[#] >> #) & ####);
			####[#] |= (######[#] & ####);
		}
	}

	####[#] = #####################(####[#]);
	####[#] = #####################(####[#]);
	####[#] = #####################(####[#]);
	####[#] = #####################(####[#]);
}



//************************************************************
//************************************************************
//********** SEND EMAIL CONVERT CHAR TO BASE64 CHAR **********
//************************************************************
//************************************************************
BYTE email_convert_byte_to_base64_alphabet (BYTE data)
{
	if (data < 26)
		return (data + 'A');
	else if (data < 52)
		return (data + 'a' - 26);
	else if (data < 62)
		###### (#### + '#' # ##);
	#### ## (#### == ##)
		###### ('+');
	#### ## (#### == ##)
		###### ('/');
	####
		###### ('=');
}



//********************************************
//********************************************
//********** RETURN SMTP SERVER URL **********
//********************************************
//********************************************
void email_return_smtp_url (BYTE *string_pointer, BYTE max_length)
{
	BYTE count = 0;
	#ifdef SMTP_USING_CONST_ROM_SETTINGS
	CONSTANT BYTE *p_source_string;				//Using hard coded constant strings
	#else
	BYTE *p_source_string;							//Using variable strings
	#endif

	p_source_string = &SMTP_SERVER_STRING[0];
	do
	{
		//Check for overflow
		## (++##### == ##########)
		{
			*###############++ = ####;
			######;
		}

		//Copy next byte
		*###############++ = *#################;
	}
	##### (#################++ != ####);

}



//******************************************
//******************************************
//********** RETURN SMTP USERNAME **********
//******************************************
//******************************************
//(Only used if authorised login is used)
void email_return_smtp_username (BYTE *string_pointer, BYTE max_length)
{
	BYTE count = 0;
	#ifdef SMTP_USING_CONST_ROM_SETTINGS
	CONSTANT BYTE *p_source_string;				//Using hard coded constant strings
	#else
	BYTE *p_source_string;							//Using variable strings
	#endif

	p_source_string = &SMTP_USERNAME_STRING[0];
	do
	{
		//Check for overflow
		## (++##### == ##########)
		{
			*###############++ = ####;
			######;
		}

		//Copy next byte
		*###############++ = *#################;
	}
	##### (#################++ != ####);
	
}



//******************************************
//******************************************
//********** RETURN SMTP PASSWORD **********
//******************************************
//******************************************
//(Only used if authorised login is used)
void email_return_smtp_password (BYTE *string_pointer, BYTE max_length)
{
	BYTE count = 0;
	#ifdef SMTP_USING_CONST_ROM_SETTINGS
	CONSTANT BYTE *p_source_string;				//Using hard coded constant strings
	#else
	BYTE *p_source_string;							//Using variable strings
	#endif

	p_source_string = &SMTP_PASSWORD_STRING[0];
	do
	{
		//Check for overflow
		## (++##### == ##########)
		{
			*###############++ = ####;
			######;
		}

		//Copy next byte
		*###############++ = *#################;
	}
	##### (#################++ != ####);
	
}



//********************************************
//********************************************
//********** RETURN SMTP TO ADDRESS **********
//********************************************
//********************************************
//(The email to be used as the destination address)
void email_return_smtp_to (BYTE *string_pointer, BYTE max_length)
{
	BYTE count = 0;
	#ifdef SMTP_USING_CONST_ROM_SETTINGS
	CONSTANT BYTE *p_source_string;				//Using hard coded constant strings
	#else
	BYTE *p_source_string;							//Using variable strings
	#endif

	p_source_string = &SMTP_TO_STRING[0];
	do
	{
		//Check for overflow
		## (++##### == ##########)
		{
			*###############++ = ####;
			######;
		}

		//Copy next byte
		*###############++ = *#################;
	}
	##### (#################++ != ####);
	
}



//**********************************************
//**********************************************
//********** RETURN SMTP FROM ADDRESS **********
//**********************************************
//**********************************************
//(The email to be used as the from address)
void email_return_smtp_sender (BYTE *string_pointer, BYTE max_length)
{
	BYTE count = 0;
	#ifdef SMTP_USING_CONST_ROM_SETTINGS
	CONSTANT BYTE *p_source_string;				//Using hard coded constant strings
	#else
	BYTE *p_source_string;							//Using variable strings
	#endif

	p_source_string = &SMTP_SENDER_STRING[0];
	do
	{
		//Check for overflow
		## (++##### == ##########)
		{
			*###############++ = ####;
			######;
		}

		//Copy next byte
		*###############++ = *#################;
	}
	##### (#################++ != ####);
	
}



//*****************************************
//*****************************************
//********** RETURN SMTP SUBJECT **********
//*****************************************
//*****************************************
void email_return_smtp_subject (BYTE *string_pointer, BYTE max_length)
{
	BYTE count = 0;
	#ifdef SMTP_USING_CONST_ROM_SETTINGS
	CONSTANT BYTE *p_source_string;
	#else
	BYTE *p_source_string;
	#endif

	p_source_string = &SMTP_SUBJECT_STRING[0];
	do
	{
		//Check for overflow
		## (++##### == ##########)
		{
			*###############++ = ####;
			######;
		}

		//Copy next byte
		*###############++ = *#################;
	}
	##### (#################++ != ####);
	
}



