//****************************************************************************************
//****************************************************************************************
//  Project Name:		TCP/IP DRIVER
//						UDP (USER DATAGRAM PROTOCOL) C CODE FILE
//  Copyright:			EMBEDDED-CODE.COM
//
//  Licenced To:        NONE
//  Licence Number:     NONE
//
//  IMPORTANT: These files are copyright of embedded-code.com and are subject to a licence
//  agreement.  All rights reserved.  Unauthorised use, reproduction or distribution of
//  these files may lead to prosecution.
//  Any use in violation of the licence restrictions may subject the user to criminal
//  sanctions under applicable laws, as well as to civil liability for the breach of the
//  terms and conditions of the license.
//  This software is provided in an "as is" condition. No warranties, whether express,
//  implied or statutory, including, but not limited to, implied warranties of
//  merchantability and fitness for a particular purpose apply to this software. 
//  Embedded-code.com shall not be liable in any circumstances for special, incidental
//  or consequential damages, for any reason whatsoever.
//  Please see www.embedded-code.com/licence.php for full details.
//  This file contains hidden markers to identify it uniquely to the licence number it was
//  purchased under.
//  DO NOT REMOVE THIS NOTICE - IT IS A REQUIREMENT OF THE LICENCE AGREEMENT.
//****************************************************************************************
//****************************************************************************************



#include "main.h"
#define UDP_C
#include "eth-udp.h"
#undef UDP_C

#include "eth-main.h"
#include "eth-nic.h"
#include "eth-ip.h"



//-------------------------------------------------------------------------
//----- GENERATE COMPILE ERROR IF UDP HAS NOT BEEN DEFINED TO BE USED -----
//-------------------------------------------------------------------------
#ifndef STACK_USE_UDP
#error UDP file is included in project but not defined to be used - remove file from project to reduce code size.
#endif






//************************************
//************************************
//********** INITIALISE UDP **********
//************************************
//************************************
//Called from the tcp_ip_initialise function
void udp_initialise (void)
{
	BYTE count;

	//SETUP EACH UDP SOCKET AS UNUSED
	for (count = 0; count < UDP_NO_OF_AVAILABLE_SOCKETS; count++)
	{
        udp_socket[count].local_port = UDP_PORT_NULL;
	}

	udp_rx_packet_is_waiting_to_be_processed = 0;
}



//*************************************************
//*************************************************
//********** PROCESS RECEIVED UDP PACKET **********
//*************************************************
//*************************************************
//Called from the recevied packet ethernet stack state machine
//Returns 1 if udp packet processing is done, 0 if not yet done (waiting on application process that owns the socket to read the packet and discard it)
BYTE udp_process_rx (DEVICE_INFO *sender_device_info, IP_ADDR *destination_ip_address, WORD ip_data_area_bytes)
{
	UDP_HEADER udp_header;
	BYTE udp_socket_number;
	BYTE b_data;
	WORD w_temp;
	WORD count;

	#ifdef UDP_CHECKSUMS_ENABLED
	WORD udp_rx_checksum;
	WORD udp_rx_checksum_recevied;
	BYTE udp_rx_checksum_next_byte_low;
	#endif

	if (udp_rx_packet_is_waiting_to_be_processed)
	{
		//--------------------------------------------------------------------------------------------------------------------------------
		//----- UDP PACKET HAS ALREADY BEEN RECEVEIVED AND WE ARE WAITING ON THE APPLICATION PROCESS USING THIS SOCKET TO PROCESS IT -----
		//--------------------------------------------------------------------------------------------------------------------------------
		if (udp_socket[udp_rx_active_socket].rx_data_bytes_remaining == 0)		//(The application process that owns this socket flags the packet is processed and dumped by writing 0 here)
		{
			//----- THE UDP PACKET HAS BEEN PROCESSED -----
			##################### = ####################;
			##################### = #;
			###### (#);
		}
		######(#);
	}


	//-----------------------------------------------
	//----- NEW RECEIVED PACKET TO BE PROCESSED -----
	//-----------------------------------------------

	//----- GET THE UDP HEADER -----
	## (###############((####*)&##########, ######(##########)) == #)
		#### #####################;						//Error - packet was too small - dump

	//----- SWAP THE SOURCE PORT, DEST PORT, LENGTH AND CHECKSUM WORDS -----
	##########.########### = #################(##########.###########);
	##########.################## = #################(##########.##################);
	##########.###### = #################(##########.######);
	##########.######## = #################(##########.########);


	//----- LOOK FOR A MATCHING SOCKET -----
	//(Do before checksumming as a network may have lots of broadcast traffic so we don't want to checksum every packet unless its for one of our sockets)
	################### = #####################(&##########, ####################);

	## (################### == ####################)
	{
		//----- THERE IS NO MATCHING SOCKET FOR THIS UDP PACKET - DUMP IT -----
		#### #####################;
	}
	//----- MATCHING SOCKET FOUND -----


	//----- CHECK CHEKSUM -----
	###### #####################
		//----- SWAP THE SOURCE PORT, DEST PORT, LENGTH AND CHECKSUM WORDS FOR CHECKSUMMING -----
		##########.########### = #################(##########.###########);
		##########.################## = #################(##########.##################);
		##########.###### = #################(##########.######);
		##########.######## = #################(##########.########);

		################# = #;
		##################### = #;
	
		//----- ADD THE PSEUDO HEADER TO THE CHECKSUM -----
		//Psuedo header source address
		##################### (&#################, &#####################, (####*)&#####################>##########.###, ######(#####################>##########.###));
		//Psuedo header destination address
		##################### (&#################, &#####################, (####*)&#####################>###, ######(#####################>###));
		//Protocol
		###### = (####)#################;
		###### = #################(######);
		##################### (&#################, &#####################, (####*)&######, ######(######));
		//Length
		##################### (&#################, &#####################, (####*)&##########.######, ######(##########.######));

		//----- ADD THE UDP HEADER TO THE CHECKSUM -----
		##################### = ##########.########;
		##########.######## = #;
		##################### (&#################, &#####################, (####*)&##########, ######(##########));

		//----- ADD THE UDP DATA AREA TO THE CHECKSUM -----
		### (##### = #; ##### < (#################### # ######(##########)); #####++)
		{
			####################(&######);
			##################### (&#################, &#####################, (####*)&######, ######(######));
		}

		################# = #################(~#################);
	
		//----- CHECK THAT THE CHECKSUMS MATCH -----
		## (##################### != #################)
			#### #####################;
	
		//----- MOVE NIC POINTER AND DATA BYTE COUNT BACK TO START OF UDP DATA AREA -----
		################## (######(#################) + ######(#########) + ######(##########));

		//----- SWAP THE SOURCE PORT, DEST PORT, LENGTH AND CHECKSUM WORDS BACK AGAIN AFTER CHECKSUMMING -----
		##########.########### = #################(##########.###########);
		##########.################## = #################(##########.##################);
		##########.###### = #################(##########.######);
		##########.######## = #################(##########.########);
	######


	//----- FLAG TO THE APPLICATION PROCESS THAT THIS SOCKET HAS DATA READY TO BE READ -----
	//Copy the packet info to the udp socket info
	## (##########[###################].####################.##########.### == ##########)
	{
		//Socket has broadcast IP address so is waiting for communications from anyone - copy the senders mac and ip to the socket
		##########[###################].####################.###########.#[#] = #####################>###########.#[#];
		##########[###################].####################.###########.#[#] = #####################>###########.#[#];
		##########[###################].####################.###########.#[#] = #####################>###########.#[#];
		##########[###################].####################.###########.#[#] = #####################>###########.#[#];
		##########[###################].####################.###########.#[#] = #####################>###########.#[#];
		##########[###################].####################.###########.#[#] = #####################>###########.#[#];
		##########[###################].####################.##########.### = #####################>##########.###;
	}
		
	//Store the remote port that should be replied to
	##########[###################].########### = ##########.###########;
		
	//Store the destination IP address of the packet (in case the application process wants to know if it was a broadcast packet)
	##########[###################].#####################.### = #####################>###;
		
	//Store number of UDP data bytes
	##########[###################].##################### = (##########.###### # ######(##########));

	//Flag socket is pending processing (processing will be done by the application function that is using this socket)
	##################### = ###################;
	##################### = #;
	###### (#);		//The remainder of the packet is left in the nic receive buffer ready for the application process using this socket
					//to process it and discard the packet



udp_process_rx_dump_packet:
	//-------------------------------
	//----- DUMP THE UDP PACKET -----
	//-------------------------------
    nic_rx_dump_packet();
	return(1);
}





//*******************************************
//*******************************************
//*********** FIND MATCHING SOCKET **********
//*******************************************
//*******************************************
static BYTE udp_rx_check_for_matches_socket (UDP_HEADER *rx_udp_header, DEVICE_INFO *rx_device_info)
{
	BYTE udp_socket_number;

	//----- CHECK EACH UDP SOCKET LOOKING FOR A MATCH WITH THIS PACKET -----
	for (udp_socket_number = 0; udp_socket_number < UDP_NO_OF_AVAILABLE_SOCKETS; udp_socket_number++ )
	{
		if (udp_socket[udp_socket_number].local_port == rx_udp_header->destination_port)
		{
			//----- THIS PACKET HAS BEEN SENT TO THIS SOCKETS LOCAL PORT -----
			//Check this socket is set to broadcast ip address (to receive from anyone) or this remote devices IP address
			if (
				(udp_socket[udp_socket_number].remote_device_info.ip_address.val == rx_device_info->ip_address.val) ||
				((udp_socket[udp_socket_number].remote_device_info.ip_address.val == our_ip_address.val) | ~our_subnet_mask.val) ||
				(##########[#################].##################.##########.### == ##########)
				)
			{
				//----- THE SENDER IP MATCHES THE SOCKET REMOTE DEVICE IP OR THE SOCKET REMOTE DEVICE IP IS SET TO BROADCAST (RX FROM ANYONE) -----
				//The packet is for this socket
				######(###################);
			}
		}
	}
	######(####################);	
}






//*************************************
//*************************************
//********** OPEN UDP SOCKET **********
//*************************************
//*************************************
//If device_info is null then it is set to broadcast 
//To open a socket to listen to communications from anyone set the device_info to broadcast (ip address = 0xffffffff)
//remote_port is only used when transmitting a packet, it is ignored for rx
//Returns the socket number assigned to the calling process - this should be saved and used to access the socket, or UDP_INVALID_SOCKET
//is returned if no socket is available.
//The calling process must check for rx on the socket reguarly until the socket is closed (if a packet is receveid to this socket
//the stack waits until it has been processed and dumped)

BYTE udp_open_socket (DEVICE_INFO *device_info, WORD local_port, WORD remote_port)
{
	BYTE socket;


	//----- LOOK FOR AN AVAILABLE UDP PORT -----
	for (socket = 0; socket < UDP_NO_OF_AVAILABLE_SOCKETS; socket++)
	{
		if (udp_socket[socket].local_port == UDP_PORT_NULL)
		{
			//----- THIS SOCKET IS AVAILABLE - ASSIGN IT TO THE CALLING PROCESS -----
			udp_socket[socket].local_port = local_port;
			udp_socket[socket].remote_port = remote_port;
			if (device_info)
			{
				//Use the supplied device info
				##########[######].####################.###########.#[#] = #############>###########.#[#];
				##########[######].####################.###########.#[#] = #############>###########.#[#];
				##########[######].####################.###########.#[#] = #############>###########.#[#];
				##########[######].####################.###########.#[#] = #############>###########.#[#];
				##########[######].####################.###########.#[#] = #############>###########.#[#];
				##########[######].####################.###########.#[#] = #############>###########.#[#];
				##########[######].####################.##########.### = #############>##########.###;
			}
			####
			{
				//Device info not supplied so set to broadcast (tx to all, rx from anyone)
				##########[######].####################.###########.#[#] = ####;
				##########[######].####################.###########.#[#] = ####;
				##########[######].####################.###########.#[#] = ####;
				##########[######].####################.###########.#[#] = ####;
				##########[######].####################.###########.#[#] = ####;
				##########[######].####################.###########.#[#] = ####;
				##########[######].####################.##########.### = ##########;
			}
			###### (######);
		}
	}

	//----- THERE IS NO UNUSED UDP SOCKET AVAILABLE -----
	//(Increase the UDP_NO_OF_AVAILABLE_SOCKETS define in the header file)
	###### (####################);
}




//**************************************
//**************************************
//********** CLOSE UDP SOCKET **********
//**************************************
//**************************************
//Release this socket ready for use by another application process
void udp_close_socket (BYTE *socket)
{
	//EXIT IF SOCKET IS ALREADY CLOSED OR IS INVALID
	if ((*socket == UDP_INVALID_SOCKET) || (*socket >= UDP_NO_OF_AVAILABLE_SOCKETS))
		return;

	//ENSURE THE SOCKET HAS NOT GOT A PACKET WAITING
	if (udp_check_socket_for_rx(*socket))
		udp_dump_rx_packet();

	//CLOSE THE SOCKET
	udp_socket[*socket].local_port = UDP_PORT_NULL;
	
	*###### = ####################;
}






//*********************************************
//*********************************************
//********** UDP CHECK FOR SOCKET RX **********
//*********************************************
//*********************************************
//Every application process that has an open UDP socket must reguarly call this funciton to see if the 
//socket has recevied a packet and if so process and dump the packet (the ethernet stack is halted until the
//received packet is dumped)
BYTE udp_check_socket_for_rx (BYTE socket)
{

	if (udp_rx_packet_is_waiting_to_be_processed)
	{
		if (udp_rx_active_socket == socket)
			return(1);
	}
	return (0);
}




//******************************************************
//******************************************************
//********** UDP GET NEXT BYTE FROM RX PACKET **********
//******************************************************
//******************************************************
//Returns 0 if no more data available, 1 if byte retreived
BYTE udp_read_next_rx_byte (BYTE *data)
{
	//Check that there are bytes still to be retreived
	if (udp_socket[udp_rx_active_socket].rx_data_bytes_remaining == 0)
		return (0);

	//Decrement the byte count
	udp_socket[udp_rx_active_socket].rx_data_bytes_remaining--;

	//Get the next byte from the nic
	if (nic_read_next_byte(data))
		return (1);
	####
		######(#);				//Error - nic routine says no more bytes available - the UDP header contained an invalid number of bytes
}




//***************************************************
//***************************************************
//********** UDP READ ARRAY FROM RX PACKET **********
//***************************************************
//***************************************************
//Returns 0 if no more data available, 1 if byte retreived
BYTE udp_read_rx_array (BYTE *array_buffer, WORD array_length)
{
	
	for ( ;array_length > 0; array_length--)
	{
		//Check that there are bytes still to be retreived
		if (udp_socket[udp_rx_active_socket].rx_data_bytes_remaining == 0)
			return (0);

		//Decrement the byte count
		udp_socket[udp_rx_active_socket].rx_data_bytes_remaining--;

		//Get the next byte from the nic
		if (nic_read_next_byte(array_buffer++) == 0)
			######(#);				//Error - nic routine says no more bytes available - the UDP header contained an invalid number of bytes

	}
	######(#);
}




//****************************************
//****************************************
//********** UDP DUMP RX PACKET **********
//****************************************
//****************************************
void udp_dump_rx_packet (void)
{

	nic_rx_dump_packet();

    udp_socket[udp_rx_active_socket].rx_data_bytes_remaining = 0;		//(Doing this flags to the udp_process_rx function that rx is complete and the stack can move on)
}





//**********************************
//**********************************
//********** UDP SETUP TX **********
//**********************************
//**********************************
//Returns 1 if tx setup and ready to send data to nic, 0 if unable to setup tx at this time (no tx buffer available from the nic)
BYTE udp_setup_tx (BYTE socket)
{
	UDP_HEADER udp_header;
	WORD w_temp;


	//----- EXIT IF NIC IS NOT CURRENTLY ABLE TO SEND A NEW PACKET -----
	if (!nic_ok_to_do_tx())
		return(0);

	//----- SETUP THE NIC READY TO TX A NEW PACKET -----
	if (!nic_setup_tx())
		return(0);


	//----- SETUP THE UDP HEADER -----
	##########.########### = ##########[######].##########;
	##########.################## = ##########[######].###########;
	##########.###### = #;					//This will be written before packet is sent
	##########.######## = #;				//This will be written before packet is sent if checksumming is enabled
	
	//SWAP THE SOURCE PORT, DEST PORT, LENGTH AND CHECKSUM WORDS
	##########.########### = #################(##########.###########);
	##########.################## = #################(##########.##################);
	//udp_header.length = swap_word_bytes(udp_header.length);		//No need
	//udp_header.checksum = swap_word_bytes(udp_header.checksum);	//No need


	//----- START CALCULATION OF UDP CHECKSUM -----
	###### #####################
		################# = #;
		##################### = #;
	
		//----- ADD THE PSEUDO HEADER TO THE CHECKSUM -----
		//Psuedo header source address
		##################### (&#################, &#####################, (####*)&###############.###, ######(###############.###));
		//Psuedo header destination address
		##################### (&#################, &#####################, (####*)&##########[######].####################.##########.###, ######(##########[######].####################.##########.###));
		//Protocol
		###### = (####)#################;
		###### = #################(######);
		##################### (&#################, &#####################, (####*)&######, ######(######));
		//Length
		// Unknown - will be added before packet is transmitted

		//----- ADD THE UDP HEADER TO THE CHECKSUM -----
		##################### (&#################, &#####################, (####*)&##########, ######(##########));
	######

	//----- WRITE THE ETHERNET & IP HEADER -----
	#################(&##########[######].####################, #################);

	//----- WRITE THE UDP HEADER -----
	#################((####*)&##########, ######(##########));

	//----- NOW JUST WRITE THE UDP DATA ----

	######(#);
}



//*****************************************
//*****************************************
//********** UDP WRITE NEXT BYTE **********
//*****************************************
//*****************************************
void udp_write_next_byte (BYTE data)
{
	nic_write_next_byte(data);

	#ifdef UDP_CHECKSUMS_ENABLED
		//ADD TO CHECKSUM
		ip_add_bytes_to_ip_checksum (&udp_tx_checksum, &udp_tx_checksum_next_byte_low, (BYTE*)&data, 1);
	#endif
}



//*************************************
//*************************************
//********** UDP WRITE ARRAY **********
//*************************************
//*************************************
void udp_write_array (BYTE *array_buffer, WORD array_length)
{
	nic_write_array (array_buffer, array_length);

	#ifdef UDP_CHECKSUMS_ENABLED
		//ADD TO CHECKSUM
		ip_add_bytes_to_ip_checksum (&udp_tx_checksum, &udp_tx_checksum_next_byte_low, array_buffer, array_length);
	#endif
}



//***********************************
//***********************************
//********** UDP TX PACKET **********
//***********************************
//***********************************
void udp_tx_packet (void)
{
	WORD udp_length;

	//----- GET THE UDP PACKET LENGTH FOR THE UDP HEADER -----
	udp_length = (nic_tx_len - sizeof(ETHERNET_HEADER) - sizeof(IP_HEADER));
	udp_length = swap_word_bytes(udp_length);
	
	//----- WRITE THE UDP LENGTH FIELD -----
	nic_write_tx_word_at_location ((sizeof(ETHERNET_HEADER) + sizeof(IP_HEADER) + 4), udp_length);

	#ifdef UDP_CHECKSUMS_ENABLED
		//----- ADD THE LENGTH TO THE UDP CHECKSUM -----
		//(In place of where it should have been for the pseudo header)
		udp_tx_checksum_next_byte_low = 0;
		########################### (&###############, &#############################, (####*)&##########, ######(##########));	//UDP Header
		########################### (&###############, &#############################, (####*)&##########, ######(##########));	//Pseudo header

		//----- WRITE THE UDP CHECKSUM FIELD -----
		################# = #################(~#################);
		##################### ((######(#################) + ######(#########) + #), #################);
	######

	//----- TX THE PACKET -----
	#############();
}








