//**************************************************
// ***** HEADER FILE : FAT.h ******
//**************************************************
#ifndef _FAT32_H_
#define _FAT32_H_

//#define FAT_DEBUG

//Structure to access Master Boot Record for getting info about partioions
struct mbrinfo_t
{
	unsigned char     nothing[446];      //ignore, placed here to fill the gap in the structure
	unsigned char     partitionData[64]; //partition records (16x4)
	unsigned int      signature;         //0xaa55
};

//Structure to access info of the first partioion of the disk
struct partitioninfo_t
{                      
	unsigned char     status;                 //0x80 - active partition
	unsigned char     headStart;              //starting head
	unsigned int      cylSectStart;           //starting cylinder and sector
	unsigned char     type;                   //partition type
	unsigned char     headEnd;                //ending head of the partition
	unsigned int      cylSectEnd;             //ending cylinder and sector
	unsigned long     firstSector;            //total sectors between MBR & the first sector of the partition
	unsigned long     totalSectors;           //size of this partition in sectors
};

//Structure to access boot sector data
struct bootsector_t
{
	unsigned char jumpBoot[3];      //default: 0x009000EB
	unsigned char OEMName[8];
	unsigned int  bytesPerSector;   //deafault: 512
	unsigned char sectorPerCluster;
	unsigned int  reservedSectorCount;
	unsigned char numberofFATs;
	unsigned int  rootEntryCount;
	unsigned int  totalSectors_F16; //must be 0 for FAT32
	unsigned char mediaType;
	unsigned int  FATsize_F16;      //must be 0 for FAT32
	unsigned int  sectorsPerTrack;
	unsigned int  numberofHeads;
	unsigned long hiddenSectors;
	unsigned long totalSectors_F32;
	unsigned long FATsize_F32;      //count of sectors occupied by one FAT
	unsigned int  extFlags;
	unsigned int  FSversion;        //0x0000 (defines version 0.0)
	unsigned long rootCluster;      //first cluster of root directory (=2)
	unsigned int  FSinfo;           //sector number of FSinfo structure (=1)
	unsigned int  BackupBootSector;
	unsigned char reserved[12];
	unsigned char driveNumber;
	unsigned char reserved1;
	unsigned char bootSignature;
	unsigned long volumeID;
	unsigned char volumeLabel[11];   //"NO NAME "
	unsigned char fileSystemType[8]; //"FAT32"
	unsigned char bootData[420];
	unsigned int  bootEndSignature;  //0xaa55
};


//Structure to access FSinfo sector data
struct fsinfo_t
{
	unsigned long leadSignature;      //0x41615252
	unsigned char reserved1[480];
	unsigned long structureSignature; //0x61417272
	unsigned long freeClusterCount;   //initial: 0xffffffff
	unsigned long nextFreeCluster;    //initial: 0xffffffff
	unsigned char reserved2[12];
	unsigned long trailSignature;     //0xaa550000
};

//Structure to access Directory Entry in the FAT
struct direntry_t
{
	unsigned char name[11];
	unsigned char attrib;         //file attributes
	unsigned char NTreserved;     //always 0
	unsigned char timeTenth;      //tenths of seconds, set to 0 here
	unsigned int  createTime;     //time file was created
	unsigned int  createDate;     //date file was created
	unsigned int  lastAccessDate;
	unsigned int  firstClusterHI; //higher word of the first cluster number
	unsigned int  writeTime;      //time of last write
	unsigned int  writeDate;      //date of last write
	unsigned int  firstClusterLO; //lower word of the first cluster number
	unsigned long fileSize;       //size of file in bytes
};

struct ldirentry_t
{
	unsigned char	ord;
	unsigned char	name1[10];
	unsigned char	attrib;
	unsigned char	type;
	unsigned char	chksum;
	unsigned char	name2[12];
	unsigned int	firstClusterLO;
	unsigned char	name3[4];
};

//Attribute definitions for file/directory
#define ATTR_READ_ONLY		0x01
#define ATTR_HIDDEN			0x02
#define ATTR_SYSTEM			0x04
#define ATTR_VOLUME_ID		0x08
#define ATTR_DIRECTORY		0x10
#define ATTR_ARCHIVE		0x20
#define ATTR_LONG_NAME		0x0f


#define DIR_ENTRY_SIZE		0x32

#define FAT32_EOC_MASK		0x0FFFFFFF
#define FAT_BAT_CLUSTER		0x0FFFFFF7


//************* variables *************
extern unsigned char buffer[512];

//************* functions *************
unsigned char FS_init(void);
unsigned long FS_firstSector(unsigned long cluster);
unsigned long FS_nextFreeCluster(unsigned long startCluster);
unsigned long FS_fatentry(unsigned long cluster, unsigned long *newval);
struct direntry_t *FS_direntry(const char *filename, unsigned long *sector);

unsigned char FS_changedir(const char *dirname);
unsigned char FS_deleteFile(const char *filename);
struct direntry_t *FS_createFile(const char *filename, unsigned char flags);

#endif

 

